@@ -1,8 +1,8 @@
-# Copyright (C) 2005-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2005-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,44 +15,105 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Handles branch-specific operations."""
 
-from bzrlib import ui, urlutils
-from bzrlib.branch import Branch, BranchFormat, BranchCheckResult, PullResult
-from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import (NoSuchFile, DivergedBranches, NoSuchRevision, 
-                           NotBranchError, UnstackableBranchFormat)
-from bzrlib.revision import is_null, ensure_null
-from bzrlib.workingtree import WorkingTree
-
-from bzrlib.plugins.svn import core, wc
-from bzrlib.plugins.svn.commit import push, push_ancestors
-from bzrlib.plugins.svn.config import BranchConfig
-from bzrlib.plugins.svn.core import SubversionException
-from bzrlib.plugins.svn.errors import NotSvnBranchPath, ERR_FS_NO_SUCH_REVISION
-from bzrlib.plugins.svn.format import get_rich_root_format
-from bzrlib.plugins.svn.repository import SvnRepository
-from bzrlib.plugins.svn.tags import SubversionTags
-from bzrlib.plugins.svn.transport import bzr_to_svn_url
-
 import os
+from subvertpy import (
+    ERR_FS_NO_SUCH_REVISION,
+    NODE_DIR,
+    SubversionException,
+    wc,
+    )
 
-class FakeControlFiles(object):
-    """Dummy implementation of ControlFiles.
-    
-    This is required as some code relies on controlfiles being 
-    available."""
-    def get_utf8(self, name):
-        raise NoSuchFile(name)
+from bzrlib import (
+    tag,
+    ui,
+    urlutils,
+    )
+from bzrlib.branch import (
+    Branch,
+    BranchFormat,
+    BranchCheckResult,
+    InterBranch,
+    PullResult,
+    )
+from bzrlib.bzrdir import (
+    BzrDir,
+    )
+from bzrlib.decorators import (
+    needs_write_lock,
+    )
+from bzrlib.errors import (
+    DivergedBranches,
+    NoSuchRevision,
+    NotBranchError,
+    UnstackableBranchFormat,
+    )
+from bzrlib.revision import (
+    NULL_REVISION,
+    is_null,
+    ensure_null,
+    )
+from bzrlib.trace import (
+    is_quiet,
+    mutter,
+    )
+from bzrlib.workingtree import (
+    WorkingTree,
+    )
 
-    def get(self, name):
-        raise NoSuchFile(name)
+from bzrlib.plugins.svn import (
+    util,
+    )
+from bzrlib.plugins.svn.push import (
+    InterToSvnRepository,
+    create_branch_with_hidden_commit,
+    dpush,
+    push,
+    )
+from bzrlib.plugins.svn.config import (
+    BranchConfig,
+    )
+from bzrlib.plugins.svn.errors import (
+    NotSvnBranchPath,
+    )
+from bzrlib.plugins.svn.foreign import (
+    ForeignBranch,
+    FakeControlFiles,
+    )
+from bzrlib.plugins.svn.format import (
+    get_rich_root_format,
+    )
+from bzrlib.plugins.svn.repository import (
+    SvnRepository,
+    )
+from bzrlib.plugins.svn.tags import (
+    SubversionTags,
+    )
+from bzrlib.plugins.svn.transport import (
+    bzr_to_svn_url,
+    )
 
-    def break_lock(self):
-        pass
+class SubversionBranchCheckResult(BranchCheckResult):
+    """Result of checking a Subversion branch."""
 
 
-class SvnBranch(Branch):
+class SubversionPullResult(PullResult):
+    """Subversion pull result. """
+
+    def report(self, to_file):
+        if not is_quiet():
+            if self.old_revid == self.new_revid:
+                to_file.write('No revisions to pull.\n')
+            else:
+                to_file.write('Now on revision %d (svn revno: %d).\n' % 
+                        (self.new_revno, self.new_revmeta.revnum))
+        self._show_tag_conficts(to_file)
+
+
+class SvnBranch(ForeignBranch):
     """Maps to a Branch in a Subversion repository """
-    def __init__(self, repository, branch_path, _skip_check=False):
+
+    def __init__(self, repository, branch_path, revnum=None, _skip_check=False,
+                 mapping=None):
         """Instantiate a new SvnBranch.
 
         :param repos: SvnRepository this branch is part of.
@@ -62,36 +123,42 @@ class SvnBranch(Branch):
             look at; none for latest.
         """
         self.repository = repository
-        super(SvnBranch, self).__init__()
-        assert isinstance(self.repository, SvnRepository)
-        self.control_files = FakeControlFiles()
         self._format = SvnBranchFormat()
+        self.layout = self.repository.get_layout()
+        assert isinstance(self.repository, SvnRepository)
+        super(SvnBranch, self).__init__(mapping or self.repository.get_mapping())
+        self.control_files = FakeControlFiles()
         self._lock_mode = None
         self._lock_count = 0
-        self.mapping = self.repository.get_mapping()
-        self.layout = self.repository.get_layout()
         self._branch_path = branch_path.strip("/")
         self.base = urlutils.join(self.repository.base, 
                         self._branch_path).rstrip("/")
         self._revmeta_cache = None
+        self._revnum = revnum
         assert isinstance(self._branch_path, str)
         if not _skip_check:
             try:
-                revnum = self.get_revnum()
-                if self.repository.transport.check_path(self._branch_path, 
-                    revnum) != core.NODE_DIR:
+                if self.check_path() != NODE_DIR:
                     raise NotBranchError(self.base)
             except SubversionException, (_, num):
                 if num == ERR_FS_NO_SUCH_REVISION:
                     raise NotBranchError(self.base)
                 raise
-        (type, self.project, _, ip) = self.layout.parse(branch_path)
-        # FIXME: Don't allow tag here
+        try:
+            (type, self.project, _, ip) = self.layout.parse(branch_path)
+        except NotSvnBranchPath:
+            raise NotBranchError(branch_path)
         if type not in ('branch', 'tag') or ip != '':
-            raise NotSvnBranchPath(branch_path, mapping=self.mapping)
+            raise NotBranchError(branch_path)
 
-    def _make_tags(self):
-        return SubversionTags(self)
+    def check_path(self):
+        return self.repository.transport.check_path(self._branch_path, self._revnum or self.repository.get_latest_revnum())
+
+    def supports_tags(self):
+        """See Branch.supports_tags()."""
+        return (self._format.supports_tags() and 
+                self.mapping.supports_tags() and
+                self.layout.supports_tags())
 
     def set_branch_path(self, branch_path):
         """Change the branch path for this branch.
@@ -101,8 +168,7 @@ class SvnBranch(Branch):
         self._branch_path = branch_path.strip("/")
 
     def _get_append_revisions_only(self):
-        value = self.get_config().get_user_option('append_revisions_only')
-        return value == 'True'
+        return self.get_config().get_append_revisions_only()
 
     def unprefix(self, relpath):
         """Remove the branch path from a relpath.
@@ -121,12 +187,13 @@ class SvnBranch(Branch):
         if revnum is None:
             return self._branch_path
 
-        if revnum == self.get_revnum():
-            return self._branch_path
+        last_revmeta, _ = self.last_revmeta()
+        if revnum == last_revmeta.revnum:
+            return last_revmeta.branch_path
 
         # Use revnum - this branch may have been moved in the past 
         return self.repository.transport.get_locations(
-                    self._branch_path, self.get_revnum(), 
+                    last_revmeta.branch_path, last_revmeta.revnum,
                     [revnum])[revnum].strip("/")
 
     def get_revnum(self):
@@ -135,21 +202,23 @@ class SvnBranch(Branch):
 
         :return: Revision number
         """
-        if self._lock_mode == 'r' and self._cached_revnum:
-            return self._cached_revnum
-        latest_revnum = self.repository.get_latest_revnum()
-        self._cached_revnum = self.repository._log.find_latest_change(
-            self.get_branch_path(), latest_revnum)
-        if self._cached_revnum is None:
-            raise NotBranchError(self.base)
-        return self._cached_revnum
+        return self.last_revmeta()[0].revnum
+
+    def last_revmeta(self):
+        """Return the revmeta element for the last revision in this branch.
+        """
+        for revmeta, mapping in self._revision_meta_history():
+            if not revmeta.is_hidden(mapping):
+                return revmeta, mapping
+        return None, None
 
     def check(self):
         """See Branch.Check.
 
         Doesn't do anything for Subversion repositories at the moment (yet).
         """
-        return BranchCheckResult(self)
+        # TODO: Check svn file properties?
+        return SubversionBranchCheckResult(self)
 
     def _create_heavyweight_checkout(self, to_location, revision_id=None, 
                                      hardlink=False):
@@ -177,8 +246,9 @@ class SvnBranch(Branch):
         :return: Revision number on the branch. 
         :raises NoSuchRevision: If the revision id was not found.
         """
-        (bp, revnum, mapping) = self.repository.lookup_revision_id(revid, 
-            ancestry=(self.get_branch_path(), self.get_revnum()))
+        (uuid, bp, revnum), mapping = self.repository.lookup_revision_id(revid, 
+            ancestry=(self.get_branch_path(), self.get_revnum()), 
+            project=self.project)
         assert bp.strip("/") == self.get_branch_path(revnum).strip("/"), \
                 "Got %r, expected %r" % (bp, self.get_branch_path(revnum))
         return revnum
@@ -225,18 +295,18 @@ class SvnBranch(Branch):
     def generate_revision_id(self, revnum):
         """Generate a new revision id for a revision on this branch."""
         assert isinstance(revnum, int)
-        try:
-            return self.repository.generate_revision_id(
-                revnum, self.get_branch_path(revnum), self.mapping)
-        except SubversionException, (_, num):
-            if num == ERR_FS_NO_SUCH_REVISION:
-                raise NoSuchRevision(self, revnum)
-            raise
+        revmeta_history = self._revision_meta_history()
+        for revmeta, mapping in revmeta_history:
+            if revmeta.revnum == revnum:
+                return revmeta.get_revision_id(mapping)
+            if revmeta.revnum < revnum:
+                break
+        raise NoSuchRevision(self, revnum)
 
     def get_config(self):
-        return BranchConfig(self)
+        return BranchConfig(self.base, self.repository.uuid)
        
-    def _get_nick(self):
+    def _get_nick(self, local=False, possible_master_transports=None):
         """Find the nick name for this branch.
 
         :return: Branch nick
@@ -251,16 +321,29 @@ class SvnBranch(Branch):
     def set_revision_history(self, rev_history):
         """See Branch.set_revision_history()."""
         if (rev_history == [] or 
-            not self.repository.has_revision(rev_history[-1])):
+            not self.repository.has_revision(rev_history[-1], project=self.project)):
             raise NotImplementedError("set_revision_history can't add ghosts")
-        push(self.repository.get_graph(), 
-             self, self.repository, rev_history[-1])
+        rev = self.repository.get_revision(rev_history[-1])
+        if rev.parent_ids:
+            base_revid = rev.parent_ids[0]
+        else:
+            base_revid = NULL_REVISION
+        push(self.repository.get_graph(), self.repository, 
+             self.get_branch_path(), self.get_config(), base_revid, 
+             self.repository, rev)
         self._clear_cached_state()
 
     def set_last_revision_info(self, revno, revid):
         """See Branch.set_last_revision_info()."""
 
-    def mainline_missing_revisions(self, other, stop_revision):
+    def _missing_revisions(self, other_repo, stop_revision, overwrite=False):
+        todo = self._mainline_missing_revisions(other_repo, stop_revision)
+        if todo is not None:
+            return todo
+        # Not possible to add cleanly onto mainline, perhaps need a replace operation
+        return self._otherline_missing_revisions(other_repo, stop_revision, overwrite)
+
+    def _mainline_missing_revisions(self, other_repo, stop_revision):
         """Find the revisions missing on the mainline.
         
         :param other: Other branch to retrieve revisions from.
@@ -268,14 +351,14 @@ class SvnBranch(Branch):
         """
         missing = []
         lastrevid = self.last_revision()
-        for revid in other.repository.iter_reverse_revision_history(stop_revision):
+        for revid in other_repo.iter_reverse_revision_history(stop_revision):
             if lastrevid == revid:
                 missing.reverse()
                 return missing
             missing.append(revid)
         return None
 
-    def otherline_missing_revisions(self, other, stop_revision, overwrite=False):
+    def _otherline_missing_revisions(self, other_repo, stop_revision, overwrite=False):
         """Find the revisions missing on the mainline.
         
         :param other: Other branch to retrieve revisions from.
@@ -283,14 +366,15 @@ class SvnBranch(Branch):
         :param overwrite: Whether or not the existing data should be overwritten
         """
         missing = []
-        for revid in other.repository.iter_reverse_revision_history(stop_revision):
-            if self.repository.has_revision(revid):
+        for revid in other_repo.iter_reverse_revision_history(stop_revision):
+            if self.repository.has_revision(revid, project=self.project):
                 missing.reverse()
                 return missing
             missing.append(revid)
         if not overwrite:
             return None
         else:
+            missing.reverse()
             return missing
  
     def last_revision_info(self):
@@ -303,9 +387,10 @@ class SvnBranch(Branch):
         if is_null(revision_id):
             return 0
         revmeta_history = self._revision_meta_history()
-        for revmeta in revmeta_history:
-            if revmeta.get_revision_id(self.mapping) == revision_id:
-                return len(revmeta_history) - revmeta_history.index(revmeta)
+        # FIXME: Maybe we can parse revision_id as a bzr-svn roundtripped revision?
+        for revmeta, mapping in revmeta_history:
+            if revmeta.get_revision_id(mapping) == revision_id:
+                return revmeta.get_revno(mapping)
         raise NoSuchRevision(self, revision_id)
 
     def get_root_id(self, revnum=None):
@@ -317,32 +402,26 @@ class SvnBranch(Branch):
 
     def set_push_location(self, location):
         """See Branch.set_push_location()."""
-        raise NotImplementedError(self.set_push_location)
+        mutter("setting push location for %s to %s", self.base, location)
 
     def get_push_location(self):
         """See Branch.get_push_location()."""
         # get_push_location not supported on Subversion
         return None
 
+    def _iter_revision_meta_ancestry(self):
+        return self.repository._iter_reverse_revmeta_mapping_ancestry(
+            self.get_branch_path(), self._revnum or self.repository.get_latest_revnum(), self.mapping, lhs_history=self._revision_meta_history())
+
     def _revision_meta_history(self):
         if self._revmeta_cache is None:
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
-                self._revmeta_cache = list(self.repository.iter_reverse_branch_changes(self.get_branch_path(), self.get_revnum(), to_revnum=0, mapping=self.mapping, pb=pb))
-            finally:
-                pb.finished()
+            self._revmeta_cache = util.lazy_readonly_list(self.repository._iter_reverse_revmeta_mapping_history(self.get_branch_path(), self._revnum or self.repository.get_latest_revnum(), to_revnum=0, mapping=self.mapping))
         return self._revmeta_cache
 
     def _gen_revision_history(self):
         """Generate the revision history from last revision
         """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            history = []
-            for revmeta in self._revision_meta_history():
-                history.append(revmeta.get_revision_id(self.mapping))
-        finally:
-            pb.finished()
+        history = [revmeta.get_revision_id(mapping) for revmeta, mapping in self._revision_meta_history() if not revmeta.is_hidden(mapping)]
         history.reverse()
         return history
 
@@ -350,10 +429,23 @@ class SvnBranch(Branch):
         """See Branch.last_revision()."""
         # Shortcut for finding the tip. This avoids expensive generation time
         # on large branches.
-        return self.generate_revision_id(self.get_revnum())
+        last_revmeta, mapping = self.last_revmeta()
+        return last_revmeta.get_revision_id(mapping)
+
+    @needs_write_lock
+    def dpull(self, source, stop_revision=None):
+        return dpush(self, source, stop_revision)
+
+    def import_last_revision_info(self, source_repo, revno, revid):
+        interrepo = InterToSvnRepository(source_repo, self.repository)
+        todo = self._missing_revisions(source_repo, revid, overwrite=False)
+        if todo is None:
+            raise DivergedBranches(self, None)
+        interrepo.push_branch(todo, self.layout, self.project, self.get_branch_path(), self.get_config())
 
     def pull(self, source, overwrite=False, stop_revision=None, 
-             _hook_master=None, run_hooks=True, _push_merged=None):
+             _hook_master=None, run_hooks=True, _push_merged=None,
+             _override_svn_revprops=None):
         """See Branch.pull()."""
         result = PullResult()
         result.source_branch = source
@@ -363,13 +455,32 @@ class SvnBranch(Branch):
         try:
             (result.old_revno, result.old_revid) = self.last_revision_info()
             self.update_revisions(source, stop_revision, overwrite, 
-                                  _push_merged=_push_merged)
+                                  _push_merged=_push_merged,
+                                  _override_svn_revprops=_override_svn_revprops)
             result.tag_conflicts = source.tags.merge_to(self.tags, overwrite)
             (result.new_revno, result.new_revid) = self.last_revision_info()
             return result
         finally:
             source.unlock()
 
+    @needs_write_lock
+    def update_revisions(self, other, stop_revision=None, overwrite=False,
+                         graph=None, _override_svn_revprops=None, 
+                         _push_merged=None):
+        """Pull in new perfect-fit revisions.
+
+        :param other: Another Branch to pull from
+        :param stop_revision: Updated until the given revision
+        :param overwrite: Always set the branch pointer, rather than checking
+            to see if it is a proper descendant.
+        :param graph: A Graph object that can be used to query history
+            information. This can be None.
+        :return: None
+        """
+        return InterBranch.get(other, self).update_revisions(stop_revision,
+            overwrite, graph, _override_svn_revprops=_override_svn_revprops, 
+            _push_merged=_push_merged)
+
     def generate_revision_history(self, revision_id, last_rev=None, 
         other_branch=None):
         """Create a new revision history that will finish with revision_id.
@@ -401,48 +512,8 @@ class SvnBranch(Branch):
             revno = self.revision_id_to_revno(revision_id)
         destination.set_last_revision_info(revno, revision_id)
 
-    def update_revisions(self, other, stop_revision=None, overwrite=False, 
-                         graph=None, _push_merged=False):
-        """See Branch.update_revisions()."""
-        if stop_revision is None:
-            stop_revision = ensure_null(other.last_revision())
-        if (self.last_revision() == stop_revision or
-            self.last_revision() == other.last_revision()):
-            return
-        if graph is None:
-            graph = self.repository.get_graph()
-        other_graph = other.repository.get_graph()
-        if not other_graph.is_ancestor(self.last_revision(), 
-                                                        stop_revision):
-            if graph.is_ancestor(stop_revision, self.last_revision()):
-                return
-            if not overwrite:
-                raise DivergedBranches(self, other)
-        todo = self.mainline_missing_revisions(other, stop_revision)
-        if todo is None:
-            # Not possible to add cleanly onto mainline, perhaps need a replace operation
-            todo = self.otherline_missing_revisions(other, stop_revision, overwrite)
-        if todo is None:
-            raise DivergedBranches(self, other)
-        if _push_merged is None:
-            _push_merged = self.layout.push_merged_revisions(self.project)
-        self._push_missing_revisions(graph, other, other_graph, todo, 
-                                     _push_merged)
-
-    def _push_missing_revisions(self, my_graph, other, other_graph, todo, 
-                                push_merged=False):
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            for revid in todo:
-                pb.update("pushing revisions", todo.index(revid), 
-                          len(todo))
-                if push_merged:
-                    parent_revids = other_graph.get_parent_map([revid])[revid]
-                    push_ancestors(self.repository, other.repository, self.layout, self.project, parent_revids, other_graph)
-                push(my_graph, self, other.repository, revid)
-                self._clear_cached_state()
-        finally:
-            pb.finished()
+    def is_locked(self):
+        return self._lock_count != 0
 
     def lock_write(self):
         """See Branch.lock_write()."""
@@ -489,22 +560,12 @@ class SvnBranch(Branch):
         """See Branch.get_physical_lock_status()."""
         return False
 
-    def sprout(self, to_bzrdir, revision_id=None):
-        """See Branch.sprout()."""
-        result = to_bzrdir.create_branch()
-        self.copy_content_into(result, revision_id=revision_id)
-        result.set_parent(self.bzrdir.root_transport.base)
-        return result
-
     def get_stacked_on_url(self):
         raise UnstackableBranchFormat(self._format, self.base)
 
     def __str__(self):
         return '%s(%r)' % (self.__class__.__name__, self.base)
 
-    def supports_tags(self):
-        return self._format.supports_tags()
-
     __repr__ = __str__
 
 
@@ -515,7 +576,7 @@ class SvnBranchFormat(BranchFormat):
 
     def __get_matchingbzrdir(self):
         """See BranchFormat.__get_matchingbzrdir()."""
-        from remote import SvnRemoteFormat
+        from bzrlib.plugins.svn.remote import SvnRemoteFormat
         return SvnRemoteFormat()
 
     _matchingbzrdir = property(__get_matchingbzrdir)
@@ -534,3 +595,152 @@ class SvnBranchFormat(BranchFormat):
 
     def supports_tags(self):
         return True
+
+    def make_tags(self, branch):
+        if branch.supports_tags():
+            return SubversionTags(branch)
+        else:
+            return tag.DisabledTags(branch)
+
+
+class InterSvnOtherBranch(InterBranch):
+    """InterBranch implementation that is optimized for copying from 
+    Subversion.
+
+    The two main differences with the generic implementation are:
+     * No revision numbers are calculated for the Subversion branch
+       (since this requires browsing the entire history)
+     * Only recent tags are fetched, since that saves a lot of 
+       history browsing operations
+    """
+
+    def update_revisions(self, stop_revision=None, overwrite=False, graph=None):
+        """See InterBranch.update_revisions()."""
+        self.source.lock_read()
+        try:
+            other_last_revision = self.source.last_revision()
+            if stop_revision is None:
+                stop_revision = other_last_revision
+                if is_null(stop_revision):
+                    # if there are no commits, we're done.
+                    return
+
+            # what's the current last revision, before we fetch [and 
+            # change it possibly]
+            last_rev = ensure_null(self.target.last_revision())
+            # we fetch here so that we don't process data twice in the 
+            # common case of having something to pull, and so that the 
+            # check for already merged can operate on the just fetched 
+            # graph, which will be cached in memory.
+            self.target.fetch(self.source, stop_revision)
+            # Check to see if one is an ancestor of the other
+            if not overwrite:
+                if graph is None:
+                    graph = self.target.repository.get_graph()
+                if self.target._check_if_descendant_or_diverged(
+                        stop_revision, last_rev, graph, self.source):
+                    # stop_revision is a descendant of last_rev, but we 
+                    # aren't overwriting, so we're done.
+                    return
+            self.target.generate_revision_history(stop_revision)
+        finally:
+            self.source.unlock()
+
+    def pull(self, overwrite=False, stop_revision=None, 
+             _hook_master=None, run_hooks=True, possible_transports=None,
+             _override_hook_target=None):
+        """See InterBranch.pull()."""
+        result = SubversionPullResult()
+        if _override_hook_target is None:
+            result.target_branch = self
+        else:
+            result.target_branch = _override_hook_target
+        result.source_branch = self.source
+        result.master_branch = None
+        result.target_branch = self.target
+        self.source.lock_read()
+        try:
+            (result.old_revno, result.old_revid) = self.target.last_revision_info()
+            try:
+                result.old_revmeta, _ = self.source.repository._get_revmeta(result.old_revid)
+                tags_since_revnum = result.old_revmeta.revnum
+            except NoSuchRevision:
+                result.old_revmeta = None
+                tags_since_revnum = None
+            if stop_revision is not None:
+                result.new_revmeta, _ = self.source.repository._get_revmeta(stop_revnum)
+                tags_until_revnum = result.new_revmeta.revnum
+            else:
+                tags_until_revnum = self.source.repository.get_latest_revnum()
+            self.update_revisions(stop_revision, overwrite)
+            result.tag_conflicts = self.source.tags.merge_to(self.target.tags, overwrite, _from_revnum=tags_since_revnum, _to_revnum=tags_until_revnum)
+            (result.new_revno, result.new_revid) = self.target.last_revision_info()
+            if stop_revision is None:
+                result.new_revmeta, _ = self.source.repository._get_revmeta(result.new_revid)
+            if _hook_master:
+                result.master_branch = _hook_master
+                result.local_branch = result.target_branch
+            else:
+                result.master_branch = result.target_branch
+                result.local_branch = None
+            if run_hooks:
+                for hook in Branch.hooks['post_pull']:
+                    hook(result)
+        finally:
+            self.source.unlock()
+        return result
+
+    @classmethod
+    def is_compatible(self, source, target):
+        return isinstance(source, SvnBranch)
+
+InterBranch.register_optimiser(InterSvnOtherBranch)
+
+
+class InterOtherSvnBranch(InterBranch):
+    """InterBranch implementation that is optimized for copying to
+    Subversion.
+
+    """
+    def update_revisions(self, stop_revision=None, overwrite=False, graph=None,
+        _push_merged=False, _override_svn_revprops=None):
+        """See Branch.update_revisions()."""
+        if stop_revision is None:
+            stop_revision = ensure_null(self.source.last_revision())
+        if (self.target.last_revision() == stop_revision or
+            self.target.last_revision() == self.source.last_revision()):
+            return
+        # Request graph from other repository, since it's most likely 
+        # than Subversion
+        graph = self.source.repository.get_graph(self.target.repository)
+        if not graph.is_ancestor(self.target.last_revision(), stop_revision):
+            if graph.is_ancestor(stop_revision, self.target.last_revision()):
+                return
+            if not overwrite:
+                raise DivergedBranches(self.target, self.source)
+        todo = self.target._missing_revisions(self.source.repository, stop_revision, overwrite)
+        if todo is None:
+            raise DivergedBranches(self.target, self.source)
+        if _push_merged is None:
+            _push_merged = (self.target.layout.push_merged_revisions(self.target.project) and 
+                            self.target.get_config().get_push_merged_revisions())
+        if self.target.mapping.supports_hidden and self.target.repository.has_revision(stop_revision):
+            # Revision is already present in the repository, so just 
+            # copy from there.
+            create_branch_with_hidden_commit(self.target.repository, 
+                self.target.get_branch_path(), stop_revision, set_metadata=True,
+                deletefirst=True)
+        else:
+            interrepo = InterToSvnRepository(self.source.repository, 
+                self.target.repository, graph)
+            interrepo.push_branch(todo, self.target.layout, self.target.project, 
+                self.target.get_branch_path(), self.target.get_config(),
+                push_merged=_push_merged)
+        self.target._clear_cached_state()
+
+    @classmethod
+    def is_compatible(self, source, target):
+        return isinstance(target, SvnBranch)
+
+
+InterBranch.register_optimiser(InterOtherSvnBranch)
