@@ -1,8 +1,8 @@
-# Copyright (C) 2005-2007 by Jelmer Vernooij
+# Copyright (C) 2005-2009 by Jelmer Vernooij
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -16,20 +16,34 @@
 """Conversion of full repositories."""
 
 import os
+from subvertpy import (
+    ERR_STREAM_MALFORMED_DATA,
+    NODE_FILE,
+    SubversionException,
+    repos,
+    )
 
-from bzrlib import osutils, ui, urlutils
-from bzrlib.bzrdir import BzrDir, Converter
-from bzrlib.errors import (BzrError, NotBranchError, NoSuchFile, 
-                           NoRepositoryPresent) 
+from bzrlib import (
+    bzrdir,
+    osutils,
+    ui,
+    urlutils,
+    )
+from bzrlib.errors import (
+    BzrError,
+    NotBranchError,
+    NoSuchFile,
+    NoSuchRevision,
+    NoRepositoryPresent,
+    ) 
 from bzrlib.repository import InterRepository
 from bzrlib.revision import ensure_null
 from bzrlib.transport import get_transport
 
-from bzrlib.plugins.svn import repos
 from bzrlib.plugins.svn.branch import SvnBranch
-from bzrlib.plugins.svn.core import SubversionException
-from bzrlib.plugins.svn.errors import ERR_STREAM_MALFORMED_DATA
+from bzrlib.plugins.svn.fetch import FetchRevisionFinder
 from bzrlib.plugins.svn.format import get_rich_root_format
+from bzrlib.plugins.svn.revmeta import filter_revisions
 
 LATEST_IMPORT_REVISION_FILENAME = "svn-import-revision"
 
@@ -104,24 +118,34 @@ def load_dumpfile(dumpfile, outputdir):
     else:
         file = open(dumpfile)
     try:
-        r.load_fs(file, StringIO(), repos.LOAD_UUID_DEFAULT)
-    except SubversionException, (_, num):
-        if num == ERR_STREAM_MALFORMED_DATA:
-            raise NotDumpFile(dumpfile)
-        raise
+        try:
+            r.load_fs(file, StringIO(), repos.LOAD_UUID_DEFAULT)
+        except SubversionException, (_, num):
+            if num == ERR_STREAM_MALFORMED_DATA:
+                raise NotDumpFile(dumpfile)
+            raise
+    finally:
+        file.close()
     return r
 
 
-def convert_repository(source_repos, output_url, scheme=None, layout=None,
+def contains_parent_path(s, p):
+    while p != "":
+        if p in s:
+            return p
+        (p, _) = urlutils.split(p)
+    return False
+
+
+def convert_repository(source_repos, output_url, layout=None,
                        create_shared_repo=True, working_trees=False, all=False,
                        format=None, filter_branch=None, keep=False, 
-                       incremental=False):
+                       incremental=False, to_revnum=None, prefix=None):
     """Convert a Subversion repository and its' branches to a 
     Bazaar repository.
 
     :param source_repos: Subversion repository
     :param output_url: URL to write Bazaar repository to.
-    :param scheme: Branching scheme to use.
     :param layout: Repository layout (object) to use
     :param create_shared_repo: Whether to create a shared Bazaar repository
     :param working_trees: Whether to create working trees
@@ -129,7 +153,20 @@ def convert_repository(source_repos, output_url, scheme=None, layout=None,
         branches, should be imported
     :param format: Format to use
     """
-    from mapping3 import SchemeDerivedLayout, set_branching_scheme
+    def remove_branches(to_transport, removed_branches, exceptions):
+        # Remove removed branches
+        for bp in removed_branches:
+            skip = False
+            for e in exceptions:
+                if bp.startswith(e+"/"):
+                    skip = True
+                    break
+            if skip:
+                continue
+            fullpath = to_transport.local_abspath(bp)
+            if not os.path.isdir(fullpath):
+                continue
+            osutils.rmtree(fullpath)
     assert not all or create_shared_repo
     if format is None:
         format = get_rich_root_format()
@@ -140,29 +177,31 @@ def convert_repository(source_repos, output_url, scheme=None, layout=None,
             return dirs[path]
         nt = to_transport.clone(path)
         try:
-            dirs[path] = BzrDir.open_from_transport(nt)
+            dirs[path] = bzrdir.BzrDir.open_from_transport(nt)
         except NotBranchError:
-            transport_makedirs(to_transport, urlutils.join(to_transport.base, path))
+            transport_makedirs(to_transport, 
+                               urlutils.join(to_transport.base, path))
             dirs[path] = format.initialize_on_transport(nt)
         return dirs[path]
 
     if layout is not None:
         source_repos.set_layout(layout)
-    elif scheme is not None:
-        set_branching_scheme(source_repos, scheme)
-        layout = SchemeDerivedLayout(source_repos, scheme)
     else:
         layout = source_repos.get_layout()
 
     if create_shared_repo:
         try:
             target_repos = get_dir("").open_repository()
-            assert (layout.is_branch("") or layout.is_tag("") or target_repos.is_shared())
+            target_repos_is_empty = False # FIXME: Call Repository.is_empty() ?
+            assert (layout.is_branch("") or 
+                    target_repos.is_shared())
         except NoRepositoryPresent:
             target_repos = get_dir("").create_repository(shared=True)
+            target_repos_is_empty = True
         target_repos.set_make_working_trees(working_trees)
     else:
         target_repos = None
+        target_repos_is_empty = False
 
     source_repos.lock_read()
     try:
@@ -171,51 +210,80 @@ def convert_repository(source_repos, output_url, scheme=None, layout=None,
                                                          source_repos.uuid)
         else:
             from_revnum = 0
-        to_revnum = source_repos.get_latest_revnum()
-        # Searching history for touched and removed branches is more expensive than 
-        # just listing the branches in HEAD, so avoid it if possible.
-        # If there's more than one subdirectory (we always have .bzr), we may 
-        # have to remove existing branches.
-        if from_revnum == 0 or (not keep and len(to_transport.list_dir(".")) > 1):
-            removed_branches, changed_branches = source_repos.find_branches_between(layout=layout, 
-                from_revnum=from_revnum, to_revnum=to_revnum)
-            existing_branches = []
-            for bp in changed_branches:
-                try:
-                    existing_branches.append(SvnBranch(source_repos, bp))
-                except NotBranchError: # Skip non-directories
-                    pass
+        project = None
+        if to_revnum is None:
+            to_revnum = source_repos.get_latest_revnum()
+        if to_revnum < from_revnum:
+            return
+        mapping = source_repos.get_mapping()
+        existing_branches = {}
+        deleted = set()
+        it = source_repos._revmeta_provider.iter_all_changes(layout, 
+                mapping.is_branch_or_tag, to_revnum, from_revnum, 
+                project=project, prefix=prefix)
+        if create_shared_repo:
+            revfinder = FetchRevisionFinder(source_repos, target_repos, 
+                                            target_repos_is_empty)
+            revmetas = []
         else:
-            existing_branches = source_repos.find_branches(layout=layout, revnum=to_revnum)
-
-        if filter_branch is not None:
-            existing_branches = filter(filter_branch, existing_branches)
+            revmetas = None
+        pb = ui.ui_factory.nested_progress_bar()
+        if all:
+            heads = None
+        else:
+            heads = set()
+        try:
+            for kind, item in it:
+                if kind == "revision":
+                    pb.update("finding branches", to_revnum-item.revnum, 
+                              to_revnum-from_revnum)
+                    if (not item.branch_path in existing_branches and 
+                        layout.is_branch(item.branch_path, project=project) and 
+                        not contains_parent_path(deleted, item.branch_path)):
+                        existing_branches[item.branch_path] = SvnBranch(
+                            source_repos, item.branch_path, revnum=item.revnum,
+                            _skip_check=True, mapping=mapping)
+                        if heads is not None:
+                            heads.add(item)
+                    if revmetas is not None:
+                        revmetas.append(item)
+                elif kind == "delete":
+                    deleted.add(item)
+        finally:
+            pb.finished()
 
         if create_shared_repo:
             inter = InterRepository.get(source_repos, target_repos)
-
-            if all:
+            if (target_repos.is_shared() and 
+                getattr(inter, '_supports_revmetas', None) and 
+                inter._supports_revmetas):
+                # TODO: Skip revisions in removed branches unless all=True
+                pb = ui.ui_factory.nested_progress_bar()
+                try:
+                    pb.update("checking revisions to fetch", 0, len(revmetas))
+                    revfinder.find_iter_revisions(revmetas, mapping, 
+                        heads=heads, prefix=prefix, pb=pb)
+                finally:
+                    pb.finished()
+                inter.fetch(needed=revfinder.get_missing())
+            else:
                 inter.fetch()
-            elif (target_repos.is_shared() and 
-                  getattr(inter, '_supports_branches', None) and 
-                  inter._supports_branches):
-                inter.fetch(branches=existing_branches)
 
         if not keep:
-            # Remove removed branches
-            for bp in removed_branches:
-                # TODO: Perhaps check if path is a valid branch with the right last
-                # revid?
-                fullpath = to_transport.local_abspath(bp)
-                if not os.path.isdir(fullpath):
-                    continue
-                osutils.rmtree(fullpath)
+            remove_branches(to_transport, deleted, existing_branches.keys())
 
+        existing_branches = existing_branches.values()
+        if filter_branch is not None:
+            existing_branches = filter(filter_branch, existing_branches)
         source_graph = source_repos.get_graph()
         pb = ui.ui_factory.nested_progress_bar()
         try:
             for i, source_branch in enumerate(existing_branches):
-                pb.update("%s:%d" % (source_branch.get_branch_path(), source_branch.get_revnum()), i, len(existing_branches))
+                try:
+                    pb.update("%s:%d" % (source_branch.get_branch_path(), 
+                        source_branch.get_revnum()), i, len(existing_branches))
+                except SubversionException, (_, ERR_FS_NOT_DIRECTORY):
+                    continue
                 target_dir = get_dir(source_branch.get_branch_path())
                 if not create_shared_repo:
                     try:
@@ -236,7 +304,13 @@ def convert_repository(source_repos, output_url, scheme=None, layout=None,
                             ensure_null(target_branch.last_revision()),
                             ensure_null(source_branch.last_revision())):
                         target_branch.set_revision_history([])
-                    target_branch.pull(source_branch)
+                    try:
+                        target_branch.pull(source_branch)
+                    except NoSuchRevision:
+                        if source_branch.check_path() == NODE_FILE:
+                            to_transport.delete_tree(source_branch.get_branch_path())
+                            continue
+                        raise
                 if working_trees and not target_dir.has_workingtree():
                     target_dir.create_workingtree()
         finally:
@@ -245,19 +319,6 @@ def convert_repository(source_repos, output_url, scheme=None, layout=None,
         source_repos.unlock()
 
     if target_repos is not None:
-        put_latest_svn_import_revision(target_repos, source_repos.uuid, to_revnum)
+        put_latest_svn_import_revision(target_repos, source_repos.uuid, 
+                                       to_revnum)
         
-
-class SvnConverter(Converter):
-    """Converts from a Subversion directory to a bzr dir."""
-    def __init__(self, target_format):
-        """Create a SvnConverter.
-        :param target_format: The format the resulting repository should be.
-        """
-        super(SvnConverter, self).__init__()
-        self.target_format = target_format
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        convert_repository(to_convert.open_repository(), to_convert.base, 
-                           format=self.target_format, all=True, pb=pb)
