@@ -1,8 +1,8 @@
-# Copyright (C) 2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2007-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,47 +15,24 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Subversion-specific errors and conversion of Subversion-specific errors."""
 
-from bzrlib.errors import (BzrError, ConnectionError, ConnectionReset, 
-                           LockError, NotBranchError, PermissionDenied, 
-                           DependencyNotPresent, NoRepositoryPresent,
-                           TransportError, UnexpectedEndOfContainerError,
-                           NoSuchRevision)
-
+import subvertpy
 import urllib
-from bzrlib.plugins.svn import core
 
+import bzrlib.errors
+from bzrlib.errors import (
+    BzrError,
+    ConnectionError,
+    ConnectionReset,
+    InvalidRevisionSpec,
+    LockError,
+    PermissionDenied,
+    NoRepositoryPresent,
+    NoSuchRevision,
+    TransportError,
+    UnexpectedEndOfContainerError,
+    )
 
-ERR_UNKNOWN_HOSTNAME = 670002
-ERR_UNKNOWN_HOSTNAME = 670002
-ERR_RA_SVN_CONNECTION_CLOSED = 210002
-ERR_WC_LOCKED = 155004
-ERR_RA_NOT_AUTHORIZED = 170001
-ERR_INCOMPLETE_DATA = 200003
-ERR_RA_SVN_MALFORMED_DATA = 210004
-ERR_RA_NOT_IMPLEMENTED = 170003
-ERR_FS_NO_SUCH_REVISION = 160006
-ERR_FS_TXN_OUT_OF_DATE = 160028
-ERR_REPOS_DISABLED_FEATURE = 165006
-ERR_STREAM_MALFORMED_DATA = 140001
-ERR_RA_ILLEGAL_URL = 170000
-ERR_RA_LOCAL_REPOS_OPEN_FAILED = 180001
-ERR_BAD_URL = 125002
-ERR_RA_DAV_REQUEST_FAILED = 175002
-ERR_RA_DAV_PATH_NOT_FOUND = 175007
-ERR_FS_NOT_DIRECTORY = 160016
-ERR_FS_NOT_FOUND = 160013
-ERR_FS_ALREADY_EXISTS = 160020
-ERR_RA_SVN_REPOS_NOT_FOUND = 210005
-ERR_WC_NOT_DIRECTORY = 155007
-ERR_ENTRY_EXISTS = 150002
-ERR_WC_PATH_NOT_FOUND = 155010
-ERR_CANCELLED = 200015
-ERR_WC_UNSUPPORTED_FORMAT = 155021
-ERR_UNKNOWN_CAPABILITY = 200026
-ERR_AUTHN_NO_PROVIDER = 215001
-ERR_RA_DAV_RELOCATED = 175011
-ERR_FS_NOT_FILE = 160017
-ERR_WC_BAD_ADM_LOG = 155009
+from bzrlib.lazy_import import lazy_import
 
 
 class InvalidBzrSvnRevision(NoSuchRevision):
@@ -65,13 +42,14 @@ class InvalidBzrSvnRevision(NoSuchRevision):
         self.revid = revid
 
 
-class NotSvnBranchPath(NotBranchError):
+class NotSvnBranchPath(BzrError):
     """Error raised when a path was specified that did not exist."""
-    _fmt = """%(path)s is not a valid Subversion branch path. 
-See 'bzr help svn-branching-schemes' for details."""
+    _fmt = """%(branch_path)s is not a valid Subversion branch path. 
+See 'bzr help svn-layout' for details."""
 
     def __init__(self, branch_path, mapping=None):
-        NotBranchError.__init__(self, urllib.quote(branch_path))
+        BzrError.__init__(self)
+        self.branch_path = urllib.quote(branch_path)
         self.mapping = mapping
 
 
@@ -89,21 +67,15 @@ class ChangesRootLHSHistory(BzrError):
 class MissingPrefix(BzrError):
     _fmt = """Prefix missing for %(path)s; please create it before pushing. """
 
-    def __init__(self, path):
+    def __init__(self, path, existing_path):
         BzrError.__init__(self)
         self.path = path
-
-
-class RaRequestFailed(BzrError):
-    _fmt = """A Subversion remote access command failed: %(message)"""
-
-    def __init__(self, message):
-        BzrError.__init__(self)
-        self.mesage = message
+        self.existing_path = existing_path
 
 
 class RevpropChangeFailed(BzrError):
-    _fmt = """Unable to set revision property %(name)s."""
+    _fmt = """Unable to set revision property %(name)s. 
+Does the repository pre-revprop-change hook allow property changes?"""
 
     def __init__(self, name):
         BzrError.__init__(self)
@@ -111,7 +83,7 @@ class RevpropChangeFailed(BzrError):
 
 
 class DavRequestFailed(BzrError):
-    _fmt = """%(msg)s"""
+    _fmt = """A Subversion remote access command failed: %(msg)s"""
 
     def __init__(self, msg):
         BzrError.__init__(self)
@@ -126,26 +98,22 @@ def convert_error(err):
     """
     (msg, num) = err.args
 
-    if num == ERR_RA_SVN_CONNECTION_CLOSED:
+    if num == subvertpy.ERR_RA_SVN_CONNECTION_CLOSED:
         return ConnectionReset(msg=msg)
-    elif num == ERR_WC_LOCKED:
+    elif num == subvertpy.ERR_WC_LOCKED:
         return LockError(message=msg)
-    elif num == ERR_RA_NOT_AUTHORIZED:
+    elif num == subvertpy.ERR_RA_NOT_AUTHORIZED:
         return PermissionDenied('.', msg)
-    elif num == ERR_INCOMPLETE_DATA:
+    elif num == subvertpy.ERR_INCOMPLETE_DATA:
         return UnexpectedEndOfContainerError()
-    elif num == ERR_RA_SVN_MALFORMED_DATA:
+    elif num == subvertpy.ERR_RA_SVN_MALFORMED_DATA:
         return TransportError("Malformed data", msg)
-    elif num == ERR_RA_NOT_IMPLEMENTED:
+    elif num == subvertpy.ERR_RA_NOT_IMPLEMENTED:
         return NotImplementedError("Function not implemented in remote server")
-    elif num == ERR_RA_DAV_REQUEST_FAILED:
-        return RaRequestFailed(msg)
-    elif num == ERR_UNKNOWN_HOSTNAME:
+    elif num == subvertpy.ERR_UNKNOWN_HOSTNAME:
         return ConnectionError(msg=msg)
-    elif num == ERR_RA_DAV_REQUEST_FAILED:
+    elif num == subvertpy.ERR_RA_DAV_REQUEST_FAILED:
         return DavRequestFailed(msg)
-    elif num > 0 and num < 1000:
-        return OSError(num, msg)
     else:
         return err
 
@@ -157,7 +125,7 @@ def convert_svn_error(unbound):
     def convert(*args, **kwargs):
         try:
             return unbound(*args, **kwargs)
-        except core.SubversionException, e:
+        except subvertpy.SubversionException, e:
             raise convert_error(e)
 
     convert.__doc__ = unbound.__doc__
@@ -165,11 +133,6 @@ def convert_svn_error(unbound):
     return convert
 
 
-class LocalCommitsUnsupported(BzrError):
-
-    _fmt = 'Local commits are not supported for lightweight Subversion checkouts.'
-
-
 class InvalidPropertyValue(BzrError):
     _fmt = 'Invalid property value for Subversion property %(property)s: %(msg)s'
 
@@ -178,12 +141,6 @@ class InvalidPropertyValue(BzrError):
         self.property = property
         self.msg = msg
 
-class RebaseNotPresent(DependencyNotPresent):
-    _fmt = "Unable to import bzr-rebase (required for svn-upgrade support): %(error)s"
-
-    def __init__(self, error):
-        DependencyNotPresent.__init__(self, 'bzr-rebase', error)
-
 
 class InvalidFileName(BzrError):
     _fmt = "Unable to convert Subversion path %(path)s because it contains characters invalid in Bazaar."
@@ -201,14 +158,83 @@ class CorruptMappingData(BzrError):
         self.path = path
 
 
-class InvalidSvnBranchPath(NotBranchError):
-    """Error raised when a path was specified that is not a child of or itself
-    a valid branch path in the current branching scheme."""
-    _fmt = """%(path)s is not a valid Subversion branch path in the current 
-repository layout. See 'bzr help svn-repository-layout' for details."""
+class LayoutUnusable(BzrError):
+    _fmt = """Unable to use layout %(layout)r with mapping %(mapping)r."""
 
-    def __init__(self, path, layout):
-        assert isinstance(path, str)
-        NotBranchError.__init__(self, urllib.quote(path))
+    def __init__(self, layout, mapping):
+        BzrError.__init__(self)
         self.layout = layout
+        self.mapping = mapping
 
+
+class AppendRevisionsOnlyViolation(bzrlib.errors.AppendRevisionsOnlyViolation):
+
+    _fmt = ('Operation denied because it would change the main history.'
+           ' Set the append_revisions_only setting to yes on'
+           ' branch "%(location)s".')
+
+
+class FileIdMapIncomplete(BzrError):
+    _fmt = """Unable to find file id for child '%(child)s' in '%(parent)s' in %(revmeta)r."""
+
+    def __init__(self, child, parent, revmeta):
+        BzrError.__init__(self)
+        self.child = child
+        self.parent = parent
+        self.revmeta = revmeta
+
+
+class InvalidFileId(BzrError):
+    _fmt = """Unable to parse file id %(fileid)s."""
+
+    def __init__(self, fileid):
+        BzrError.__init__(self)
+        self.fileid = fileid
+
+
+class DifferentSubversionRepository(BzrError):
+    _fmt = """UUID %(got)s does not match expected UUID %(expected)s."""
+
+    def __init__(self, got, expected):
+        BzrError.__init__(self)
+        self.got = got
+        self.expected = expected
+
+
+class InconsistentLHSParent(BzrError):
+    _fmt = """Recorded left hand side parent %(recorded_lhs)s for revision %(revision)r does not match actual left hand side parent %(actual_lhs)s."""
+
+    def __init__(self, revision, recorded_lhs, actual_lhs):
+        BzrError.__init__(self)
+        self.revision = revision
+        self.recorded_lhs = recorded_lhs
+        self.actual_lhs = actual_lhs
+
+
+class UnknownMapping(BzrError):
+    _fmt = """Attempt to use unknown mapping. %(extra)s """
+
+    def __init__(self, mapping, extra=None):
+        BzrError.__init__(self, extra=(extra or ""))
+        self.mapping = mapping
+
+
+class AbsentPath(BzrError):
+    _fmt = """Unable to access %(path)s: no permission?. """
+    
+    def __init__(self, path):
+        BzrError.__init__(self, path=path)
+
+
+class AmbiguousRevisionSpec(InvalidRevisionSpec):
+    _fmt = """Revision specifier %(revspec)s is ambiguous for %(branch)s. """
+
+    def __init__(self, revspec, branch):
+        BzrError.__init__(self, revspec=revspec, branch=branch)
+
+
+class NoCustomBranchPaths(BzrError):
+    _fmt = """Layout %(layout)r does not support custom branch paths."""
+
+    def __init__(self, layout=None):
+        BzrError.__init__(self, layout=layout)
