@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2008 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,27 +15,21 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Subversion BzrDir formats."""
 
-from bzrlib.bzrdir import BzrDirFormat, format_registry
-from bzrlib.errors import UninitializableFormat
-from bzrlib.lazy_import import lazy_import
+from bzrlib import errors as bzr_errors
+from bzrlib.bzrdir import (
+    BzrDirFormat,
+    format_registry,
+    )
 from bzrlib.lockable_files import TransportLock
 from bzrlib.trace import mutter
 
-import os
-
-lazy_import(globals(), """
-from bzrlib.plugins.svn import errors, remote
-
-from bzrlib import errors as bzr_errors
-""")
-
 def get_rich_root_format():
     format = BzrDirFormat.get_default_format()
     if format.repository_format.rich_root_data:
         return format
     # Default format does not support rich root data, 
-    # fall back to 1.6-rich-root
-    format = format_registry.make_bzrdir('rich-root-pack')
+    # fall back to 1.9-rich-root
+    format = format_registry.make_bzrdir('1.9-rich-root')
     assert format.repository_format.rich_root_data
     return format
 
@@ -45,47 +39,102 @@ def get_nested_tree_format():
     if format.repository_format.supports_tree_reference:
         return format
     # Default format does not support rich root data, 
-    # fall back to pack-0.92-subtree
-    format = format_registry.make_bzrdir('pack-0.92-subtree')
+    # fall back to development-subtree
+    format = format_registry.make_bzrdir('development-subtree')
     assert format.repository_format.supports_tree_reference
     return format
 
 
-class SvnRemoteFormat(BzrDirFormat):
-    """Format for the Subversion smart server."""
+class SvnControlFormat(BzrDirFormat):
+    """Format for a Subversion control dir."""
     _lock_class = TransportLock
 
     def __init__(self):
-        super(SvnRemoteFormat, self).__init__()
+        super(SvnControlFormat, self).__init__()
+        self.__repository_format = None
+
+    @classmethod
+    def _check_versions(cls):
         from bzrlib.plugins.svn import lazy_check_versions
         lazy_check_versions()
-        from bzrlib.plugins.svn.repository import SvnRepositoryFormat
-        self.repository_format = SvnRepositoryFormat()
+
+    @property
+    def repository_format(self):
+        if self.__repository_format is None:
+            self._check_versions()
+            from bzrlib.plugins.svn.repository import SvnRepositoryFormat
+            self.__repository_format = SvnRepositoryFormat()
+        return self.__repository_format
+
+    def is_supported(self):
+        """See BzrDir.is_supported()."""
+        return True
+ 
+
+class SvnRemoteFormat(SvnControlFormat):
+    """Format for the Subversion smart server."""
+
+    def __init__(self):
+        super(SvnRemoteFormat, self).__init__()
 
     @classmethod
     def probe_transport(klass, transport):
-        from bzrlib.plugins.svn.transport import get_svn_ra_transport
-        from bzrlib.plugins.svn import core
+        from bzrlib.transport.local import LocalTransport
         format = klass()
 
+        if isinstance(transport, LocalTransport):
+            # Cheaper way to figure out if there is a svn repo
+            maybe = False
+            subtransport = transport
+            while subtransport:
+                if subtransport.has("format"):
+                    maybe = True
+                    break
+                prevsubtransport = subtransport
+                subtransport = prevsubtransport.clone("..")
+                if subtransport.base == prevsubtransport.base:
+                    break
+            if not maybe:
+                raise bzr_errors.NotBranchError(path=transport.base)
+
+        klass._check_versions()
+        from bzrlib.plugins.svn.transport import get_svn_ra_transport
+        from bzrlib.plugins.svn.errors import DavRequestFailed
+        import subvertpy
         try:
             transport = get_svn_ra_transport(transport)
-        except core.SubversionException, (msg, num):
-            if num in (errors.ERR_RA_ILLEGAL_URL, \
-                       errors.ERR_RA_LOCAL_REPOS_OPEN_FAILED, \
-                       errors.ERR_BAD_URL):
+        except subvertpy.SubversionException, (msg, num):
+            if num in (subvertpy.ERR_RA_ILLEGAL_URL, \
+                       subvertpy.ERR_RA_LOCAL_REPOS_OPEN_FAILED, \
+                       subvertpy.ERR_BAD_URL):
                 mutter("Unable to open %r with Subversion: %s", 
                         transport, msg)
                 raise bzr_errors.NotBranchError(path=transport.base)
+        except bzr_errors.InvalidURL:
+            raise bzr_errors.NotBranchError(path=transport.base)
+        except DavRequestFailed, e:
+            if "501 Unsupported method" in e.msg:
+                raise bzr_errors.NotBranchError(path=transport.base)
+            else:
+                raise
 
         return format
 
     def _open(self, transport):
-        from bzrlib.plugins.svn.core import SubversionException
+        self._check_versions()
+        import subvertpy
+        from bzrlib.plugins.svn import remote
+        ERR_XML_MALFORMED = getattr(subvertpy, "ERR_XML_MALFORMED", 130003)
         try: 
             return remote.SvnRemoteAccess(transport, self)
-        except SubversionException, (_, num):
-            if num == errors.ERR_RA_DAV_REQUEST_FAILED:
+        except subvertpy.SubversionException, (_, num):
+            if num in (subvertpy.ERR_RA_DAV_REQUEST_FAILED, 
+                       subvertpy.ERR_RA_DAV_NOT_VCC):
+                raise bzr_errors.NotBranchError(transport.base)
+            if num == ERR_XML_MALFORMED:
+                # This *could* be an indication of an actual corrupt 
+                # svn server, but usually it just means a broken 
+                # xml page
                 raise bzr_errors.NotBranchError(transport.base)
             raise
 
@@ -97,9 +146,11 @@ class SvnRemoteFormat(BzrDirFormat):
 
     def initialize_on_transport(self, transport):
         """See BzrDir.initialize_on_transport()."""
+        self._check_versions()
         from bzrlib.plugins.svn.transport import get_svn_ra_transport
         from bzrlib.transport.local import LocalTransport
-        from bzrlib.plugins.svn import repos
+        import os
+        from subvertpy import repos
 
         if not isinstance(transport, LocalTransport):
             raise NotImplementedError(self.initialize, 
@@ -114,39 +165,30 @@ class SvnRemoteFormat(BzrDirFormat):
         os.chmod(revprop_hook, os.stat(revprop_hook).st_mode | 0111)
         return self.open(get_svn_ra_transport(transport), _found=True)
 
-    def is_supported(self):
-        """See BzrDir.is_supported()."""
-        return True
 
-
-class SvnWorkingTreeDirFormat(BzrDirFormat):
+class SvnWorkingTreeDirFormat(SvnControlFormat):
     """Working Tree implementation that uses Subversion working copies."""
-    _lock_class = TransportLock
-
-    def __init__(self):
-        super(SvnWorkingTreeDirFormat, self).__init__()
-        from bzrlib.plugins.svn.repository import SvnRepositoryFormat
-        self.repository_format = SvnRepositoryFormat()
 
     @classmethod
     def probe_transport(klass, transport):
         from bzrlib.transport.local import LocalTransport
-        from bzrlib.plugins.svn import wc
         format = klass()
 
         if isinstance(transport, LocalTransport) and \
-            transport.has(wc.get_adm_dir()):
+            transport.has(".svn"):
             return format
 
         raise bzr_errors.NotBranchError(path=transport.base)
 
     def _open(self, transport):
+        self._check_versions()
         from bzrlib.plugins.svn.workingtree import SvnCheckout
-        from bzrlib.plugins.svn import core
+        from bzrlib.plugins.svn import errors
+        import subvertpy
         try:
             return SvnCheckout(transport, self)
-        except core.SubversionException, (_, num):
-            if num in (errors.ERR_RA_LOCAL_REPOS_OPEN_FAILED,):
+        except subvertpy.SubversionException, (_, num):
+            if num in (subvertpy.ERR_RA_LOCAL_REPOS_OPEN_FAILED,):
                 raise errors.NoSvnRepositoryPresent(transport.base)
             raise
 
@@ -157,10 +199,12 @@ class SvnWorkingTreeDirFormat(BzrDirFormat):
         return 'Subversion Local Checkout'
 
     def initialize_on_transport(self, transport):
-        raise UninitializableFormat(self)
+        raise bzr_errors.UninitializableFormat(self)
 
     def get_converter(self, format=None):
         """See BzrDirFormat.get_converter()."""
+        self._check_versions()
         if format is None:
             format = get_rich_root_format()
-        raise NotImplementedError(self.get_converter)
+        from bzrlib.plugins.svn.workingtree import SvnCheckoutConverter
+        return SvnCheckoutConverter(format)
