@@ -1,8 +1,8 @@
-# Copyright (C) 2006 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,88 +15,72 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Cache of the Subversion history log."""
 
-from bzrlib import urlutils
+import subvertpy
+
+from bzrlib import (
+    ui,
+    urlutils,
+    )
 from bzrlib.errors import NoSuchRevision
-import bzrlib.ui as ui
 
-from bzrlib.plugins.svn import changes, core
+from bzrlib.plugins.svn import changes
 from bzrlib.plugins.svn.cache import CacheTable
-from bzrlib.plugins.svn.core import SubversionException
-from bzrlib.plugins.svn.errors import ERR_FS_NO_SUCH_REVISION, ERR_FS_NOT_FOUND, ERR_FS_NOT_DIRECTORY
-from bzrlib.plugins.svn.ra import DIRENT_KIND
 from bzrlib.plugins.svn.transport import SvnRaTransport
+from bzrlib.plugins.svn.util import lazy_dict
 
-class lazy_dict(object):
-    def __init__(self, initial, create_fn, *args):
-        self.initial = initial
-        self.create_fn = create_fn
-        self.args = args
-        self.dict = None
+# Maximum number of extra revisions to fetch in caching logwalker
+MAX_OVERHEAD_FETCH = 1000
 
-    def _ensure_init(self):
-        if self.dict is None:
-            self.dict = self.create_fn(*self.args)
-            self.create_fn = None
+def iter_changes(paths, from_revnum, to_revnum, get_revision_paths, 
+    revprop_list, limit=0, pb=None):
+    ascending = (to_revnum > from_revnum)
 
-    def __len__(self):
-        self._ensure_init()
-        return len(self.dict)
+    revnum = from_revnum
 
-    def __getitem__(self, key):
-        if key in self.initial:
-            return self.initial.__getitem__(key)
-        self._ensure_init()
-        return self.dict.__getitem__(key)
+    if paths is None:
+        path = ""
+    else:
+        assert len(paths) == 1
+        path = paths[0].strip("/")
 
-    def __delitem__(self, key):
-        self._ensure_init()
-        return self.dict.__delitem__(key)
+    assert from_revnum >= to_revnum or path == ""
 
-    def __setitem__(self, key, value):
-        self._ensure_init()
-        return self.dict.__setitem__(key, value)
+    i = 0
 
-    def __contains__(self, key):
-        if key in self.initial:
-            return True
-        self._ensure_init()
-        return self.dict.__contains__(key)
+    while ((not ascending and revnum >= to_revnum) or
+           (ascending and revnum <= to_revnum)):
+        if pb is not None:
+            if ascending:
+                pb.update("determining changes", revnum-from_revnum, to_revnum-from_revnum)
+            else:
+                pb.update("determining changes", from_revnum-revnum, from_revnum-to_revnum)
+        if not (revnum > 0 or path == ""):
+            raise AssertionError("Inconsistent path,revnum: %r,%r" % (revnum, path))
+        revpaths = get_revision_paths(revnum)
 
-    def get(self, key, default=None):
-        if key in self.initial:
-            return self.initial[key]
-        self._ensure_init()
-        return self.dict.get(key, default)
+        if ascending:
+            next = (path, revnum+1)
+        else:
+            next = changes.find_prev_location(revpaths, path, revnum)
 
-    def has_key(self, key):
-        if self.initial.has_key(key):
-            return True
-        self._ensure_init()
-        return self.dict.has_key(key)
+        revprops = revprop_list(revnum)
 
-    def keys(self):
-        self._ensure_init()
-        return self.dict.keys()
+        if path == "" or changes.changes_path(revpaths, path, True):
+            assert isinstance(revnum, int)
+            yield (revpaths, revnum, revprops)
+            i += 1
+            if limit != 0 and i == limit:
+                break
 
-    def values(self):
-        self._ensure_init()
-        return self.dict.values()
+        if next is None:
+            break
 
-    def items(self):
-        self._ensure_init()
-        return self.dict.items()
+        assert (ascending and next[1] > revnum) or \
+               (not ascending and next[1] < revnum)
+        (path, revnum) = next
+        assert isinstance(path, str)
+        assert isinstance(revnum, int)
 
-    def __iter__(self):
-        self._ensure_init()
-        return self.dict.__iter__()
-
-    def __repr__(self):
-        self._ensure_init()
-        return repr(self.dict)
-
-    def __eq__(self, other):
-        self._ensure_init()
-        return self.dict.__eq__(other)
 
 
 class LogCache(CacheTable):
@@ -120,7 +104,7 @@ class LogCache(CacheTable):
             create unique index if not exists revprop_rev_name on revprop(rev, name);
             create unique index if not exists revinfo_rev on revinfo(rev);
         """)
-        self._commit_interval = 1000
+        self._commit_interval = 5000
     
     def find_latest_change(self, path, revnum):
         if path == "":
@@ -134,40 +118,6 @@ class LogCache(CacheTable):
                      and action in ('A', 'R')))
         """, (revnum, path, path + "/*", path)).fetchone()[0]
 
-    def path_added(self, path, from_revnum, to_revnum):
-        """Determine whether a path was recently added. 
-        
-        It returns the latest revision number before to_revnum in which the 
-        path was added or replaced, or None if the path was neither added nor
-        replaced after from_revnum but before to_revnum.
-        """
-        assert from_revnum < to_revnum
-        return self.cachedb.execute("""
-            select max(rev) from changed_path
-            where rev > ? and rev <= ?
-            and (?=path or ? glob (path || '/*'))
-            and action in ('A', 'R')
-        """, (from_revnum, to_revnum, path, path)).fetchone()[0]
-    
-    def get_change(self, path, revnum):
-        return self.cachedb.execute("""
-            select action, copyfrom_path, copyfrom_rev
-            from changed_path
-            where path=? and rev=?
-        """, (path, revnum)).fetchone()
-
-    def get_previous(self, path, revnum):
-        """Determine the change that created the given path or its
-        nearest ancestor, in order to determine where it came from."""
-        return self.cachedb.execute("""
-            select path, action, copyfrom_path, copyfrom_rev
-            from changed_path
-            where rev=? and action in ('A', 'R')
-            and (path=? or path='' or ? glob (path || '/*'))
-            order by path desc
-            limit 1
-        """, (revnum, path, path)).fetchone()
-
     def get_revision_paths(self, revnum):
         """Return all history information for a given revision number.
         
@@ -181,16 +131,6 @@ class LogCache(CacheTable):
             paths[p.encode("utf-8")] = (act, cf, cr)
         return paths
     
-    def changes_path(self, path, revnum):
-        """Check whether a path was changed in a particular revision:
-
-        :param path: path to check for
-        :param revnum: revision number to fetch
-        """
-        if path == '':
-            return self.cachedb.execute("select count(*) from changed_path where rev=?", (revnum,)).fetchone()[0] > 0
-        return self.cachedb.execute("select count(*) from changed_path where rev=? and (path=? or path glob (? || '/*'))", (revnum, path, path)).fetchone()[0] > 0
-
     def insert_path(self, rev, path, action, copyfrom_path=None, copyfrom_rev=-1):
         """Insert new history information into the cache.
         
@@ -202,8 +142,13 @@ class LogCache(CacheTable):
         """
         assert action in ("A", "R", "D", "M")
         assert not path.startswith("/")
+        assert isinstance(path, unicode)
+        assert copyfrom_path is None or isinstance(copyfrom_path, unicode)
         self.cachedb.execute("replace into changed_path (rev, path, action, copyfrom_path, copyfrom_rev) values (?, ?, ?, ?, ?)", (rev, path, action, copyfrom_path, copyfrom_rev))
 
+    def drop_revprops(self, revnum):
+        self.cachedb.execute("update revinfo set all_revprops = 0 where rev = ?", (revnum,))
+
     def get_revprops(self, revnum):
         """Retrieve all the cached revision properties.
 
@@ -222,12 +167,13 @@ class LogCache(CacheTable):
         :param name: Name of the revision property.
         :param value: Contents of the revision property.
         """
-        self.cachedb.execute("replace into revprop (rev, name, value) values (?, ?, ?)", (rev, name.decode("utf-8"), value.decode("utf-8")))
+        assert isinstance(name, str) and isinstance(value, str)
+        self.cachedb.execute("replace into revprop (rev, name, value) values (?, ?, ?)", (rev, name.decode("utf-8", "replace"), value.decode("utf-8", "replace")))
     
     def insert_revprops(self, revision, revprops):
         if revprops is None:
             return
-        for k, v in revprops.items():
+        for k, v in revprops.iteritems():
             self.insert_revprop(revision, k, v)
 
     def has_all_revprops(self, revnum):
@@ -235,7 +181,10 @@ class LogCache(CacheTable):
 
         :param revnum: Revision number of the revision.
         """
-        return self.cachedb.execute("select all_revprops from revinfo where rev = ?", (revnum,)).fetchone()[0]
+        row = self.cachedb.execute("select all_revprops from revinfo where rev = ?", (revnum,)).fetchone()
+        if row is None:
+            return False
+        return row[0]
 
     def insert_revinfo(self, rev, all_revprops):
         """Insert metadata for a revision.
@@ -256,13 +205,14 @@ class LogCache(CacheTable):
 
 class CachingLogWalker(CacheTable):
     """Subversion log browser."""
+
     def __init__(self, actual, cache_db=None):
         self.cache = LogCache(cache_db)
         self.actual = actual
+        self.quick_revprops = actual.quick_revprops
         self._transport = actual._transport
-        self.find_children = actual.find_children
-
         self.saved_revnum = self.cache.last_revnum()
+        self._latest_revnum = None
 
     def find_latest_change(self, path, revnum):
         """Find latest revision that touched path.
@@ -272,7 +222,7 @@ class CachingLogWalker(CacheTable):
         """
         assert isinstance(path, str)
         assert isinstance(revnum, int) and revnum >= 0
-        self.fetch_revisions(revnum)
+        self._fetch_revisions(revnum)
 
         self.mutter("latest change: %r:%r", path, revnum)
         revnum = self.cache.find_latest_change(path.strip("/"), revnum)
@@ -291,112 +241,23 @@ class CachingLogWalker(CacheTable):
             where paths is a dictionary with all changes that happened 
             in revnum.
         """
-        assert from_revnum >= 0 and to_revnum >= 0
-
-        ascending = (to_revnum > from_revnum)
-
-        revnum = from_revnum
-
+        assert from_revnum >= 0 and to_revnum >= 0, "%r -> %r" % (from_revnum, to_revnum)
         self.mutter("iter changes %r->%r (%r)", from_revnum, to_revnum, paths)
+        self._fetch_revisions(max(from_revnum, to_revnum), pb=pb)
 
-        if paths is None:
-            path = ""
-        else:
-            assert len(paths) == 1
-            path = paths[0].strip("/")
+        return iter(iter_changes(paths, from_revnum, to_revnum, 
+            self.get_revision_paths, self.revprop_list, limit, pb))
 
-        assert from_revnum >= to_revnum or path == ""
-
-        self.fetch_revisions(max(from_revnum, to_revnum), pb=pb)
-        i = 0
-
-        while ((not ascending and revnum >= to_revnum) or
-               (ascending and revnum <= to_revnum)):
-            if pb is not None:
-                if ascending:
-                    pb.update("determining changes", revnum, to_revnum)
-                else:
-                    pb.update("determining changes", from_revnum-revnum, from_revnum)
-            assert revnum > 0 or path == "", "Inconsistent path,revnum: %r,%r" % (revnum, path)
-            revpaths = self._get_revision_paths(revnum)
-
-            if ascending:
-                next = (path, revnum+1)
-            else:
-                next = changes.find_prev_location(revpaths, path, revnum)
-
-            revprops = lazy_dict({}, self.revprop_list, revnum)
-
-            if changes.changes_path(revpaths, path, True):
-                assert isinstance(revnum, int)
-                yield (revpaths, revnum, revprops)
-                i += 1
-                if limit != 0 and i == limit:
-                    break
-
-            if next is None:
-                break
-
-            assert (ascending and next[1] > revnum) or \
-                   (not ascending and next[1] < revnum)
-            (path, revnum) = next
-            assert isinstance(path, str)
-            assert isinstance(revnum, int)
-
-    def get_previous(self, path, revnum):
-        """Return path,revnum pair specified pair was derived from.
-
-        :param path:  Path to check
-        :param revnum:  Revision to check
-        """
-        assert revnum >= 0
-        self.fetch_revisions(revnum)
-        self.mutter("get previous %r:%r", path, revnum)
-        if path == "":
-            if revnum == 0:
-                return (None, -1)
-            return (path, revnum - 1)
-        row = self.cache.get_previous(path, revnum)
-        if row is None:
-            return (path, revnum-1)
-        (branch_path, action, copyfrom_path, copyfrom_rev) = row
-        branch_path = branch_path.encode('utf-8')
-        if copyfrom_path is not None:
-            copyfrom_path = copyfrom_path.encode('utf-8')
-        assert path == branch_path or path.startswith(branch_path + "/")
-        if copyfrom_rev == -1:
-            assert path == branch_path
-            return (None, -1) # newly added
-        return (copyfrom_path + path[len(branch_path):], copyfrom_rev)
-
-    def _get_revision_paths(self, revnum):
+    def get_revision_paths(self, revnum):
         if revnum == 0:
-            return {'': ('A', None, -1)}
-        self.fetch_revisions(revnum)
+            return changes.REV0_CHANGES
+        self._fetch_revisions(revnum)
 
         return self.cache.get_revision_paths(revnum)
 
-    def get_change(self, path, revnum):
-        """Return action,copyfrom_path,copyfrom_revnum tuple for given
-        path, or None if no modification occurred in given revision.
-
-        :param path:  Path to check
-        :param revnum:  Revision to check
-        """
-        assert revnum >= 0
-        self.fetch_revisions(revnum)
-        
-        self.mutter("get previous %r:%r", path, revnum)
-        if revnum == 0:
-            if path == "":
-                return ('A', None, -1)
-            return None
-        return self.cache.get_change(path, revnum)
-
     def revprop_list(self, revnum):
-        self.mutter("revprop list: %r", revnum)
-
-        self.fetch_revisions(revnum)
+        self.mutter('revprop list: %d' % revnum)
+        self._fetch_revisions(revnum)
 
         if revnum > 0:
             has_all_revprops = self.cache.has_all_revprops(revnum)
@@ -417,11 +278,7 @@ class CachingLogWalker(CacheTable):
         self.cache.commit()
         return revprops
 
-    def changes_path(self, path, revnum):
-        self.fetch_revisions(revnum)
-        return self.cache.changes_path(path, revnum)
-
-    def fetch_revisions(self, to_revnum=None, pb=None):
+    def _fetch_revisions(self, to_revnum=None, pb=None):
         """Fetch information about all revisions in the remote repository
         until to_revnum.
 
@@ -430,9 +287,12 @@ class CachingLogWalker(CacheTable):
         assert isinstance(self.saved_revnum, int)
         if to_revnum <= self.saved_revnum:
             return
-        latest_revnum = self.actual._transport.get_latest_revnum()
-        assert isinstance(latest_revnum, int)
-        to_revnum = max(latest_revnum, to_revnum)
+
+        # Try to fetch log data in lumps, if possible.
+        if self._latest_revnum is None:
+            self._latest_revnum = self.actual._transport.get_latest_revnum()
+        assert isinstance(self._latest_revnum, int)
+        to_revnum = max(min(self._latest_revnum, to_revnum+MAX_OVERHEAD_FETCH), to_revnum)
 
         # Subversion 1.4 clients and servers can only deliver a limited set of revprops
         if self._transport.has_capability("log-revprops"):
@@ -447,9 +307,9 @@ class CachingLogWalker(CacheTable):
             for p in orig_paths:
                 copyfrom_path = orig_paths[p][1]
                 if copyfrom_path is not None:
-                    copyfrom_path = copyfrom_path.strip("/")
+                    copyfrom_path = copyfrom_path.strip("/").decode("utf-8")
 
-                self.cache.insert_path(revision, p.strip("/"), orig_paths[p][0], copyfrom_path, orig_paths[p][2])
+                self.cache.insert_path(revision, p.strip("/").decode("utf-8"), orig_paths[p][0], copyfrom_path, orig_paths[p][2])
             self.cache.insert_revprops(revision, revprops)
             self.cache.insert_revinfo(revision, todo_revprops is None)
             self.saved_revnum = revision
@@ -466,8 +326,8 @@ class CachingLogWalker(CacheTable):
             nested_pb.update('fetching svn revision info', 0, to_revnum)
             try:
                 self.actual._transport.get_log(rcvr, [""], self.saved_revnum, to_revnum, 0, True, True, False, todo_revprops)
-            except SubversionException, (_, num):
-                if num == ERR_FS_NO_SUCH_REVISION:
+            except subvertpy.SubversionException, (_, num):
+                if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
                     raise NoSuchRevision(branch=self, 
                         revision="Revision number %d" % to_revnum)
                 raise
@@ -477,10 +337,12 @@ class CachingLogWalker(CacheTable):
         self.cache.commit()
 
 
-def struct_revpaths_to_tuples(changed_paths):
+def strip_slashes(changed_paths):
+    if changed_paths is None:
+        return {}
     assert isinstance(changed_paths, dict)
     revpaths = {}
-    for k, (action, copyfrom_path, copyfrom_rev) in changed_paths.items():
+    for k, (action, copyfrom_path, copyfrom_rev) in changed_paths.iteritems():
         if copyfrom_path is None:
             copyfrom_path = None
         else:
@@ -491,6 +353,7 @@ def struct_revpaths_to_tuples(changed_paths):
 
 class LogWalker(object):
     """Easy way to access the history of a Subversion repository."""
+
     def __init__(self, transport, limit=None):
         """Create a new instance.
 
@@ -499,6 +362,7 @@ class LogWalker(object):
         assert isinstance(transport, SvnRaTransport)
 
         self._transport = transport
+        self.quick_revprops = (self._transport.has_capability("log-revprops") == True)
 
     def find_latest_change(self, path, revnum):
         """Find latest revision that touched path.
@@ -511,11 +375,11 @@ class LogWalker(object):
 
         try:
             return self._transport.iter_log([path], revnum, 0, 2, True, False, False, []).next()[1]
-        except SubversionException, (_, num):
-            if num == ERR_FS_NO_SUCH_REVISION:
+        except subvertpy.SubversionException, (_, num):
+            if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
                 raise NoSuchRevision(branch=self, 
                     revision="Revision number %d" % revnum)
-            if num == ERR_FS_NOT_FOUND:
+            if num == subvertpy.ERR_FS_NOT_FOUND:
                 return None
             raise
 
@@ -545,11 +409,11 @@ class LogWalker(object):
 
             for (changed_paths, revnum, known_revprops, has_children) in iterator:
                 if pb is not None:
-                    pb.update("determining changes", from_revnum-revnum, from_revnum)
+                    pb.update("determining changes", from_revnum-revnum, from_revnum-to_revnum)
                 if revnum == 0 and changed_paths is None:
-                    revpaths = {"": ('A', None, -1)}
+                    revpaths = changes.REV0_CHANGES
                 elif isinstance(changed_paths, dict):
-                    revpaths = struct_revpaths_to_tuples(changed_paths)
+                    revpaths = strip_slashes(changed_paths)
                 else:
                     revpaths = {}
                 if todo_revprops is None:
@@ -557,13 +421,13 @@ class LogWalker(object):
                 else:
                     revprops = lazy_dict(known_revprops, self._transport.revprop_list, revnum)
                 yield (revpaths, revnum, revprops)
-        except SubversionException, (_, num):
-            if num == ERR_FS_NO_SUCH_REVISION:
+        except subvertpy.SubversionException, (_, num):
+            if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
                 raise NoSuchRevision(branch=self, 
                     revision="Revision number %d" % from_revnum)
             raise
 
-    def _get_revision_paths(self, revnum):
+    def get_revision_paths(self, revnum):
         """Obtain dictionary with all the changes in a particular revision.
 
         :param revnum: Subversion revision number
@@ -572,74 +436,36 @@ class LogWalker(object):
         """
         # To make the existing code happy:
         if revnum == 0:
-            return {'': ('A', None, -1)}
+            return changes.REV0_CHANGES
 
         try:
-            return struct_revpaths_to_tuples(
+            return strip_slashes(
                 self._transport.iter_log([""], revnum, revnum, 1, True, True, False, []).next()[0])
-        except SubversionException, (_, num):
-            if num == ERR_FS_NO_SUCH_REVISION:
+        except subvertpy.SubversionException, (_, num):
+            if num == subvertpy.ERR_FS_NO_SUCH_REVISION:
                 raise NoSuchRevision(branch=self, 
                     revision="Revision number %d" % revnum)
             raise
 
-    def changes_path(self, path, revnum):
-        return self._get_revision_paths(revnum).has_key(path)
-        
-    def find_children(self, path, revnum):
-        """Find all children of path in revnum.
 
-        :param path:  Path to check
-        :param revnum:  Revision to check
-        """
-        assert isinstance(path, str), "invalid path"
-        path = path.strip("/")
-        conn = self._transport.connections.get(self._transport.get_svn_repos_root())
-        results = []
-        unchecked_dirs = set([path])
-        try:
-            while len(unchecked_dirs) > 0:
-                nextp = unchecked_dirs.pop()
-                try:
-                    dirents = conn.get_dir(nextp, revnum, DIRENT_KIND)[0]
-                except SubversionException, (_, num):
-                    if num == ERR_FS_NOT_DIRECTORY:
-                        continue
-                    raise
-                for k, v in dirents.items():
-                    childp = urlutils.join(nextp, k)
-                    if v['kind'] == core.NODE_DIR:
-                        unchecked_dirs.add(childp)
-                    results.append(childp)
-        finally:
-            self._transport.connections.add(conn)
-        return results
+class DictBasedLogWalker(object):
 
-    def get_previous(self, path, revnum):
-        """Return path,revnum pair specified pair was derived from.
+    def __init__(self, paths, revprops):
+        self.paths = paths
+        self.revprops = revprops
 
-        :param path:  Path to check
-        :param revnum:  Revision to check
-        """
-        assert revnum >= 0
+    def iter_changes(self, paths, from_revnum, to_revnum=0, limit=0, pb=None):
+        return iter(iter_changes(paths, from_revnum, to_revnum, 
+            self.get_revision_paths, self.revprop_list, limit, pb))
+
+    def get_revision_paths(self, revnum):
         if revnum == 0:
-            return (None, -1)
+            return changes.REV0_CHANGES
 
-        try:
-            paths = struct_revpaths_to_tuples(self._transport.iter_log([path], revnum, revnum, 1, True, False, False, []).next()[0])
-        except SubversionException, (_, num):
-            if num == ERR_FS_NO_SUCH_REVISION:
-                raise NoSuchRevision(branch=self, 
-                    revision="Revision number %d" % revnum)
-            raise
+        return self.paths.get(revnum, {})
 
-        if not path in paths:
-            return (None, -1)
-
-        if paths[path][2] == -1:
-            if paths[path][0] in ('A', 'R'):
-                return (None, -1)
-            return (path, revnum-1)
-
-        return (paths[path][1], paths[path][2])
+    def revprop_list(self, revnum):
+        if revnum == 0:
+            return {}
+        return self.revprops.get(revnum, {})
 
