@@ -1,8 +1,8 @@
-# Copyright (C) 2006 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,14 +15,42 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Branching scheme implementations."""
 
-from bzrlib import ui, urlutils
+import base64
+import bz2
+from subvertpy import properties
+import urllib
+
+from bzrlib import (
+    ui,
+    urlutils,
+    )
 from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
 
-from base64 import urlsafe_b64decode, urlsafe_b64encode
-from bzrlib.plugins.svn.errors import InvalidSvnBranchPath
-from bzrlib.plugins.svn import properties
-import bz2
+from bzrlib.plugins.svn.layout.guess import (
+    GUESS_SAMPLE_SIZE,
+    find_commit_paths,
+    )
+from bzrlib.plugins.svn.layout.standard import (
+    CustomLayout,
+    RootLayout,
+    TrunkLayout,
+    )
+from bzrlib.plugins.svn.errors import LayoutUnusable
+
+
+class InvalidSvnBranchPath(BzrError):
+    """Error raised when a path was specified that is not a child of or itself
+    a valid branch path in the current branching scheme."""
+    _fmt = """%(path)s is not a valid Subversion branch path in the current 
+repository layout. See 'bzr help svn-repository-layout' for details."""
+
+    def __init__(self, path, layout):
+        assert isinstance(path, str)
+        BzrError.__init__(self)
+        self.path = urllib.quote(path)
+        self.layout = layout
+
 
 class BranchingScheme(object):
     """ Divides SVN repository data up into branches. Since there
@@ -44,14 +72,14 @@ class BranchingScheme(object):
         """
         raise NotImplementedError
 
-    def get_tag_path(self, name):
+    def get_tag_path(self, name, project=""):
         """Find the path for a tag.
 
         :param name: Tag name.
         """
         raise NotImplementedError
 
-    def get_branch_path(self, name):
+    def get_branch_path(self, name, project=""):
         """Find the path for a named branch.
 
         :param name: Branch name.
@@ -139,11 +167,11 @@ def parse_list_scheme_text(text):
 
 
 def prop_name_unquote(text):
-    return urlsafe_b64decode(text.replace(".", "="))
+    return base64.urlsafe_b64decode(text.replace(".", "="))
 
 
 def prop_name_quote(text):
-    return urlsafe_b64encode(text).replace("=", ".")
+    return base64.urlsafe_b64encode(text).replace("=", ".")
 
 
 class ListBranchingScheme(BranchingScheme):
@@ -197,8 +225,11 @@ class ListBranchingScheme(BranchingScheme):
         raise InvalidSvnBranchPath(path, self)
 
     def __eq__(self, other):
-        return (self.branch_list == other.branch_list and \
-                self.tag_list == other.tag_list)
+        return (self.branch_list == getattr(other, "branch_list", None) and \
+                self.tag_list == getattr(other, "tag_list", None))
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
     def to_lines(self):
         return self.branch_list
@@ -411,28 +442,6 @@ class SingleBranchingSchemev0(SingleBranchingScheme):
         return "single-%s" % self.path
 
 
-def _find_common_prefix(paths):
-    prefix = ""
-    # Find a common prefix
-    parts = paths[0].split("/")
-    for i in range(len(parts)+1):
-        for j in paths:
-            if j.split("/")[:i] != parts[:i]:
-                return prefix
-        prefix = "/".join(parts[:i])
-    return prefix
-
-
-def find_commit_paths(changed_paths):
-    """Find the commit-paths used in a bunch of revisions.
-
-    :param changed_paths: List of changed_paths (dictionary with path -> action)
-    :return: List of potential commit paths.
-    """
-    for changes in changed_paths:
-        yield _find_common_prefix(changes.keys())
-
-
 def guess_scheme_from_branch_path(relpath):
     """Try to guess the branching scheme from a branch path.
 
@@ -536,29 +545,26 @@ def scheme_from_branch_list(branch_list):
     return ListBranchingScheme(branch_list) 
 
 
-help_schemes = """Subversion Branching Schemes
+def scheme_from_layout(layout):
+    if getattr(layout, "scheme", None) is not None:
+        return layout.scheme
+    if isinstance(layout, TrunkLayout):
+        return TrunkBranchingScheme(layout.level or 0)
+    if isinstance(layout, RootLayout):
+        return NoBranchingScheme()
+    if isinstance(layout, CustomLayout) and len(layout.branches) == 1:
+        return SingleBranchingScheme(layout.branches[0])
+    raise LayoutUnusable(layout, self)
 
-Subversion is basically a versioned file system. It does not have 
-any notion of branches and what is a branch in Subversion is therefor
-up to the user. 
 
-In order for Bazaar to access a Subversion repository it has to know 
-what paths to consider branches. It does this by using so-called branching 
-schemes. When you connect to a repository for the first time, Bazaar
-will try to determine the branching scheme to use using some simple 
-heuristics. It is always possible to change the branching scheme it should 
-use later.
+def repository_guess_scheme(repository, last_revnum, branch_path=None):
+    pb = ui.ui_factory.nested_progress_bar()
+    try:
+        (guessed_scheme, scheme) = guess_scheme_from_history(
+            repository._log.iter_changes(None, last_revnum, max(0, last_revnum-GUESS_SAMPLE_SIZE)), last_revnum, branch_path)
+    finally:
+        pb.finished()
+    mutter("Guessed branching scheme: %r, guess scheme to use: %r" % 
+            (guessed_scheme, scheme))
+    return (guessed_scheme, scheme)
 
-There are some conventions in use in Subversion for repository layouts. 
-The most common one is probably the trunk/branches/tags 
-layout, where the repository contains a "trunk" directory with the main 
-development branch, other branches in a "branches" directory and tags as 
-subdirectories of a "tags" directory. This branching scheme is named 
-"trunk" in Bazaar.
-
-Another option is simply having just one branch at the root of the repository. 
-This scheme is called "none" by Bazaar.
-
-The branching scheme bzr-svn should use for a repository can be set in the 
-configuration file ~/.bazaar/subversion.conf.
-"""
