@@ -1,8 +1,8 @@
-# Copyright (C) 2005-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2005-2009 Jelmer Vernooij <jelmer@samba.org>
  
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -19,9 +19,13 @@ from bzrlib.trace import mutter
 from bzrlib.revision import NULL_REVISION
 from bzrlib.versionedfile import ConstantMapper
 
+from bzrlib.plugins.svn.cache import CacheTable
+
 class DiskCachingParentsProvider(object):
-    def __init__(self, actual, cachetransport):
-        self._cache = ParentsCache(cachetransport)
+    """Parents provider that caches parents in a SQLite database."""
+
+    def __init__(self, actual, cachedb=None):
+        self._cache = ParentsCache(cachedb)
         self.actual = actual
 
     def get_parent_map(self, keys):
@@ -33,9 +37,9 @@ class DiskCachingParentsProvider(object):
                 todo.add(k)
             else:
                 ret[k] = parents
-        if len(todo):
+        if len(todo) > 0:
             newfound = self.actual.get_parent_map(todo)
-            for revid, parents in newfound.items():
+            for revid, parents in newfound.iteritems():
                 if revid == NULL_REVISION:
                     continue
                 self._cache.insert_parents(revid, parents)
@@ -43,25 +47,29 @@ class DiskCachingParentsProvider(object):
         return ret
 
 
-PARENTMAP_VERSION = 1
+class ParentsCache(CacheTable):
 
-
-class ParentsCache(object):
-
-    def __init__(self, cache_transport):
-        mapper = ConstantMapper("parentmap-v%d" % PARENTMAP_VERSION)
-        self.parentmap_knit = make_file_factory(True, mapper)(cache_transport)
+    def _create_table(self):
+        self.cachedb.executescript("""
+        create table if not exists parent (rev text, parent text, idx int);
+        create unique index if not exists rev_parent_idx on parent (rev, idx);
+        create unique index if not exists rev_parent on parent (rev, parent);
+        """)
+        self._commit_interval = 200
 
     def insert_parents(self, revid, parents):
         if "cache" in debug.debug_flags:
             mutter('insert parents: %r -> %r', revid, parents)
-        self.parentmap_knit.add_lines((revid,), [(r, ) for r in parents], [])
+        if len(parents) == 0:
+            self.cachedb.execute("replace into parent (rev, parent, idx) values (?, NULL, -1)", (revid,))
+        else:
+            for i, p in enumerate(parents):
+                self.cachedb.execute("replace into parent (rev, parent, idx) values (?, ?, ?)", (revid, p, i))
 
     def lookup_parents(self, revid):
         if "cache" in debug.debug_flags:
             mutter('lookup parents: %r', revid)
-        try:
-            return tuple([r for (r,) in self.parentmap_knit.get_parent_map([(revid,)])[(revid,)]])
-        except KeyError:
+        rows = self.cachedb.execute("select parent from parent where rev = ? order by idx", (revid, )).fetchall()
+        if len(rows) == 0:
             return None
-
+        return tuple([row[0].encode("utf-8") for row in rows if row[0] is not None])
