@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,18 +15,28 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Blackbox tests."""
 
+import os
+import sys
+
 import bzrlib.gpg
 from bzrlib.repository import Repository
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.tests import KnownFailure
 
-from bzrlib.plugins.svn.mapping3 import BzrSvnMappingv3FileProps
-from bzrlib.plugins.svn.mapping3.scheme import NoBranchingScheme
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.convert import load_dumpfile
+from bzrlib.plugins.svn.layout.standard import RootLayout
+from bzrlib.plugins.svn.tests import SubversionTestCase
 
-import os, sys
+class TestBranch(SubversionTestCase, ExternalBase):
+
+    def setUp(self):
+        SubversionTestCase.setUp(self)
+        SubversionTestCase.tearDown(self)
+        ExternalBase.setUp(self)
+
+    def tearDown(self):
+        ExternalBase.tearDown(self)
 
-class TestBranch(ExternalBase, TestCaseWithSubversionRepository):
     def test_branch_empty(self):
         repos_url = self.make_repository('d')
         self.run_bzr("branch %s dc" % repos_url)
@@ -41,6 +51,12 @@ class TestBranch(ExternalBase, TestCaseWithSubversionRepository):
         self.commit_something(repos_url)
         self.run_bzr("branch %s dc" % repos_url)
         self.check_output("2\n", "revno de")
+
+    def test_branch_onerev_stacked(self):
+        repos_url = self.make_client('d', 'de')
+        self.commit_something(repos_url)
+        self.run_bzr("branch --stacked %s dc" % repos_url)
+        self.check_output("2\n", "revno de")
         
     def test_log_empty(self):
         repos_url = self.make_repository('d')
@@ -67,6 +83,23 @@ class TestBranch(ExternalBase, TestCaseWithSubversionRepository):
         self.run_bzr("push -d dc %s" % repos_url)
         self.check_output("", "status dc")
 
+    def test_missing(self):
+        repos_url = self.make_repository('d')
+        
+        self.run_bzr("init dc")
+
+        os.chdir("dc")
+        output, err = self.run_bzr("missing %s" % repos_url, retcode=1)
+        self.assertContainsRe(output, "You are missing 1 revision\\(s\\):")
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir('trunk')
+        trunk.add_file("trunk/foo").modify()
+        dc.close()
+
+        output, err = self.run_bzr("missing %s" % repos_url, retcode=1)
+        self.assertContainsRe(output, "You are missing 2 revision\\(s\\):")
+
     def test_push_overwrite(self):
         repos_url = self.make_repository('d')
         
@@ -82,6 +115,12 @@ class TestBranch(ExternalBase, TestCaseWithSubversionRepository):
         self.run_bzr("push --overwrite -d dc %s/trunk" % repos_url)
         self.check_output("", "status dc")
 
+    def test_dpush_empty(self):
+        repos_url = self.make_repository('dp')
+        self.run_bzr("init --rich-root-pack dc")
+        os.chdir("dc")
+        self.run_bzr("dpush %s" % repos_url)
+
     def test_dpush(self):
         repos_url = self.make_repository('d')
         
@@ -111,7 +150,6 @@ class TestBranch(ExternalBase, TestCaseWithSubversionRepository):
         self.check_output("", "status dc")
 
     def test_dpush_wt_diff(self):
-        raise KnownFailure
         repos_url = self.make_repository('d')
         
         dc = self.get_commit_editor(repos_url)
@@ -276,19 +314,31 @@ Node-copyfrom-path: x
 
 
 """)
-        self.check_output("", 'svn-import --scheme=none %s dc' % filename)
+        self.check_output("", 'svn-import --layout=none %s dc' % filename)
+        load_dumpfile(filename, "realrepo")
+        svnrepo = Repository.open("realrepo")
+        self.assertEquals(uuid, svnrepo.uuid)
+        svnrepo.set_layout(RootLayout())
+        mapping = svnrepo.get_mapping()
         newrepos = Repository.open("dc")
-        mapping = BzrSvnMappingv3FileProps(NoBranchingScheme())
         self.assertTrue(newrepos.has_revision(
-            mapping.generate_revision_id(uuid, 5, "")))
+            mapping.revision_id_foreign_to_bzr((uuid, "", 5))))
         self.assertTrue(newrepos.has_revision(
-            mapping.generate_revision_id(uuid, 1, "")))
+            mapping.revision_id_foreign_to_bzr((uuid, "", 1))))
         inv1 = newrepos.get_inventory(
-                mapping.generate_revision_id(uuid, 1, ""))
+                mapping.revision_id_foreign_to_bzr((uuid, "", 1)))
         inv2 = newrepos.get_inventory(
-                mapping.generate_revision_id(uuid, 5, ""))
+                mapping.revision_id_foreign_to_bzr((uuid, "", 5)))
         self.assertNotEqual(inv1.path2id("y"), inv2.path2id("y"))
 
+    def test_svn_import_bzr_branch(self):
+        self.run_bzr('init foo')
+        self.run_bzr_error('bzr: ERROR: Source repository is not a Subversion repository.\n', 'svn-import foo dc')
+
+    def test_svn_import_bzr_repo(self):
+        self.run_bzr('init-repo foo')
+        self.run_bzr('init foo/bar')
+        self.run_bzr_error('bzr: ERROR: Source repository is not a Subversion repository.\n', 'svn-import foo/bar dc')
 
     def test_list(self):
         repos_url = self.make_repository("a")
@@ -391,3 +441,49 @@ if len(sys.argv) == 2:
 
         self.monkey_patch_gpg()
         self.run_bzr('sign-my-commits')
+
+    def test_knit_corruption(self):
+        cwd = os.getcwd()
+        svn_url = self.make_client('d', 'wc')
+
+        os.chdir('wc')
+        for d in ['trunk', 'branches', 'tags']:
+           os.mkdir(d)
+        f = open('trunk/file', 'wb')
+        f.write('Hi\n')
+        f.close()
+        self.client_add("trunk")
+        self.client_add("tags")
+        self.client_add("branches")
+        self.client_commit(".", "initial check-in")
+        self.client_update(".")
+        os.chdir(cwd)
+
+        self.run_bzr('init-repo --1.9-rich-root --no-trees shared')
+        os.chdir('shared')
+        self.run_bzr('branch %s/trunk trunk' % svn_url)
+        os.chdir(cwd)
+
+        self.run_bzr('branch shared/trunk bzr-branch')
+        os.chdir('bzr-branch')
+        f = open('file', 'ab')
+        f.write('Bye\n')
+        f.close()
+        self.run_bzr('ci -m "Add bye."')
+        f = open('file', 'ab')
+        f.write('Good riddance.\n')
+        f.close()
+        self.run_bzr('ci -m "Add good riddance."')
+
+        self.run_bzr('svn-push %s/branches/my-branch' % svn_url)
+        os.chdir(cwd)
+
+        self.run_bzr('co %s/trunk bzr-trunk' % svn_url)
+        os.chdir('bzr-trunk')
+        self.run_bzr('merge %s/branches/my-branch' % svn_url)
+        self.run_bzr('nick my-branch')
+        self.run_bzr('ci -m "Merge my-branch"')
+
+        os.chdir(cwd)
+        os.chdir('shared/trunk')
+        self.run_bzr('pull')
