@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,26 +16,34 @@
 
 """Branch tests."""
 
+import os
+import subvertpy
+from unittest import TestCase
+
 from bzrlib import urlutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoSuchFile, NoSuchRevision, NotBranchError, NoSuchTag
+from bzrlib.errors import (
+    NoSuchFile,
+    NoSuchRevision,
+    NoSuchTag,
+    NotBranchError,
+    TagsNotSupported,
+    )
 from bzrlib.repository import Repository
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
 
-import os
-from unittest import TestCase
-
-from bzrlib.plugins.svn import core
-from bzrlib.plugins.svn.branch import FakeControlFiles, SvnBranchFormat
+from bzrlib.plugins.svn.branch import (
+    FakeControlFiles,
+    SvnBranchFormat,
+    )
 from bzrlib.plugins.svn.convert import load_dumpfile
 from bzrlib.plugins.svn.mapping import SVN_PROP_BZR_REVISION_ID
-from bzrlib.plugins.svn.mapping3 import BzrSvnMappingv3FileProps
-from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import SubversionTestCase
+
+class WorkingSubversionBranch(SubversionTestCase):
 
-class WorkingSubversionBranch(TestCaseWithSubversionRepository):
     def test_last_rev_rev_hist(self):
         repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
@@ -59,6 +67,24 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         b = Branch.open(repos_url + "/trunk")
         self.assertEquals(["foo"], b.tags.get_tag_dict().keys())
 
+    def test_tags_other_project(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        other = dc.add_dir("otherproj")
+        other.add_dir("otherproj/trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        my = dc.add_dir("myproj")
+        trunk = my.add_dir("myproj/trunk")
+        tags = my.add_dir("myproj/tags")
+        tags.add_dir("myproj/tags/foo", "otherproj/trunk")
+        dc.close()
+
+        b = Branch.open(repos_url + "/myproj/trunk")
+        self.assertEquals(["foo"], b.tags.get_tag_dict().keys())
+
     def test_tag_set(self):
         repos_url = self.make_repository('a')
 
@@ -75,7 +101,7 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         b = Branch.open(repos_url + "/trunk")
         b.tags.set_tag(u"mytag", b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping()))
 
-        self.assertEquals(core.NODE_DIR, 
+        self.assertEquals(subvertpy.NODE_DIR, 
                 b.repository.transport.check_path("tags/mytag", 3))
 
     def test_tags_delete(self):
@@ -93,6 +119,19 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         b = Branch.open(repos_url + "/trunk")
         self.assertEquals([], b.tags.get_tag_dict().keys())
 
+    def test_tag_set_not_supported(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_dir("trunk/gui")
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk/gui")
+        self.assertRaises(TagsNotSupported,
+                b.tags.set_tag, u"mytag", 
+                b.repository.generate_revision_id(1, "trunk/gui", b.repository.get_mapping()))
+
     def test_tag_lookup(self):
         repos_url = self.make_repository("a")
        
@@ -141,6 +180,30 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         self.assertEqual("trunk2", branch.get_branch_path(2))
         self.assertEqual("trunk", branch.get_branch_path(1))
 
+    def test_pull_internal(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.add_dir("branches")
+        branches.add_dir("branches/foo", "trunk", 1)
+        dc.close()
+
+        otherbranch = Branch.open(urlutils.join(repos_url, "branches", "foo"))
+        branch = Branch.open(urlutils.join(repos_url, "trunk"))
+        result = branch.pull(otherbranch)
+        self.assertEquals(branch.last_revision(), otherbranch.last_revision())
+        self.assertEquals(result.new_revid, otherbranch.last_revision())
+        self.assertEquals(result.old_revid, branch.revision_history()[0])
+        self.assertEquals(result.old_revno, 1)
+        self.assertEquals(result.new_revno, 2)
+        self.assertEquals(result.master_branch, None)
+        self.assertEquals(result.source_branch, otherbranch)
+        self.assertEquals(result.target_branch, branch)
+
     def test_get_branch_path_subdir(self):
         repos_url = self.make_repository("a")
 
@@ -268,7 +331,7 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
     def test_set_push_location(self):
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
-        self.assertRaises(NotImplementedError, branch.set_push_location, [])
+        branch.set_push_location("http://bar/bloe")
 
     def test_get_parent(self):
         repos_url = self.make_repository('a')
@@ -280,42 +343,6 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         branch = Branch.open(repos_url)
         self.assertIs(None, branch.get_push_location())
 
-    def test_revision_history(self):
-        repos_url = self.make_repository('a')
-
-        branch = Branch.open(repos_url)
-        self.assertEqual([branch.generate_revision_id(0)], 
-                branch.revision_history())
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("foo").modify()
-        dc.change_prop(SVN_PROP_BZR_REVISION_ID+"none", 
-                "42 mycommit\n")
-        dc.close()
-        
-        branch = Branch.open(repos_url)
-        repos = Repository.open(repos_url)
-        
-        mapping = repos.get_mapping()
-
-        self.assertEqual([repos.generate_revision_id(0, "", mapping), 
-                    repos.generate_revision_id(1, "", mapping)], 
-                branch.revision_history())
-
-        dc = self.get_commit_editor(repos_url)
-        dc.open_file("foo").modify()
-        dc.close()
-
-        branch = Branch.open(repos_url)
-        repos = Repository.open(repos_url)
-
-        mapping = repos.get_mapping()
-
-        self.assertEqual([
-            repos.generate_revision_id(0, "", mapping),
-            "mycommit",
-            repos.generate_revision_id(2, "", mapping)],
-            branch.revision_history())
 
     def test_revision_id_to_revno_none(self):
         """The None revid should map to revno 0."""
@@ -330,37 +357,6 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         branch = Branch.open(repos_url)
         self.assertRaises(NoSuchRevision, branch.revision_id_to_revno, "bla")
     
-    def test_revision_id_to_revno_simple(self):
-        repos_url = self.make_repository('a')
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("foo").modify()
-        dc.change_prop("bzr:revision-id:v3-none", 
-                            "2 myrevid\n")
-        dc.close()
-
-        branch = Branch.open(repos_url)
-        self.assertEquals(2, branch.revision_id_to_revno("myrevid"))
-
-    def test_revision_id_to_revno_older(self):
-        repos_url = self.make_repository('a')
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("foo").modify()
-        dc.change_prop("bzr:revision-id:v3-none", 
-                            "2 myrevid\n")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.open_file("foo").modify()
-        dc.change_prop("bzr:revision-id:v3-none", 
-                            "2 myrevid\n3 mysecondrevid\n")
-        dc.close()
-
-        branch = Branch.open(repos_url)
-        self.assertEquals(3, branch.revision_id_to_revno("mysecondrevid"))
-        self.assertEquals(2, branch.revision_id_to_revno("myrevid"))
-
     def test_get_nick_none(self):
         repos_url = self.make_repository('a')
 
@@ -693,11 +689,11 @@ foohosts""")
         newbranch.repository.lock_read()
         texts = newbranch.repository.texts
         host_fileid = tree.inventory.path2id("hosts")
-        mapping = BzrSvnMappingv3FileProps(TrunkBranchingScheme())
+        mapping = oldbranch.repository.get_mapping()
         self.assertVersionsPresentEquals(texts, host_fileid, [
-            mapping.generate_revision_id(uuid, 1, "trunk"),
-            mapping.generate_revision_id(uuid, 2, "trunk"),
-            mapping.generate_revision_id(uuid, 3, "trunk"),
+            mapping.revision_id_foreign_to_bzr((uuid, "trunk", 1)),
+            mapping.revision_id_foreign_to_bzr((uuid, "trunk", 2)),
+            mapping.revision_id_foreign_to_bzr((uuid, "trunk", 3)),
             oldbranch.generate_revision_id(6)])
         newbranch.repository.unlock()
 
@@ -720,7 +716,8 @@ foohosts""")
         dc.close()
 
         branch = Branch.open('d')
-        self.assertEqual("svn-v3-none:%s::1" % (branch.repository.uuid),  branch.generate_revision_id(1))
+        mapping = branch.repository.get_mapping()
+        self.assertEqual(mapping.revision_id_foreign_to_bzr((branch.repository.uuid, "", 1)), branch.generate_revision_id(1))
 
     def test_create_checkout(self):
         repos_url = self.make_repository('d')
