@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -18,12 +18,15 @@
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoRepositoryPresent, UninitializableFormat
+from bzrlib.errors import (
+    NoRepositoryPresent,
+    UninitializableFormat,
+    )
 from bzrlib.tests import TestCase
 
 from bzrlib.plugins.svn.workingtree import SvnWorkingTreeFormat
 from bzrlib.plugins.svn.format import SvnWorkingTreeDirFormat
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import SubversionTestCase
 
 class TestWorkingTreeFormat(TestCase):
     def setUp(self):
@@ -47,14 +50,14 @@ class TestCheckoutFormat(TestCase):
         self.format = SvnWorkingTreeDirFormat()
 
     def test_get_converter(self):
-        self.assertRaises(NotImplementedError, self.format.get_converter)
+        convert = self.format.get_converter()
 
     def test_initialize(self):
         self.assertRaises(UninitializableFormat, 
                           self.format.initialize_on_transport, None)
 
 
-class TestCheckout(TestCaseWithSubversionRepository):
+class TestCheckout(SubversionTestCase):
     def test_not_for_writing(self):
         self.make_client("d", "dc")
         x = BzrDir.create_branch_convenience("dc/foo")
@@ -65,6 +68,11 @@ class TestCheckout(TestCaseWithSubversionRepository):
         x = BzrDir.open("dc")
         self.assertRaises(NoRepositoryPresent, x.open_repository)
 
+    def test_create_repository(self):
+        self.make_client("d", "dc")
+        x = BzrDir.open("dc")
+        self.assertRaises(UninitializableFormat, x.create_repository)
+
     def test_find_repository(self):
         self.make_client("d", "dc")
         x = BzrDir.open("dc")
