@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -18,24 +18,40 @@
 
 """Commit and push tests."""
 
-from bzrlib.branch import Branch, PullResult
+from copy import copy
+import os
+from subvertpy import ra
+from subvertpy.properties import time_to_cstring
+
+from bzrlib.branch import (
+    Branch,
+    PullResult,
+    )
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import DivergedBranches, BzrError
-from bzrlib.tests import TestCase
+from bzrlib.errors import (
+    DivergedBranches,
+    BzrError,
+    )
+from bzrlib.repository import Repository
+from bzrlib.tests import (
+    TestCase,
+    TestSkipped,
+    )
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
-from copy import copy
-import os
-
-from bzrlib.plugins.svn import ra
-from bzrlib.plugins.svn.commit import set_svn_revprops, _revision_id_to_svk_feature
+from bzrlib.plugins.svn import format
+from bzrlib.plugins.svn.commit import (
+    set_svn_revprops,
+    _revision_id_to_svk_feature,
+    )
 from bzrlib.plugins.svn.errors import RevpropChangeFailed
-from bzrlib.plugins.svn.properties import time_to_cstring
+from bzrlib.plugins.svn.mapping import mapping_registry
 from bzrlib.plugins.svn.transport import SvnRaTransport
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import SubversionTestCase
+
+class TestNativeCommit(SubversionTestCase):
 
-class TestNativeCommit(TestCaseWithSubversionRepository):
     def test_simple_commit(self):
         self.make_client('d', 'dc')
         self.build_tree({'dc/foo/bla': "data"})
@@ -82,7 +98,10 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
 
     def test_commit_unicode_filename(self):
         self.make_client('d', 'dc')
-        self.build_tree({u'dc/I²C': "data"})
+        try:
+            self.build_tree({u'dc/I²C': "data"})
+        except UnicodeError:
+            raise TestSkipped("This platform does not support unicode symlinks")
         self.client_add(u"dc/I²C".encode("utf-8"))
         wt = WorkingTree.open("dc")
         wt.commit(message="data")
@@ -130,20 +149,6 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
                 wt.branch.generate_revision_id(1),
                 wt.branch.last_revision())
 
-    def test_commit_parents(self):
-        repos_url = self.make_client('d', 'dc')
-        self.build_tree({'dc/foo/bla': "data"})
-        self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
-        wt.set_pending_merges(["some-ghost-revision"])
-        self.assertEqual(["some-ghost-revision"], wt.pending_merges())
-        wt.commit(message="data")
-        self.assertEqual("some-ghost-revision\n", 
-                self.client_get_prop(repos_url, "bzr:ancestry:v3-none", 1))
-        self.assertEqual((wt.branch.generate_revision_id(0), "some-ghost-revision"),
-                         wt.branch.repository.get_revision(
-                             wt.branch.last_revision()).parent_ids)
-
     def test_commit_rename_file(self):
         repos_url = self.make_client('d', 'dc')
         self.build_tree({'dc/foo': "data"})
@@ -151,16 +156,19 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
         wt = WorkingTree.open("dc")
         wt.set_pending_merges(["some-ghost-revision"])
         oldid = wt.path2id("foo")
-        wt.commit(message="data")
+        wt.commit(message="data") # 1
         wt.rename_one("foo", "bar")
-        wt.commit(message="doe")
+        self.assertEquals(oldid, wt.path2id("bar"))
+        self.assertEquals(None, wt.path2id("foo"))
+        wt.commit(message="doe") # 2
         paths = self.client_log(repos_url, 2, 0)[2][0]
         self.assertEquals('D', paths["/foo"][0])
         self.assertEquals('A', paths["/bar"][0])
         self.assertEquals('/foo', paths["/bar"][1])
         self.assertEquals(1, paths["/bar"][2])
-        self.assertEquals("bar\t%s\n" % oldid, 
-                          self.client_get_prop(repos_url, "bzr:file-ids", 2))
+        svnrepo = Repository.open(repos_url)
+        self.assertEquals({u"bar": oldid}, 
+                svnrepo._revmeta_provider.get_revision("", 2).get_fileid_overrides(svnrepo.get_mapping()))
 
     def test_commit_rename_file_from_directory(self):
         repos_url = self.make_client('d', 'dc')
@@ -178,29 +186,6 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
         self.assertEquals('/adir/foo', paths["/bar"][1])
         self.assertEquals(1, paths["/bar"][2])
 
-    def test_commit_revision_id(self):
-        repos_url = self.make_client('d', 'dc')
-        wt = WorkingTree.open("dc")
-        self.build_tree({'dc/foo/bla': "data", 'dc/bla': "otherdata"})
-        wt.add('bla')
-        wt.commit(message="data")
-
-        branch = Branch.open(repos_url)
-        builder = branch.get_commit_builder([branch.last_revision()], 
-                revision_id="my-revision-id")
-        tree = branch.repository.revision_tree(branch.last_revision())
-        new_tree = copy(tree)
-        ie = new_tree.inventory.root
-        ie.revision = None
-        builder.record_entry_contents(ie, [tree.inventory], '', new_tree, 
-                                      None)
-        builder.finish_inventory()
-        builder.commit("foo")
-
-        self.assertEqual("3 my-revision-id\n", 
-            self.client_get_prop("dc", 
-                "bzr:revision-id:v3-none", 2))
-
     def test_commit_sets_mergeinfo(self):
         repos_url = self.make_repository('d')
 
@@ -246,32 +231,6 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
         except NotImplementedError:
             pass # Svn 1.4
 
-    def test_commit_metadata(self):
-        repos_url = self.make_client('d', 'dc')
-        wt = WorkingTree.open("dc")
-        self.build_tree({'dc/foo/bla': "data", 'dc/bla': "otherdata"})
-        wt.add('bla')
-        wt.commit(message="data")
-
-        branch = Branch.open(repos_url)
-        builder = branch.get_commit_builder([branch.last_revision()], 
-                timestamp=4534.0, timezone=2, committer="fry",
-                revision_id="my-revision-id")
-        tree = branch.repository.revision_tree(branch.last_revision())
-        new_tree = copy(tree)
-        ie = new_tree.inventory.root
-        ie.revision = None
-        builder.record_entry_contents(ie, [tree.inventory], '', new_tree, None)
-        builder.finish_inventory()
-        builder.commit("foo")
-
-        self.assertEqual("3 my-revision-id\n", 
-                self.client_get_prop("dc", "bzr:revision-id:v3-none", 2))
-
-        self.assertEqual(
-                "timestamp: 1970-01-01 01:15:36.000000000 +0000\ncommitter: fry\n",
-                self.client_get_prop("dc", "bzr:revision-info", 2))
-
     def test_mwh(self):
         repo = self.make_client('d', 'sc')
         def mv(*mvs):
@@ -309,7 +268,8 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
         self.assertEqual("data", open('pyd/bar').read())
         
 
-class TestPush(TestCaseWithSubversionRepository):
+class TestPush(SubversionTestCase):
+
     def setUp(self):
         super(TestPush, self).setUp()
         self.repos_url = self.make_client('d', 'sc')
@@ -359,13 +319,29 @@ class TestPush(TestCaseWithSubversionRepository):
                           self.newdir.open_branch())
 
     def test_unicode_filename(self):
-        self.build_tree({u'dc/I²C': 'other data'})
+        try:
+            self.build_tree({u'dc/I²C': 'other data'})
+        except UnicodeError:
+            raise TestSkipped("This platform does not support unicode paths")
         wt = self.newdir.open_workingtree()
         wt.add(u'I²C')
         wt.commit(message="Commit from Bzr")
         self.assertEqual(1, int(self.olddir.open_branch().pull(
                                 self.newdir.open_branch())))
 
+    def test_rename_from_unicode_filename(self):
+        try:
+            self.build_tree({u'dc/I²C': 'other data'})
+        except UnicodeError:
+            raise TestSkipped("This platform does not support symlink paths")
+        wt = self.newdir.open_workingtree()
+        wt.add(u'I²C')
+        wt.commit(message="Commit from Bzr")
+        wt.rename_one(u'I²C', u'I²C2')
+        wt.commit(message="Commit from Bzr2")
+        self.assertEqual(2, int(self.olddir.open_branch().pull(
+                                self.newdir.open_branch())))
+
     def test_change(self):
         self.build_tree({'dc/foo/bla': 'other data'})
         wt = self.newdir.open_workingtree()
@@ -528,7 +504,43 @@ class TestPush(TestCaseWithSubversionRepository):
         self.assertEquals('D', paths["/adir/foob"][0])
 
 
-class TestPushNested(TestCaseWithSubversionRepository):
+class JoinedCommitTests(SubversionTestCase):
+
+    def test_commit_join(self):
+        repos_url = self.make_client('d', 'sc')
+
+        self.build_tree({'sc/trunk/foo/bla': "data"})
+        self.client_add("sc/trunk")
+        self.client_commit("sc", "foo")
+
+        self.olddir = BzrDir.open("sc/trunk")
+        os.mkdir("dc")
+        self.newdir = self.olddir.sprout("dc")
+
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.add_dir("branches")
+        newdir = branches.add_dir("branches/newbranch")
+        newdir.add_file("branches/newbranch/foob").modify()
+        dc.close()
+
+        wt = self.newdir.open_workingtree()
+        self.build_tree({"dc/lala": "data"})
+        wt.add(["lala"])
+        wt.commit(message="init")
+        joinedwt = BzrDir.create_standalone_workingtree("dc/newdir", 
+            format=format.get_rich_root_format())
+        joinedwt.pull(Branch.open(repos_url+"/branches/newbranch"))
+        wt.subsume(joinedwt)
+        wt.commit(message="doe")
+
+        self.olddir.open_branch().pull(self.newdir.open_branch())
+        paths = self.client_log(repos_url, 4, 0)[4][0]
+        mutter('paths %r' % paths)
+        self.assertEquals(('A', "/branches/newbranch", 2), paths["/trunk/newdir"])
+
+
+class TestPushNested(SubversionTestCase):
+
     def setUp(self):
         super(TestPushNested, self).setUp()
         self.repos_url = self.make_client('d', 'sc')
@@ -553,7 +565,8 @@ class TestPushNested(TestCaseWithSubversionRepository):
         self.assertFalse(os.path.exists("sc/foo/trunk/filel"))
 
 
-class HeavyWeightCheckoutTests(TestCaseWithSubversionRepository):
+class HeavyWeightCheckoutTests(SubversionTestCase):
+
     def test_bind(self):
         repos_url = self.make_repository("d")
         master_branch = Branch.open(repos_url)
@@ -571,6 +584,7 @@ class HeavyWeightCheckoutTests(TestCaseWithSubversionRepository):
         wt = local_dir.open_workingtree()
         local_dir.open_branch().bind(master_branch)
         self.build_tree({'b/file': 'data'})
+        wt.branch.nick = "somenick"
         wt.add('file')
         revid = wt.commit(message="Commit from Bzr")
         master_branch = Branch.open(repos_url)
@@ -584,20 +598,23 @@ class HeavyWeightCheckoutTests(TestCaseWithSubversionRepository):
         wt = local_dir.open_workingtree()
         local_dir.open_branch().bind(master_branch)
         self.build_tree({'b/file': 'data'})
+        wt.branch.nick = "some-nick"
         wt.add('file')
         oldid = wt.path2id("file")
         revid1 = wt.commit(message="Commit from Bzr")
         wt.rename_one('file', 'file2')
         revid2 = wt.commit(message="Commit from Bzr")
         master_branch = Branch.open(repos_url)
-        self.assertEquals("file\t%s\n" % oldid, 
-                          self.client_get_prop(repos_url, "bzr:file-ids", 1))
-        self.assertEquals("file2\t%s\n" % oldid, 
-                          self.client_get_prop(repos_url, "bzr:file-ids", 2))
+        rm_provider = master_branch.repository._revmeta_provider
+        mapping = master_branch.repository.get_mapping()
+        self.assertEquals({u"file": oldid}, 
+                rm_provider.get_revision("", 1).get_fileid_overrides(mapping))
+        self.assertEquals({u"file2": oldid}, 
+                rm_provider.get_revision("", 2).get_fileid_overrides(mapping))
         tree1 = master_branch.repository.revision_tree(revid1)
         tree2 = master_branch.repository.revision_tree(revid2)
         delta = tree2.changes_from(tree1)
-        mutter("changes %r" % list(master_branch.repository.iter_reverse_branch_changes("", 2, 0, master_branch.repository.get_mapping())))
+        mutter("changes %r" % list(rm_provider.iter_reverse_branch_changes("", 2, 0)))
         self.assertEquals(0, len(delta.added))
         self.assertEquals(0, len(delta.removed))
         self.assertEquals(1, len(delta.renamed))
@@ -609,6 +626,7 @@ class HeavyWeightCheckoutTests(TestCaseWithSubversionRepository):
         local_dir = master_branch.bzrdir.sprout("b")
         wt = local_dir.open_workingtree()
         local_dir.open_branch().bind(master_branch)
+        wt.branch.nick = "some-nick"
         self.build_tree({'b/dir/file': 'data'})
         wt.add('dir')
         wt.add('dir/file')
@@ -616,23 +634,28 @@ class HeavyWeightCheckoutTests(TestCaseWithSubversionRepository):
         fileid = wt.path2id("dir/file")
         revid1 = wt.commit(message="Commit from Bzr")
         master_branch = Branch.open(repos_url)
-        self.assertEquals("dir\t%s\n" % dirid +
-                          "dir/file\t%s\n" % fileid, 
-                          self.client_get_prop(repos_url, "bzr:file-ids", 1))
+        rm_provider = master_branch.repository._revmeta_provider
+        mapping = master_branch.repository.get_mapping()
+        self.assertEquals({"dir": dirid, 
+                          "dir/file": fileid},
+                          rm_provider.get_revision("", 1).get_fileid_overrides(mapping))
 
 
-class RevpropTests(TestCaseWithSubversionRepository):
+class RevpropTests(SubversionTestCase):
+
     def test_change_revprops(self):
-        repos_url = self.make_repository("d")
+        repos_url = self.make_repository("d", allow_revprop_changes=True)
 
         dc = self.get_commit_editor(repos_url, message="My commit")
         dc.add_file("foo.txt").modify()
         dc.close()
 
-        transport = SvnRaTransport(repos_url)
-        set_svn_revprops(transport, 1, {"svn:author": "Somebody", 
+        repository = Repository.open(repos_url)
+        set_svn_revprops(repository, 1, {"svn:author": "Somebody", 
                                         "svn:date": time_to_cstring(1000000*473385600)})
 
+        self.assertEquals(1, repository.get_latest_revnum())
+
         self.assertEquals(("Somebody", "1985-01-01T00:00:00.000000Z", "My commit"), 
                           self.client_log(repos_url, 1, 1)[1][1:])
 
@@ -643,14 +666,16 @@ class RevpropTests(TestCaseWithSubversionRepository):
         dc.add_file("foo.txt").modify()
         dc.close()
 
-        transport = SvnRaTransport(repos_url)
+        repository = Repository.open(repos_url)
         self.assertRaises(RevpropChangeFailed, 
-                lambda: set_svn_revprops(transport, 1, {"svn:author": "Somebody", "svn:date": time_to_cstring(1000000*473385600)}))
+                lambda: set_svn_revprops(repository, 1, {"svn:author": "Somebody", "svn:date": time_to_cstring(1000000*473385600)}))
 
 
 class SvkTestCase(TestCase):
+
     def test_revid_svk_map(self):
         self.assertEqual("auuid:/:6", 
-              _revision_id_to_svk_feature("svn-v3-undefined:auuid::6"))
+              _revision_id_to_svk_feature("svn-v3-undefined:auuid::6",
+                  mapping_registry.parse_revision_id))
 
 
