@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,21 +16,46 @@
 
 """Full repository conversion tests."""
 
-from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir, format_registry
-from bzrlib.errors import NotBranchError, NoSuchFile, IncompatibleRepositories
-from bzrlib.urlutils import local_path_to_url
-from bzrlib.repository import Repository
-from bzrlib.tests import TestCaseInTempDir
-
 import os, sys
+from subvertpy import repos
 
-from bzrlib.plugins.svn import repos
-from bzrlib.plugins.svn.convert import convert_repository, NotDumpFile, load_dumpfile
-from bzrlib.plugins.svn.format import get_rich_root_format
-from bzrlib.plugins.svn.mapping3 import set_branching_scheme
-from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme, NoBranchingScheme
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.branch import (
+    Branch,
+    )
+from bzrlib.bzrdir import (
+    BzrDir,
+    format_registry,
+    )
+from bzrlib.errors import (
+    IncompatibleRepositories,
+    NotBranchError,
+    NoSuchFile,
+    )
+from bzrlib.urlutils import (
+    local_path_to_url,
+    )
+from bzrlib.repository import (
+    Repository,
+    )
+from bzrlib.tests import (
+    TestCaseInTempDir,
+    )
+
+from bzrlib.plugins.svn.layout.standard import (
+    RootLayout,
+    TrunkLayout,
+    )
+from bzrlib.plugins.svn.convert import (
+    NotDumpFile,
+    convert_repository,
+    load_dumpfile,
+    )
+from bzrlib.plugins.svn.format import (
+    get_rich_root_format,
+    )
+from bzrlib.plugins.svn.tests import (
+    SubversionTestCase,
+    )
 
 class TestLoadDumpfile(TestCaseInTempDir):
     def test_loaddumpfile(self):
@@ -61,7 +86,8 @@ PROPS-END
         self.assertRaises(NotDumpFile, load_dumpfile, dumpfile, "d")
 
 
-class TestConversion(TestCaseWithSubversionRepository):
+class TestConversion(SubversionTestCase):
+
     def setUp(self):
         super(TestConversion, self).setUp()
         self.repos_url = self.make_repository('d')
@@ -82,9 +108,25 @@ class TestConversion(TestCaseWithSubversionRepository):
     def get_commit_editor(self):
         return super(TestConversion, self).get_commit_editor(self.repos_url)
 
+    def test_parent_branch_outside_prefix(self):
+        dc = self.get_commit_editor()
+
+        foo = dc.add_dir("foo")
+        foo.add_dir("foo/trunk")
+        dc.close()
+
+        dc = self.get_commit_editor()
+        bar = dc.add_dir("bar")
+        bar.add_dir("bar/trunk", "foo/trunk")
+        dc.close()
+
+        convert_repository(Repository.open(self.repos_url), "e",
+                           TrunkLayout(1),
+                           all=True, create_shared_repo=True, prefix="bar")
+
     def test_sets_parent_urls(self):
         convert_repository(Repository.open(self.repos_url), "e", 
-                           TrunkBranchingScheme(), 
+                           TrunkLayout(0), 
                            all=False, create_shared_repo=True)
         self.assertEquals(self.repos_url+"/trunk", 
                 Branch.open("e/trunk").get_parent())
@@ -105,10 +147,10 @@ class TestConversion(TestCaseWithSubversionRepository):
 
         oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", 
-                           TrunkBranchingScheme(), 
+                           TrunkLayout(0), 
                            all=False, create_shared_repo=True)
         newrepos = Repository.open("e")
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch", oldrepos.get_mapping())))
 
     def test_fetch_filebranch(self):
@@ -118,9 +160,9 @@ class TestConversion(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(self.repos_url)
-        convert_repository(oldrepos, "e", TrunkBranchingScheme())
+        convert_repository(oldrepos, "e", TrunkLayout(0))
         newrepos = Repository.open("e")
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch", oldrepos.get_mapping())))
 
     def test_fetch_dead(self):
@@ -136,7 +178,7 @@ class TestConversion(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(self.repos_url)
-        convert_repository(oldrepos, "e", TrunkBranchingScheme(), 
+        convert_repository(oldrepos, "e", TrunkLayout(0), 
                            all=True, create_shared_repo=True)
         newrepos = Repository.open("e")
         self.assertTrue(newrepos.has_revision(
@@ -156,10 +198,9 @@ class TestConversion(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(self.repos_url)
-        convert_repository(oldrepos, "e", TrunkBranchingScheme(), 
+        convert_repository(oldrepos, "e", TrunkLayout(0), 
             create_shared_repo=True,
             filter_branch=lambda branch: branch.get_branch_path().endswith("somebranch"))
-        newrepos = Repository.open("e")
         self.assertTrue(os.path.exists("e/branches/somebranch"))
         self.assertFalse(os.path.exists("e/branches/anotherbranch"))
 
@@ -168,13 +209,13 @@ class TestConversion(TestCaseWithSubversionRepository):
         dir.create_repository(shared=True)
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True)
+                TrunkLayout(0), create_shared_repo=True)
 
         self.assertTrue(Repository.open("e").is_shared())
 
     def test_shared_import_continue_remove(self):
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True)
+                TrunkLayout(0), create_shared_repo=True)
 
         dc = self.get_commit_editor()
         dc.delete("trunk")
@@ -186,11 +227,11 @@ class TestConversion(TestCaseWithSubversionRepository):
         dc.close()
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                           TrunkBranchingScheme(), create_shared_repo=True)
+                           TrunkLayout(0), create_shared_repo=True)
 
     def test_shared_import_remove_nokeep(self):
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True)
+                TrunkLayout(0), create_shared_repo=True)
 
         dc = self.get_commit_editor()
         dc.delete("trunk")
@@ -199,29 +240,29 @@ class TestConversion(TestCaseWithSubversionRepository):
         self.assertTrue(os.path.exists("e/trunk"))
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                           TrunkBranchingScheme(), create_shared_repo=True)
+                           TrunkLayout(0), create_shared_repo=True)
 
         self.assertFalse(os.path.exists("e/trunk"))
 
     def test_shared_import_continue_with_wt(self):
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), working_trees=True)
+                TrunkLayout(0), working_trees=True)
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), working_trees=True)
+                TrunkLayout(0), working_trees=True)
 
-    def test_shared_import_nonescheme_empty(self):
+    def test_shared_import_rootlayout_empty(self):
         dir = BzrDir.create("e", format=get_rich_root_format())
         dir.create_repository(shared=True)
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                NoBranchingScheme(), create_shared_repo=True)
+                RootLayout(), create_shared_repo=True)
 
     def test_shared_import_with_wt(self):
         dir = BzrDir.create("e", format=get_rich_root_format())
         dir.create_repository(shared=True)
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True, 
+                TrunkLayout(0), create_shared_repo=True, 
                 working_trees=True)
 
         self.assertTrue(os.path.isfile(os.path.join(
@@ -232,7 +273,7 @@ class TestConversion(TestCaseWithSubversionRepository):
         dir.create_repository(shared=True)
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True, 
+                TrunkLayout(0), create_shared_repo=True, 
                 working_trees=False)
 
         self.assertFalse(os.path.isfile(os.path.join(
@@ -244,13 +285,13 @@ class TestConversion(TestCaseWithSubversionRepository):
 
         self.assertRaises(IncompatibleRepositories, 
             lambda: convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True, 
+                TrunkLayout(0), create_shared_repo=True, 
                 working_trees=False))
 
     def test_shared_import_continue_branch(self):
         oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", 
-                TrunkBranchingScheme(), create_shared_repo=True)
+                TrunkLayout(0), create_shared_repo=True)
 
         mapping = oldrepos.get_mapping()
 
@@ -264,7 +305,7 @@ class TestConversion(TestCaseWithSubversionRepository):
                 Branch.open("e/trunk").last_revision())
 
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True)
+                TrunkLayout(0), create_shared_repo=True)
 
         self.assertEqual(Repository.open(self.repos_url).generate_revision_id(3, "trunk", mapping), 
                         Branch.open("e/trunk").last_revision())
@@ -272,25 +313,25 @@ class TestConversion(TestCaseWithSubversionRepository):
  
     def test_shared_import(self):
         convert_repository(Repository.open(self.repos_url), "e", 
-                TrunkBranchingScheme(), create_shared_repo=True)
+                TrunkLayout(0), create_shared_repo=True)
 
         self.assertTrue(Repository.open("e").is_shared())
     
     def test_simple(self):
-        convert_repository(Repository.open(self.repos_url), os.path.join(self.test_dir, "e"), TrunkBranchingScheme())
+        convert_repository(Repository.open(self.repos_url), os.path.join(self.test_dir, "e"), TrunkLayout(0))
         self.assertTrue(os.path.isdir(os.path.join(self.test_dir, "e", "trunk")))
         self.assertTrue(os.path.isdir(os.path.join(self.test_dir, "e", "branches", "abranch")))
 
     def test_convert_to_nonexistant(self):
-        self.assertRaises(NoSuchFile, convert_repository, Repository.open(self.repos_url), os.path.join(self.test_dir, "e", "foo", "bar"), TrunkBranchingScheme())
+        self.assertRaises(NoSuchFile, convert_repository, Repository.open(self.repos_url), os.path.join(self.test_dir, "e", "foo", "bar"), TrunkLayout(0))
 
     def test_notshared_import(self):
         convert_repository(Repository.open(self.repos_url), "e", 
-                           TrunkBranchingScheme(), create_shared_repo=False)
+                           TrunkLayout(0), create_shared_repo=False)
 
         self.assertRaises(NotBranchError, Repository.open, "e")
 
-class TestConversionFromDumpfile(TestCaseWithSubversionRepository):
+class TestConversionFromDumpfile(SubversionTestCase):
     def test_dumpfile_open_empty(self):
         dumpfile = os.path.join(self.test_dir, "dumpfile")
         open(dumpfile, 'w').write(
@@ -310,9 +351,9 @@ PROPS-END
 """)
         branch_path = os.path.join(self.test_dir, "f")
         repos = self.load_dumpfile(dumpfile, 'g')
-        convert_repository(repos, branch_path, NoBranchingScheme())
+        convert_repository(repos, branch_path, RootLayout())
         branch = Repository.open(branch_path)
-        self.assertEqual(['svn-v3-none:6987ef2d-cd6b-461f-9991-6f1abef3bd59::0'], branch.all_revision_ids())
+        self.assertEqual(len(branch.all_revision_ids()), 1)
         Branch.open(branch_path)
 
     def load_dumpfile(self, dumpfile, target_path):
@@ -338,7 +379,7 @@ PROPS-END
 """)
         branch_path = os.path.join(self.test_dir, "f")
         repos = self.load_dumpfile(dumpfile, 'g')
-        convert_repository(repos, branch_path, TrunkBranchingScheme())
+        convert_repository(repos, branch_path, TrunkLayout(0))
         repository = Repository.open(branch_path)
         self.assertEqual([], repository.all_revision_ids())
         self.assertRaises(NotBranchError, Branch.open, branch_path)
@@ -402,12 +443,12 @@ data
 """)
         repos = self.load_dumpfile(filename, 'g')
         convert_repository(repos, os.path.join(self.test_dir, "e"), 
-                           TrunkBranchingScheme())
+                           TrunkLayout(0))
         mapping = repos.get_mapping()
         abspath = self.test_dir
         if sys.platform == 'win32':
             abspath = '/' + abspath
         branch = Branch.open(os.path.join(self.test_dir, "e", "trunk"))
         self.assertEqual(local_path_to_url(os.path.join(self.test_dir, "e", "trunk")), branch.base.rstrip("/"))
-        self.assertEqual(mapping.generate_revision_id("6987ef2d-cd6b-461f-9991-6f1abef3bd59", 1, 'trunk'), branch.last_revision())
+        self.assertEqual(mapping.revision_id_foreign_to_bzr(("6987ef2d-cd6b-461f-9991-6f1abef3bd59", 'trunk', 1)), branch.last_revision())
 
