@@ -1,9 +1,9 @@
-# Copyright (C) 2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2007-2009 Jelmer Vernooij <jelmer@samba.org>
 # *-* coding: utf-8 *-*
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -17,43 +17,34 @@
 
 """Subversion fetch tests."""
 
+import os, sys
 import shutil
+
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.osutils import has_symlinks
 from bzrlib.repository import Repository
 from bzrlib.revision import NULL_REVISION
-from bzrlib.tests import TestSkipped, KnownFailure
+from bzrlib.tests import (
+    KnownFailure,
+    TestSkipped,
+    )
 from bzrlib.trace import mutter
 
-from bzrlib.plugins.svn import format, remote
+from bzrlib.plugins.svn import (
+    format,
+    remote,
+    )
 from bzrlib.plugins.svn.convert import load_dumpfile
 from bzrlib.plugins.svn.errors import InvalidFileName
-from bzrlib.plugins.svn.mapping3 import set_branching_scheme
-from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.layout.standard import (
+    RootLayout,
+    TrunkLayout,
+    )
+from bzrlib.plugins.svn.tests import SubversionTestCase
 from bzrlib.plugins.svn.transport import SvnRaTransport
 
-import os, sys
-
-class TestFetchWorks(TestCaseWithSubversionRepository):
-    def test_fetch_fileid_renames(self):
-        repos_url = self.make_repository('d')
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("test").modify("data")
-        dc.change_prop("bzr:file-ids", "test\tbla\n")
-        dc.change_prop("bzr:revision-info", "")
-        dc.close()
-
-        oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f", format.get_rich_root_format())
-        newrepos = dir.create_repository()
-        oldrepos.copy_content_into(newrepos)
-        mapping = oldrepos.get_mapping()
-        self.assertEqual("bla", newrepos.get_inventory(
-            oldrepos.generate_revision_id(1, "", mapping)).path2id("test"))
-
+class TestFetchWorks(SubversionTestCase):
     def test_fetch_trunk1(self):
         repos_url = self.make_repository('d')
 
@@ -64,7 +55,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(1))
+        oldrepos.set_layout(TrunkLayout(1))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -78,28 +69,28 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         ch.add_dir("trunk/check/debian")
         ch.add_file("trunk/check/stamp-h.in").modify("foo")
         dc.add_dir("tags")
-        dc.close()
+        dc.close() #1
 
         dc = self.get_commit_editor(repos_url)
         t = dc.open_dir("trunk")
         ch = t.open_dir("trunk/check")
         deb = ch.open_dir("trunk/check/debian")
         deb.add_file("trunk/check/debian/pl").modify("bar")
-        dc.close()
+        dc.close() #2
 
         dc = self.get_commit_editor(repos_url)
         t = dc.open_dir("trunk")
         ch = t.open_dir("trunk/check")
         deb = ch.open_dir("trunk/check/debian")
         deb.add_file("trunk/check/debian/voo").modify("bar")
-        dc.close()
+        dc.close() #3
 
         dc = self.get_commit_editor(repos_url)
         t = dc.open_dir("trunk")
         ch = t.open_dir("trunk/check")
         deb = ch.open_dir("trunk/check/debian")
         deb.add_file("trunk/check/debian/blie").modify("oeh")
-        dc.close()
+        dc.close() #4
 
         dc = self.get_commit_editor(repos_url)
         t = dc.open_dir("trunk")
@@ -107,7 +98,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         deb = ch.open_dir("trunk/check/debian")
         deb.add_file("trunk/check/debian/bar").modify("oeh")
         ch.add_file("trunk/check/bar").modify("bla")
-        dc.close()
+        dc.close() #5
 
         self.make_checkout(repos_url, "dc")
         self.client_copy("dc/trunk", "dc/tags/R_0_9_2", revnum=2)
@@ -120,9 +111,9 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
                          revnum=4)
         self.build_tree({"dc/tags/R_0_9_2/check/debian/blie": "oehha"})
         self.client_update("dc")
-        self.client_commit("dc", "strange revision")
+        self.client_commit("dc", "strange revision") # 6
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(0))
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -138,7 +129,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         self.assertRaises(InvalidFileName, oldrepos.copy_content_into, newrepos)
@@ -146,68 +137,11 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
     def test_fetch_null(self):
         repos_url = self.make_repository('d')
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(1))
+        oldrepos.set_layout(TrunkLayout(1))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos, NULL_REVISION)
 
-    def test_fetch_complex_ids_dirs(self):
-        repos_url = self.make_repository('d')
-
-        dc = self.get_commit_editor(repos_url)
-        dir = dc.add_dir("dir")
-        dir.add_dir("dir/adir")
-        dc.change_prop("bzr:revision-info", "")
-        dc.change_prop("bzr:file-ids", "dir\tbloe\ndir/adir\tbla\n")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_dir("bdir", "dir/adir")
-        dir = dc.open_dir("dir")
-        dir.delete("dir/adir")
-        dc.change_prop("bzr:revision-info", "properties: \n")
-        dc.change_prop("bzr:file-ids", "bdir\tbla\n")
-        dc.close()
-
-        oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f", format.get_rich_root_format())
-        newrepos = dir.create_repository()
-        oldrepos.copy_content_into(newrepos)
-        mapping = oldrepos.get_mapping()
-        tree = newrepos.revision_tree(oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEquals("bloe", tree.path2id("dir"))
-        self.assertIs(None, tree.path2id("dir/adir"))
-        self.assertEquals("bla", tree.path2id("bdir"))
-
-    def test_fetch_complex_ids_files(self):
-        repos_url = self.make_repository('d')
-
-        dc = self.get_commit_editor(repos_url)
-        dir = dc.add_dir("dir")
-        dir.add_file("dir/adir").modify("contents")
-        dc.change_prop("bzr:revision-info", "")
-        dc.change_prop("bzr:file-ids", "dir\tbloe\ndir/adir\tbla\n")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("bdir", "dir/adir")
-        dir = dc.open_dir("dir")
-        dir.delete("dir/adir")
-        dc.change_prop("bzr:revision-info", "properties: \n")
-        dc.change_prop("bzr:file-ids", "bdir\tbla\n")
-        dc.close()
-
-        oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f", format.get_rich_root_format())
-        newrepos = dir.create_repository()
-        oldrepos.copy_content_into(newrepos)
-        mapping = oldrepos.get_mapping()
-        tree = newrepos.revision_tree(oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEquals("bloe", tree.path2id("dir"))
-        self.assertIs(None, tree.path2id("dir/adir"))
-        mutter('entries: %r' % tree.inventory.entries())
-        self.assertEquals("bla", tree.path2id("bdir"))
-
     def test_fetch_copy_remove_old(self):
         repos_url = self.make_client('d', 'dc')
         self.build_tree({'dc/trunk/afile': 'foo', 'dc/tags': None, 
@@ -240,7 +174,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(0))
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -254,12 +188,24 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
 
         self.client_set_revprop(repos_url, 1, "bzr:gpg-signature", "SIGNATURE")
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(0))
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
         self.assertEquals("SIGNATURE", newrepos.get_signature_text(oldrepos.generate_revision_id(1, "trunk", oldrepos.get_mapping())))
 
+    def test_fetch_file_branch(self):
+        repos_url = self.make_repository('d')
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_file("trunk")
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+
     def test_fetch_special_char_edit(self):
         repos_url = self.make_repository('d')
         
@@ -274,7 +220,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(0))
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -288,7 +234,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         raardir.add_file(u'trunk/é/f\x2cle'.encode("utf-8")).modify("data")
         dc.close()
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(0))
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -307,7 +253,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme(0))
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -347,6 +293,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -373,6 +320,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -391,6 +339,38 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         finally:
             newrepos.unlock()
 
+    def test_fetch_move_root(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        br1 = dc.add_dir("branch1")
+        br2 = br1.add_dir("branch1/bloe")
+        br2.add_file("branch1/bloe/README").modify("BLA")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("branch2", "branch1")
+        dc.delete("branch1")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        br1 = dc.open_dir("branch2")
+        bloe = br1.open_dir("branch2/bloe")
+        bloe.open_file("branch2/bloe/README").modify("TEST")
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
+        to_bzrdir = BzrDir.create("f", format.get_rich_root_format())
+        repo = to_bzrdir.create_repository()
+        last_rev = oldrepos.generate_revision_id(3, "branch2", 
+            oldrepos.get_mapping())
+        repo.fetch(oldrepos, last_rev)
+        self.assertEquals(repo.get_inventory(last_rev).root.file_id,
+            oldrepos.get_mapping().generate_file_id(
+                (oldrepos.uuid, "branch1", 1), u""))
+
+
     def test_fetch_replace(self):
         repos_url = self.make_repository('d')
 
@@ -404,6 +384,7 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         dc.close()
 
         oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -438,8 +419,8 @@ class TestFetchWorks(TestCaseWithSubversionRepository):
         tmp.add_dir("branches/tmp/abranch", "trunk/mydir")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -525,6 +506,7 @@ Node-action: delete
 
         load_dumpfile("dumpfile", "old")
         oldrepos = Repository.open("old")
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -666,6 +648,7 @@ Node-copyfrom-path: u
 
         load_dumpfile("dumpfile", "old")
         oldrepos = Repository.open("old")
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -812,6 +795,7 @@ Node-copyfrom-path: bla
 
         load_dumpfile("dumpfile", "old")
         oldrepos = Repository.open("old")
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -992,6 +976,7 @@ Node-copyfrom-path: bla
 
         load_dumpfile("dumpfile", "old")
         oldrepos = Repository.open("old")
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1133,6 +1118,7 @@ Node-copyfrom-path: x
 
         load_dumpfile("dumpfile", "old")
         oldrepos = Repository.open("old")
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1296,6 +1282,7 @@ Node-copyfrom-path: x
 
         load_dumpfile("dumpfile", "old")
         oldrepos = Repository.open("old")
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1310,6 +1297,51 @@ Node-copyfrom-path: x
                 oldrepos.generate_revision_id(5, "", mapping))
         self.assertNotEqual(inv1.path2id("y"), inv2.path2id("y"))
 
+    def test_fetch_dir_replace_self(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("branches")
+        trunk = dc.add_dir("trunk")
+        dir1 = trunk.add_dir("trunk/dir1")
+        dir2 = dir1.add_dir("trunk/dir1/dir2")
+        dir3 = dir2.add_dir("trunk/dir1/dir2/dir3")
+        file = dir3.add_file("trunk/dir1/dir2/dir3/file.txt")
+        file.modify()
+        file = dir1.add_file("trunk/dir1/file.txt")
+        file.modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        dir1 = trunk.open_dir("trunk/dir1")
+        dir1.open_file("trunk/dir1/file.txt").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        dir1 = trunk.open_dir("trunk/dir1")
+        dir2 = dir1.open_dir("trunk/dir1/dir2")
+        dir3 = dir1.open_dir("trunk/dir1/dir2/dir3")
+        dir3.open_file("trunk/dir1/dir2/dir3/file.txt").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.open_dir("branches")
+        branch1 = branches.add_dir("branches/1.0", "trunk", 3)
+        branch1.delete("branches/1.0/dir1")
+        branch1.add_dir("branches/1.0/dir1", "trunk/dir1", 2)
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+        lastrevid = oldrepos.generate_revision_id(4, "branches/1.0", oldrepos.get_mapping())
+        tree = newrepos.revision_tree(lastrevid)
+        self.assertTrue(tree.path2id("dir1/file.txt") is not None)
+
     def test_fetch_dir_upgrade(self):
         repos_url = self.make_repository('d')
 
@@ -1325,40 +1357,19 @@ Node-copyfrom-path: x
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
 
         branch = Branch.open("%s/branches/mybranch" % repos_url)
         mapping = oldrepos.get_mapping()
-        self.assertEqual([oldrepos.generate_revision_id(2, "branches/mybranch", mapping)], 
-                         branch.revision_history())
-
-    def test_fetch_file_from_non_branch(self):
-        repos_url = self.make_repository('d')
-
-        dc = self.get_commit_editor(repos_url)
-        old_trunk = dc.add_dir("old-trunk")
-        lib = old_trunk.add_dir("old-trunk/lib")
-        lib.add_file("old-trunk/lib/file").modify("data")
-        dc.close()
-
-        dc = self.get_commit_editor(repos_url)
-        trunk = dc.add_dir("trunk")
-        lib = trunk.add_dir("trunk/lib")
-        lib.add_file("trunk/lib/file", "old-trunk/lib/file")
-        dc.close()
-
-        oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
-        dir = BzrDir.create("f", format.get_rich_root_format())
-        newrepos = dir.create_repository()
-        oldrepos.copy_content_into(newrepos)
-
-        branch = Branch.open("%s/trunk" % repos_url)
-        self.assertEqual([oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())], 
-                         branch.revision_history())
+        if mapping.is_branch("trunk/lib"):
+            self.assertEqual([oldrepos.generate_revision_id(1, "trunk/lib", mapping), oldrepos.generate_revision_id(2, "branches/mybranch", mapping)], 
+                             branch.revision_history())
+        else:
+            self.assertEqual([oldrepos.generate_revision_id(2, "branches/mybranch", mapping)], 
+                             branch.revision_history())
 
     def test_fetch_dir_from_non_branch(self):
         repos_url = self.make_repository('d')
@@ -1375,7 +1386,7 @@ Node-copyfrom-path: x
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1398,13 +1409,18 @@ Node-copyfrom-path: x
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
 
         branch = Branch.open("%s/trunk" % repos_url)
-        self.assertEqual([oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())],
+        if oldrepos.get_mapping().is_branch("old-trunk"):
+            self.assertEqual([oldrepos.generate_revision_id(1, "old-trunk", oldrepos.get_mapping()), 
+                              oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())],
+                         branch.revision_history())
+        else:
+            self.assertEqual([oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())],
                          branch.revision_history())
 
 
@@ -1424,7 +1440,7 @@ Node-copyfrom-path: x
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1454,7 +1470,7 @@ Node-copyfrom-path: x
         dc.close()
 
         oldrepos = Repository.open(repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1491,15 +1507,15 @@ Node-copyfrom-path: x
         branches.add_dir("branches/foobranch", "trunk")
         dc.close()
 
-        repos = remote.SvnRemoteAccess(SvnRaTransport("svn+"+repos_url), format.SvnRemoteFormat()).find_repository()
-        set_branching_scheme(repos, TrunkBranchingScheme())
+        repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url), format.SvnRemoteFormat()).find_repository()
+        repos.set_layout(TrunkLayout(0))
 
         mapping = repos.get_mapping()
 
         tree = repos.revision_tree(
              repos.generate_revision_id(3, "branches/foobranch", mapping))
 
-        self.assertEqual(mapping.generate_file_id(repos.uuid, 1, "trunk", u""), tree.inventory.root.file_id)
+        self.assertEqual(mapping.generate_file_id((repos.uuid, "trunk", 1), u""), tree.inventory.root.file_id)
 
     def test_fetch_odd(self):
         repos_url = self.make_repository('d')
@@ -1534,7 +1550,7 @@ Node-copyfrom-path: x
         dc.close()
 
         repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url), format.SvnRemoteFormat()).find_repository()
-        set_branching_scheme(repos, TrunkBranchingScheme())
+        repos.set_layout(TrunkLayout(0))
 
         mapping = repos.get_mapping()
 
@@ -1550,7 +1566,8 @@ Node-copyfrom-path: x
         f.change_prop("svn:executable", "*")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir1 = BzrDir.create("f", format.get_rich_root_format())
         dir2 = BzrDir.create("g", format.get_rich_root_format())
         newrepos1 = dir1.create_repository()
@@ -1576,7 +1593,8 @@ Node-copyfrom-path: x
         blie.change_prop("svn:executable", "")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1601,7 +1619,8 @@ Node-copyfrom-path: x
         dc.open_file("bla").modify("data2")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1625,7 +1644,8 @@ Node-copyfrom-path: x
         l.change_prop("svn:special", "*")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1640,7 +1660,7 @@ Node-copyfrom-path: x
     def test_fetch_symlink_with_newlines(self):
         if not has_symlinks():
             return
-        raise KnownFailure("Bazaar doesn't support newlines in symlink targets")
+        raise KnownFailure("Bazaar doesn't support newlines in symlink targets (#219832)")
         repos_url = self.make_repository('d')
 
         dc = self.get_commit_editor(repos_url)
@@ -1650,7 +1670,7 @@ Node-copyfrom-path: x
         l.change_prop("svn:special", "*")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1672,6 +1692,7 @@ Node-copyfrom-path: x
         dc.close()
 
         oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1682,6 +1703,59 @@ Node-copyfrom-path: x
                 oldrepos.generate_revision_id(1, "", mapping))
         self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
 
+    def test_fetch_special_unbecomes_symlink(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        l = dc.add_file("mylink")
+        l.modify("link bla")
+        l.change_prop("svn:special", "*")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        l = dc.open_file("mylink")
+        l.modify("bloebla")
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+        mapping = oldrepos.get_mapping()
+        self.assertTrue(newrepos.has_revision(
+            oldrepos.generate_revision_id(2, "", mapping)))
+        inv1 = newrepos.get_inventory(
+                oldrepos.generate_revision_id(2, "", mapping))
+        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
+
+    def test_fetch_special_becomes_symlink(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        l = dc.add_file("mylink")
+        l.change_prop("svn:special", "*")
+        l.modify("bloebla")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        l = dc.open_file("mylink")
+        l.modify("link bla")
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+        mapping = oldrepos.get_mapping()
+        self.assertTrue(newrepos.has_revision(
+            oldrepos.generate_revision_id(2, "", mapping)))
+        inv1 = newrepos.get_inventory(
+                oldrepos.generate_revision_id(2, "", mapping))
+        raise KnownFailure("not allowing svn:special invalid files to be restored to symlinks yet")
+        self.assertEqual('symlink', inv1[inv1.path2id("mylink")].kind)
+
     def test_fetch_symlink_kind_change(self):
         repos_url = self.make_repository('d')
 
@@ -1693,9 +1767,10 @@ Node-copyfrom-path: x
         dc = self.get_commit_editor(repos_url)
         dc.open_file("mylink").change_prop("svn:special", "*")
         dc.close()
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
+        oldrepos.set_layout(RootLayout())
         oldrepos.copy_content_into(newrepos)
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
@@ -1719,7 +1794,8 @@ Node-copyfrom-path: x
         dc.open_file("bla").change_prop("svn:executable", "*")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1735,23 +1811,6 @@ Node-copyfrom-path: x
         self.assertEqual(oldrepos.generate_revision_id(2, "", mapping), 
                          inv2[inv2.path2id("bla")].revision)
 
-    def test_fetch_ghosts(self):
-        repos_url = self.make_repository('d')
-
-        dc = self.get_commit_editor(repos_url)
-        dc.add_file("bla").modify("data")
-        dc.change_prop("bzr:ancestry:v3-none", "aghost\n")
-        dc.close()
-
-        oldrepos = Repository.open("svn+"+repos_url)
-        dir = BzrDir.create("f", format.get_rich_root_format())
-        newrepos = dir.create_repository()
-        oldrepos.copy_content_into(newrepos)
-        mapping = oldrepos.get_mapping()
-
-        rev = newrepos.get_revision(oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue("aghost" in rev.parent_ids)
-
     def test_fetch_svk_merge(self):
         repos_url = self.make_repository('d')
 
@@ -1766,7 +1825,7 @@ Node-copyfrom-path: x
         foo.open_file("branches/foo/bla").modify("more data")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
 
         dc = self.get_commit_editor(repos_url)
         dc.open_dir("trunk").change_prop("svk:merge", 
@@ -1783,23 +1842,67 @@ Node-copyfrom-path: x
         mutter('parent ids: %r' % rev.parent_ids)
         self.assertTrue(oldrepos.generate_revision_id(2, "branches/foo", mapping) in rev.parent_ids)
 
-    def test_fetch_invalid_ghosts(self):
+    def build_intertwined(self):
         repos_url = self.make_repository('d')
-        
         dc = self.get_commit_editor(repos_url)
-        dc.add_file("bla").modify("data")
-        dc.change_prop("bzr:ancestry:v3-none", "a ghost\n")
+        trunk = dc.add_dir("trunk")
+        trunk.change_prop("bzr:revision-id:v3-trunk0", "1 revid1\n")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.change_prop("bzr:revision-id:v3-trunk0", "1 revid1\n"
+                                                       "2 revid2\n")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.change_prop("bzr:revision-id:v4", "3 revid3\n")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.change_prop("bzr:revision-id:v3-trunk0", "1 revid1\n"
+                                                       "2 revid2\n"
+                                                       "4 revid4\n")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.change_prop("bla", "bloe")
+        dc.close()
+
+        return repos_url
+
+    def test_fetch_intertwined_mappings_old(self):
+        repos_url = self.build_intertwined()
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
-        oldrepos.copy_content_into(newrepos)
-        
-        mapping = oldrepos.get_mapping()
+        oldrepos.copy_content_into(newrepos, "revid4")
+        from bzrlib.plugins.svn.mapping3.base import BzrSvnMappingv3
+        from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
+        revid3 = oldrepos.generate_revision_id(3, "trunk", 
+                BzrSvnMappingv3(TrunkBranchingScheme(0)))
+        self.assertEquals(set(["revid1", "revid2", revid3, "revid4"]), 
+                set(newrepos.all_revision_ids()))
 
-        rev = newrepos.get_revision(oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual([oldrepos.generate_revision_id(0, "", mapping)], rev.parent_ids)
+    def test_fetch_intertwined_mappings_new(self):
+        repos_url = self.build_intertwined()
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        from bzrlib.plugins.svn.mapping4 import BzrSvnMappingv4
+        revid5 = oldrepos.generate_revision_id(5, "trunk", BzrSvnMappingv4())
+        oldrepos.copy_content_into(newrepos, revid5)
+        from bzrlib.plugins.svn.mapping3.base import BzrSvnMappingv3
+        from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
+        revid3 = oldrepos.generate_revision_id(3, "trunk", 
+                BzrSvnMappingv3(TrunkBranchingScheme(0)))
+        self.assertEquals(set(["revid1", "revid2", revid3, "revid4", revid5]), 
+                set(newrepos.all_revision_ids()))
 
     def test_fetch_property_change_only(self):
         repos_url = self.make_repository('d')
@@ -1820,7 +1923,8 @@ Node-copyfrom-path: x
         dc.change_prop("some:property", "some data4\n")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(RootLayout())
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1855,8 +1959,8 @@ Node-copyfrom-path: x
         trunk.change_prop("some:property", "some data3\n")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1868,6 +1972,76 @@ Node-copyfrom-path: x
             oldrepos.generate_revision_id(4, "trunk", mapping),
             ]), set(newrepos.all_revision_ids()))
 
+    def test_fetch_replace_self_open(self):
+        # Not actually a replace, merely just an open. Bug #308353
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        adir = trunk.add_dir("trunk/adir")
+        adir.add_file("trunk/adir/afile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("otherbranch")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.delete("trunk/adir")
+        newdir = trunk.add_dir("trunk/adir", "trunk/adir", 2)
+        newdir.add_file("trunk/adir/foofile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        adir = trunk.open_dir("trunk/adir")
+        adir.open_file("trunk/adir/afile").modify()
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        self.assertEquals({'trunk/adir': (u'R', 'trunk/adir', 2), 'trunk/adir/foofile': (u'A', None, -1)}, 
+                          oldrepos._revmeta_provider.get_revision("trunk", 3).get_paths())
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+
+    def test_fetch_replace_self_open_older(self):
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        adir = trunk.add_dir("trunk/adir")
+        adir.add_file("trunk/adir/afile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.open_dir("trunk/adir").add_dir("trunk/adir/cdir")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.delete("trunk/adir")
+        newdir = trunk.add_dir("trunk/adir", "trunk/adir", 1)
+        newdir.add_file("trunk/adir/foofile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        adir = trunk.open_dir("trunk/adir")
+        adir.open_file("trunk/adir/afile").modify()
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        self.assertEquals({'trunk/adir': (u'R', 'trunk/adir', 1), 'trunk/adir/foofile': (u'A', None, -1)}, 
+                          oldrepos._revmeta_provider.get_revision("trunk", 3).get_paths())
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+
     def test_fetch_crosscopy(self):
         repos_url = self.make_repository('d')
         
@@ -1899,8 +2073,8 @@ Node-copyfrom-path: x
         stationary.add_file("branches/abranch/bdir/stationary/traveller").modify("data")
         dc.close()
 
-        oldrepos = Repository.open("svn+"+repos_url)
-        set_branching_scheme(oldrepos, TrunkBranchingScheme())
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f", format.get_rich_root_format())
         newrepos = dir.create_repository()
         mapping = oldrepos.get_mapping()
