@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,17 +16,18 @@
 
 """Log walker tests."""
 
+import os
+
+from bzrlib import debug
 from bzrlib.errors import NoSuchRevision
 from bzrlib.tests import TestCase
 
-import os
-from bzrlib import debug
-
 from bzrlib.plugins.svn import logwalker
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import SubversionTestCase
 from bzrlib.plugins.svn.transport import SvnRaTransport
 
-class TestLogWalker(TestCaseWithSubversionRepository):
+class TestLogWalker(SubversionTestCase):
+
     def setUp(self):
         super(TestLogWalker, self).setUp()
         debug.debug_flags.add("transport")
@@ -93,8 +94,8 @@ class TestLogWalker(TestCaseWithSubversionRepository):
     def test_get_branch_invalid_revision(self):
         repos_url = self.make_repository("a")
         walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-        self.assertRaises(NoSuchRevision, list, 
-                          walker.iter_changes(["/"], 20))
+        self.assertRaises(NoSuchRevision, lambda: list(
+                          walker.iter_changes(["/"], 20)))
 
     def test_branch_log_all(self):
         repos_url = self.make_repository("a")
@@ -399,8 +400,7 @@ class TestLogWalker(TestCaseWithSubversionRepository):
             self.assertTrue(rev == 0 or paths.has_key("foo"))
             self.assertTrue(rev in (0, 1))
 
-        iter = walker.iter_changes([""], 2)
-        self.assertRaises(NoSuchRevision, list, iter)
+        self.assertRaises(NoSuchRevision, lambda: list(walker.iter_changes([""], 2)))
 
     def test_get_branch_log_follow(self):
         repos_url = self.make_repository("a")
@@ -423,141 +423,6 @@ class TestLogWalker(TestCaseWithSubversionRepository):
                                      'trunk/afile': ('A', None, -1), 
                                      'trunk': (u'A', None, -1)}, 1)], items)
 
-    def test_get_previous_root(self):
-        repos_url = self.make_repository("a")
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual((None, -1), walker.get_previous("", 0))
-
-    def test_get_previous_simple(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        t.add_file("trunk/file").modify()
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.open_dir("trunk")
-        t.add_file("trunk/afile").modify()
-        t.change_prop("myprop", "mydata")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(("trunk", 1), walker.get_previous("trunk", 2))
-
-    def test_get_previous_added(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        t.add_file("trunk/afile").modify()
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.open_dir("trunk")
-        t.open_file("trunk/afile").modify()
-        t.change_prop("myprop", "mydata")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual((None, -1), walker.get_previous("trunk", 1))
-
-    def test_get_previous_copy(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        t.add_file("trunk/afile")
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        cb.add_dir("anotherfile", "trunk")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(("trunk", 1), walker.get_previous("anotherfile", 2))
-
-    def test_get_previous_replace(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        cb.add_dir("trunk")
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        cb.delete("trunk")
-        cb.add_dir("trunk")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual((None, -1), walker.get_previous("trunk", 2))
-
-    def test_find_children_empty(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        cb.add_dir("trunk")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual([], list(walker.find_children("trunk", 1)))
-
-    def test_find_children_one(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        t.add_file("trunk/data")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(['trunk/data'], list(walker.find_children("trunk", 1)))
-
-    def test_find_children_nested(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        td = t.add_dir("trunk/data")
-        td.add_file("trunk/data/bla")
-        t.add_file("trunk/file")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(
-                set(['trunk/data', 'trunk/data/bla', 'trunk/file']), 
-                set(walker.find_children("trunk", 1)))
-
-    def test_find_children_later(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        td = t.add_dir("trunk/data")
-        td.add_file("trunk/data/bla")
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.open_dir("trunk")
-        t.add_file("trunk/file")
-        cb.close()
-        
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(set(['trunk/data', 'trunk/data/bla']), 
-                set(walker.find_children("trunk", 1)))
-        self.assertEqual(set(['trunk/data', 'trunk/data/bla', 'trunk/file']), 
-                set(walker.find_children("trunk", 2)))
-
     def test_revprop_list(self):
         repos_url = self.make_repository("a")
 
@@ -589,63 +454,27 @@ class TestLogWalker(TestCaseWithSubversionRepository):
         props = walker.revprop_list(1)
         self.assertEquals("blaaa", props["foo"])
 
-    def test_find_children_copy(self):
-        repos_url = self.make_repository("a")
+    def test_iter_changes_prefix(self):
+        repos_url = self.make_repository('d')
 
         cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        td = t.add_dir("trunk/data")
-        td.add_file("trunk/data/bla").modify()
-        db = t.add_dir("trunk/db")
-        db.add_file("trunk/db/f1").modify()
-        db.add_file("trunk/db/f2").modify()
+        foo = cb.add_dir("foo")
+        foo.add_dir("foo/trunk")
         cb.close()
 
         cb = self.get_commit_editor(repos_url)
-        t = cb.open_dir("trunk")
-        td = t.open_dir("trunk/data")
-        td.add_dir("trunk/data/fg", "trunk/db")
+        cb.add_dir("bar", "foo")
+        cb.close()
+
+        cb = self.get_commit_editor(repos_url)
+        bar = cb.open_dir("bar")
+        bar.open_dir("bar/trunk").change_prop("some2:property", "some data\n")
         cb.close()
 
         walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(set(['trunk/data', 'trunk/data/bla', 
-                          'trunk/data/fg', 'trunk/data/fg/f1', 
-                          'trunk/data/fg/f2', 'trunk/db',
-                          'trunk/db/f1', 'trunk/db/f2']), 
-                set(walker.find_children("trunk", 2)))
-
-    def test_find_children_copy_del(self):
-        repos_url = self.make_repository("a")
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.add_dir("trunk")
-        td = t.add_dir("trunk/data")
-        td.add_file("trunk/data/bla")
-        db = t.add_dir("trunk/db")
-        db.add_file("trunk/db/f1")
-        db.add_file("trunk/db/f2")
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.open_dir("trunk")
-        td = t.open_dir("trunk/data")
-        td.add_dir("trunk/data/fg", "trunk/db")
-        cb.close()
-
-        cb = self.get_commit_editor(repos_url)
-        t = cb.open_dir("trunk")
-        td = t.open_dir("trunk/data")
-        fg = td.open_dir("trunk/data/fg")
-        fg.delete("trunk/data/fg/f2")
-        cb.close()
-
-        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
-
-        self.assertEqual(set(['trunk/data', 'trunk/data/bla', 
-                          'trunk/data/fg', 'trunk/data/fg/f1', 'trunk/db',
-                          'trunk/db/f1', 'trunk/db/f2']), 
-                set(walker.find_children("trunk", 3)))
+        self.assertEquals([({'bar/trunk': (u'M', None, -1)}, 3), 
+                           ({'bar': (u'A', 'foo', 1)}, 2), 
+                           ({"foo": ('A', None, -1), 'foo/trunk': (u'A', None, -1)}, 1)], [l[:2] for l in walker.iter_changes(["bar"], 3)])
 
     def test_iter_changes_property_change(self):
         repos_url = self.make_repository('d')
@@ -675,6 +504,25 @@ class TestLogWalker(TestCaseWithSubversionRepository):
                            ({'trunk': (u'M', None, -1)}, 2), 
                            ({'trunk/bla': (u'A', None, -1), 'trunk': (u'A', None, -1)}, 1)], [l[:2] for l in walker.iter_changes(["trunk"], 3)])
 
+    def test_iter_changes_pointless(self):
+        repos_url = self.make_repository('d')
+
+        cb = self.get_commit_editor(repos_url)
+        cb.close()
+
+        cb = self.get_commit_editor(repos_url)
+        t = cb.add_dir("trunk")
+        cb.close()
+
+        walker = self.get_log_walker(transport=SvnRaTransport(repos_url))
+        self.assertEquals([({'trunk': (u'A', None, -1)}, 2)],
+                [l[:2] for l in walker.iter_changes(["trunk"], 2)])
+
+        self.assertEquals([({'trunk': (u'A', None, -1)}, 2), 
+                           ({}, 1),
+                           ({'': ('A', None, -1)}, 0) ],
+                [l[:2] for l in walker.iter_changes([""], 2)])
+
 
 class TestCachingLogWalker(TestLogWalker):
     def setUp(self):
@@ -693,7 +541,7 @@ class TestLogCache(TestCase):
         self.cache = logwalker.LogCache()
 
     def test_insert_path(self):
-        self.cache.insert_path(42, "foo", "A", None, -1)
+        self.cache.insert_path(42, u"foo", "A", None, -1)
         self.assertEquals({"foo": ("A", None, -1)}, self.cache.get_revision_paths(42))
 
     def test_insert_revprop(self):
@@ -707,24 +555,22 @@ class TestLogCache(TestCase):
         self.assertFalse(self.cache.has_all_revprops(42))
 
     def test_find_latest_change(self):
-        self.cache.insert_path(42, "foo", "A")
+        self.cache.insert_path(42, u"foo", "A")
         self.assertEquals(42, self.cache.find_latest_change("foo", 42))
         self.assertEquals(42, self.cache.find_latest_change("foo", 45))
 
-    def test_changes_path(self):
-        self.cache.insert_path(42, "foo", "A")
-        self.assertTrue(self.cache.changes_path("foo", 42))
-        self.assertFalse(self.cache.changes_path("foo", 41))
 
-    def test_path_added(self):
-        self.cache.insert_path(42, "foo", "A")
-        self.assertEquals(42, self.cache.path_added("foo", 41, 43))
-        self.assertEquals(None, self.cache.path_added("foo", 42, 43))
-        self.assertEquals(None, self.cache.path_added("foo", 44, 49))
 
-    def test_get_change(self):
-        self.cache.insert_path(41, "foo", "A", "bla", 32)
-        self.cache.insert_path(42, "foo", "A")
-        self.assertEquals(("A", None, -1), self.cache.get_change("foo", 42))
-        self.assertEquals(("A", "bla", 32), self.cache.get_change("foo", 41))
+class DictBasedLogwalkerTestCase(TestCase):
 
+    def test_empty(self):
+        lw = logwalker.DictBasedLogWalker({}, {})
+        self.assertEquals([({'': ('A', None, -1)}, 0, {})], 
+                list(lw.iter_changes([""], 0)))
+
+    def test_simple_root(self):
+        lw = logwalker.DictBasedLogWalker({1:{"/": ('A', None, -1)}}, 
+                {1:{"svn:log": "foo"}})
+        self.assertEquals([({'/': ('A', None, -1)}, 1, {'svn:log': 'foo'}),
+            ({'': ('A', None, -1)}, 0, {})], 
+                list(lw.iter_changes([""], 1)))
