@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,26 +16,55 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
+from subvertpy import ra
+
+from bzrlib import osutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import AlreadyBranchError, BzrError, DivergedBranches
-from bzrlib.merge import Merger, Merge3Merger
-from bzrlib.osutils import has_symlinks
+from bzrlib.errors import (
+    AlreadyBranchError,
+    BzrError,
+    DivergedBranches,
+    )
+from bzrlib.inventory import InventoryDirectory
+from bzrlib.merge import (
+    Merger,
+    Merge3Merger,
+    )
 from bzrlib.progress import DummyProgress
 from bzrlib.repository import Repository
+from bzrlib.revision import Revision
 from bzrlib.trace import mutter
+from bzrlib.tests import TestCase
 
-import os
-
-from bzrlib.plugins.svn import core, format, ra
+from bzrlib.plugins.svn import (
+    format,
+    transport,
+    )
 from bzrlib.plugins.svn.errors import MissingPrefix
-from bzrlib.plugins.svn.commit import push, dpush
-from bzrlib.plugins.svn.mapping import SVN_PROP_BZR_REVISION_ID
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.layout.standard import (
+    RootLayout,
+    TrunkLayout,
+    )
+from bzrlib.plugins.svn.mapping import (
+    SVN_PROP_BZR_REVISION_ID,
+    )
+from bzrlib.plugins.svn.push import (
+    determine_branch_path,
+    dpush, 
+    push,
+    )
+from bzrlib.plugins.svn.tests import (
+    SubversionTestCase,
+    )
+
+
+class TestDPush(SubversionTestCase):
 
-class TestDPush(TestCaseWithSubversionRepository):
     def setUp(self):
         super(TestDPush, self).setUp()
+        transport.disabled_capabilities.update(["commit-revprops", "log-revprops"])
         self.repos_url = self.make_repository('d')
 
         dc = self.commit_editor()
@@ -47,6 +76,10 @@ class TestDPush(TestCaseWithSubversionRepository):
         os.mkdir("dc")
         self.bzrdir = self.svndir.sprout("dc")
 
+
+    def tearDown(self):
+        transport.disabled_capabilities = set()
+
     def commit_editor(self):
         return self.get_commit_editor(self.repos_url)
 
@@ -95,6 +128,22 @@ class TestDPush(TestCaseWithSubversionRepository):
                 "", 
                 r.get_mapping()) for rev in (c.get_latest_revnum()-1, c.get_latest_revnum())]), 
                 set(revid_map.values()))
+
+    def test_dpush_add(self):
+        wt = self.bzrdir.open_workingtree()
+        self.build_tree({'dc/foo/blal': 'other data'})
+        wt.add(["foo/blal"])
+        newid1 = wt.commit(message="Commit from Bzr")
+        self.build_tree({'dc/foo/bliel': 'yet other data'})
+        wt.add(["foo/bliel"])
+        newid2 = wt.commit(message="Commit from Bzr")
+
+        revid_map = dpush(self.svndir.open_branch(), self.bzrdir.open_branch())
+
+        self.assertEquals(set([newid1, newid2]), set(revid_map.keys()))
+        repos = self.svndir.find_repository()
+        revmeta = repos._revmeta_provider.get_revision("", 3)
+        self.assertEquals(revmeta.get_paths(), {"foo/bliel": ('A', None, -1)})
  
     def test_diverged(self):
         dc = self.commit_editor()
@@ -114,7 +163,8 @@ class TestDPush(TestCaseWithSubversionRepository):
                           self.bzrdir.open_branch())
 
 
-class TestPush(TestCaseWithSubversionRepository):
+class TestPush(SubversionTestCase):
+
     def setUp(self):
         super(TestPush, self).setUp()
         self.repos_url = self.make_repository('d')
@@ -235,7 +285,7 @@ class TestPush(TestCaseWithSubversionRepository):
                         self.svndir.open_branch().last_revision())
 
     def test_symlink(self):
-        if not has_symlinks():
+        if not osutils.has_symlinks():
             return
         os.symlink("bla", "dc/south")
         assert os.path.islink("dc/south")
@@ -338,9 +388,8 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        c = ra.RemoteAccess(self.repos_url)
-        self.assertEqual("3 some-rid\n", 
-                c.get_dir("", c.get_latest_revnum())[2][SVN_PROP_BZR_REVISION_ID+"none"])
+        self.assertEquals((3, "some-rid"), 
+                self.svndir.open_branch().last_revision_info())
 
     def test_commit_check_rev_equal(self):
         self.build_tree({'dc/file': 'data'})
@@ -435,26 +484,82 @@ class TestPush(TestCaseWithSubversionRepository):
         wt = self.bzrdir.open_workingtree()
         revid = wt.commit(message="Commit from Bzr")
         b = Branch.open("%s/trunk" % self.repos_url)
-        push(b.repository.get_graph(), b, wt.branch.repository, 
-             wt.branch.revision_history()[-2])
+        push(b.repository.get_graph(), b.repository, b.get_branch_path(),
+                b.get_config(), b.last_revision(), wt.branch.repository, 
+             wt.branch.repository.get_revision(wt.branch.revision_history()[-2]))
         mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0])
-        self.assertEquals('M',
-            self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0]['/trunk'][0])
+        if not b.mapping.can_use_revprops and b.mapping.can_use_fileprops:
+            self.assertEquals('M',
+                self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0]['/trunk'][0])
         b = Branch.open("%s/trunk" % self.repos_url)
-        push(b.repository.get_graph(), b, wt.branch.repository, wt.branch.last_revision())
+        push(b.repository.get_graph(), b.repository, b.get_branch_path(),
+                b.get_config(), b.last_revision(), wt.branch.repository, 
+                wt.branch.repository.get_revision(wt.branch.last_revision()))
         mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0])
         self.assertEquals("/branches/mybranch", 
             self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0]['/trunk'][1])
 
-class PushNewBranchTests(TestCaseWithSubversionRepository):
-    def test_single_revision(self):
-        repos_url = self.make_repository("a")
+    def test_comics(self):
+        dc = self.commit_editor()
+        trunk = dc.add_dir("trunk")
+        comics = trunk.add_dir("trunk/comics")
+        comics.add_dir("trunk/comics/bin")
+        dc.close()
+
+        self.svndir = BzrDir.open("%s/trunk" % self.repos_url)
+        os.mkdir("mybranch")
+        self.bzrdir = self.svndir.sprout("mybranch")
+        wt = self.bzrdir.open_workingtree()
+
+        wt.rename_one("comics", "old-comics")
+        wt.mkdir("comics")
+        wt.rename_one("old-comics", "comics/core")
+        wt.rename_one("comics/core/bin", "comics/bin")
+
+        wt.commit("Strange comics")
+
+        self.svndir.open_branch().pull(wt.branch)
+
+        paths = self.svndir.find_repository()._revmeta_provider.get_revision("", 3).get_paths()
+        self.assertEquals({
+            'trunk/comics': (u'R', None, -1),
+            'trunk/comics/bin': (u'A', 'trunk/comics/bin', 2),
+            'trunk/comics/core': (u'A', 'trunk/comics', 2),
+            'trunk/comics/core/bin': (u'D', None, -1)}, paths)
+
+
+class PushNewBranchTests(SubversionTestCase):
+
+    def _create_single_rev_bzrwt(self):
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/test': "Tour"})
         bzrwt.add("test")
         revid = bzrwt.commit("Do a commit")
+        return bzrwt, revid
+
+    def test_fetch_after_push(self):
+        repos_url = self.make_repository("a")
         newdir = BzrDir.open("%s/trunk" % repos_url)
+        bzrwt, revid = self._create_single_rev_bzrwt()
+        newbranch = newdir.import_branch(bzrwt.branch)
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+        newtree = newrepos.revision_tree(revid)
+
+        bzrwt.lock_read()
+        self.assertEquals(bzrwt.inventory.root.file_id,
+                          newtree.inventory.root.file_id)
+        bzrwt.unlock()
+
+    def test_single_revision(self):
+        repos_url = self.make_repository("a")
+        newdir = BzrDir.open("%s/trunk" % repos_url)
+        bzrwt, revid = self._create_single_rev_bzrwt()
         newbranch = newdir.import_branch(bzrwt.branch)
         newtree = newbranch.repository.revision_tree(revid)
         bzrwt.lock_read()
@@ -464,7 +569,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         self.assertEquals(revid, newbranch.last_revision())
         self.assertEquals([revid], newbranch.revision_history())
 
-    def test_single_revision_single_branching_scheme(self):
+    def test_single_revision_single_branch(self):
         repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
@@ -516,6 +621,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         wt.commit("add still more files", rev_id="mydiver")
 
         repos = Repository.open(repos_url)
+        repos.set_layout(RootLayout())
         wt.branch.repository.fetch(repos)
         mapping = repos.get_mapping()
         other_rev = repos.generate_revision_id(3, "", mapping)
@@ -616,8 +722,8 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         self.assertTrue(os.path.exists("bzrco1/bar2.txt"))
         wt1.branch.push(Branch.open(repos_url+"/trunk"))
         r = Repository.open(repos_url)
-        props = r.branchprop_list.get_changed_properties("trunk", 3)
-        self.assertEquals(props['bzr:text-parents'], 'bar2.txt\tside1\n')
+        revmeta = r._revmeta_provider.get_revision("trunk", 3)
+        self.assertEquals({"bar2.txt": "side1"}, revmeta.get_text_revisions(r.get_mapping()))
 
         os.mkdir("cpy")
         cpy = BzrDir.create("cpy", format.get_rich_root_format())
@@ -695,6 +801,35 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         self.assertEquals(u"Do á commït",
                 Repository.open(repos_url).get_revision(revid1).message)
 
+    def test_kind_change_file_to_directory(self):
+        repos_url = self.make_repository("a")
+        bzrwt = BzrDir.create_standalone_workingtree("c", 
+            format=format.get_rich_root_format())
+        self.build_tree({'c/foo.txt': "foo"})
+        bzrwt.add("foo.txt")
+        revid1 = bzrwt.commit(u"somecommit")
+        os.remove("c/foo.txt")
+        self.build_tree({"c/foo.txt/bar": "contents"})
+        bzrwt.add("foo.txt")
+        revid2 = bzrwt.commit(u"somecommit")
+        newdir = BzrDir.open(repos_url+"/trunk")
+        newdir.import_branch(bzrwt.branch)
+
+    def test_kind_change_directory_to_file(self):
+        repos_url = self.make_repository("a")
+        bzrwt = BzrDir.create_standalone_workingtree("c", 
+            format=format.get_rich_root_format())
+        self.build_tree({'c/foo.txt/bar': "foo"})
+        bzrwt.add("foo.txt")
+        revid1 = bzrwt.commit(u"somecommit")
+        os.remove("c/foo.txt/bar")
+        os.rmdir("c/foo.txt")
+        self.build_tree({"c/foo.txt": "contents"})
+        bzrwt.add("foo.txt")
+        revid2 = bzrwt.commit(u"somecommit")
+        newdir = BzrDir.open(repos_url+"/trunk")
+        newdir.import_branch(bzrwt.branch)
+
     def test_multiple_part_exists(self):
         repos_url = self.make_repository("a")
 
@@ -844,7 +979,72 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         BzrDir.open(repos_url+"/trunk").sprout("n")
         copybranch = Branch.open("n")
         check(copybranch)
-    
+
+    def _create_bzrwt_with_changed_root(self):
+        bzrwt = BzrDir.create_standalone_workingtree("c", 
+            format=format.get_rich_root_format())
+
+        self.build_tree({"c/foo": "bla"})
+        bzrwt.add(["foo"])
+        revid1 = bzrwt.commit("Initial")
+        bzrwt.lock_write()
+        old_ie = bzrwt.inventory.root.copy()
+        new_ie = bzrwt.inventory.root.copy()
+        foo_ie = bzrwt.inventory[bzrwt.path2id("foo")].copy()
+        new_ie.file_id = "mynewroot"
+        foo_ie.parent_id = new_ie.file_id
+        bzrwt.apply_inventory_delta([
+            ("", None, old_ie.file_id, None),
+            (None, "", new_ie.file_id, new_ie),
+            ("foo", "foo", foo_ie.file_id, foo_ie)])
+        bzrwt.unlock()
+        revid2 = bzrwt.commit(message="Commit from Bzr")
+        return bzrwt, old_ie, new_ie, foo_ie, revid1, revid2
+
+    def test_change_root(self):
+        repos_url = self.make_repository("a")
+        bzrwt, old_ie, new_ie, foo_ie, revid1, revid2 = self._create_bzrwt_with_changed_root()
+        newdir = BzrDir.open(repos_url+"/trunk")
+        newbranch = newdir.import_branch(bzrwt.branch)
+        self.assertEquals({"trunk": ("R", None, -1), 
+                           "trunk/foo": ("A", "trunk/foo", 1)},
+            newbranch.repository._revmeta_provider.get_revision("trunk", 2).get_paths())
+        tree1 = newbranch.repository.revision_tree(revid1)
+        tree2 = newbranch.repository.revision_tree(revid2)
+        self.assertEquals(tree1.path2id(""), old_ie.file_id)
+        self.assertEquals(tree2.path2id(""), new_ie.file_id)
+        self.assertEquals(tree1.path2id("foo"), foo_ie.file_id)
+        self.assertEquals(tree2.path2id("foo"), foo_ie.file_id)
+
+        self.assertEquals(bzrwt.branch.revision_history(), 
+                          newbranch.revision_history())
+
+    def test_change_root_fetch(self):
+        repos_url = self.make_repository("a")
+        (bzrwt, old_ie, new_ie, foo_ie, revid1, revid2) = \
+            self._create_bzrwt_with_changed_root()
+        newdir = BzrDir.open(repos_url+"/trunk")
+        newbranch = newdir.import_branch(bzrwt.branch)
+
+        oldrepos = Repository.open(repos_url)
+        oldrepos.set_layout(TrunkLayout(0))
+        dir = BzrDir.create("f", format.get_rich_root_format())
+        newrepos = dir.create_repository()
+        oldrepos.copy_content_into(newrepos)
+
+        log = self.client_log(repos_url, 2, 0)
+        self.assertEquals({'/trunk': ('A', None, -1), 
+                           '/trunk/foo': ('A', None, -1)}, log[1][0])
+        self.assertEquals({'/trunk': ('R', None, -1), 
+                           '/trunk/foo': ('A', '/trunk/foo', 1)}, log[2][0])
+
+        tree1 = newrepos.revision_tree(revid1)
+        tree2 = newrepos.revision_tree(revid2)
+        self.assertEquals(tree1.path2id(""), old_ie.file_id)
+        self.assertEquals(tree2.path2id(""), new_ie.file_id)
+        self.assertEquals(tree1.path2id("foo"), foo_ie.file_id)
+        self.assertEquals(tree2.path2id("foo"), foo_ie.file_id)
+
     def test_rename_dir(self):
         repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
@@ -916,9 +1116,6 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         trunk.pull(bzrwt.branch)
 
         self.assertEquals([revid1, revid2, revid3], trunk.revision_history())
-        self.assertEquals(
-                '1 initialrevid\n2 changerevid\n3 mergerevid\n',
-                self.client_get_prop(repos_url+"/branches/foo", SVN_PROP_BZR_REVISION_ID+"trunk0", r.get_latest_revnum()))
 
     def test_complex_replace_dir(self):
         repos_url = self.make_repository("a")
@@ -943,54 +1140,55 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         os.mkdir("n")
         BzrDir.open(repos_url+"/trunk").sprout("n")
 
-    def test_push_unnecessary_merge(self):        
-        from bzrlib.debug import debug_flags
-        debug_flags.add("commit")
-        debug_flags.add("fetch")
+    def test_subdir_becomes_branch_root(self):
         repos_url = self.make_repository("a")
-        bzrwt = BzrDir.create_standalone_workingtree("c", 
-            format=format.get_rich_root_format())
-        self.build_tree({'c/registry/generic.c': "Tour"})
-        bzrwt.add("registry")
-        bzrwt.add("registry/generic.c")
-        revid1 = bzrwt.commit("Add initial directory + file", 
-                              rev_id="initialrevid")
-
-        # Push first branch into Subversion
-        newdir = BzrDir.open(repos_url+"/trunk")
-        newbranch = newdir.import_branch(bzrwt.branch)
-
-        c = ra.RemoteAccess(repos_url)
-        self.assertTrue(c.check_path("trunk/registry/generic.c", c.get_latest_revnum()) == core.NODE_FILE)
 
         dc = self.get_commit_editor(repos_url)
-        trunk = dc.open_dir("trunk")
-        registry = trunk.open_dir("trunk/registry")
-        registry.open_file("trunk/registry/generic.c").modify("BLA")
+        trunk = dc.add_dir("trunk")
+        subdir = trunk.add_dir("trunk/mysubdir")
+        subdir.add_file("trunk/mysubdir/myfile").modify("blabla")
         dc.close()
-        mapping = newdir.find_repository().get_mapping()
-        merge_revid = newdir.find_repository().generate_revision_id(2, "trunk", mapping)
 
-        # Merge 
-        self.build_tree({'c/registry/generic.c': "DE"})
-        bzrwt.add_pending_merge(merge_revid)
-        self.assertEquals(bzrwt.get_parent_ids()[1], merge_revid)
-        revid2 = bzrwt.commit("Merge something", rev_id="mergerevid")
-        bzr_parents = bzrwt.branch.repository.get_revision(revid2).parent_ids
-        trunk = Branch.open(repos_url + "/trunk")
-        trunk.pull(bzrwt.branch)
+        os.mkdir("dc")
+        svndir = BzrDir.open(repos_url+"/trunk")
+        bzrdir = svndir.sprout("dc")
+        wt = bzrdir.open_workingtree()
+        wt.lock_write()
+        wt.apply_inventory_delta([
+            ("", None, wt.inventory.root.file_id, None),
+            ("mysubdir", "", wt.inventory.path2id("mysubdir"), 
+                InventoryDirectory(wt.inventory.path2id("mysubdir"), "", None))])
+        os.rename("dc/mysubdir/myfile", "dc/myfile")
+        osutils.rmtree("dc/mysubdir")
+        wt.unlock()
+        wt.commit("Change branch root")
+        svnbranch = svndir.open_branch()
+        svnbranch.pull(wt.branch)
+        self.assertEquals(svnbranch.last_revision(), wt.branch.last_revision())
+        self.assertEquals(["myfile"], svndir.root_transport.list_dir("."))
+        paths = svnbranch.repository._revmeta_provider._log.get_revision_paths(2)
+        self.assertEquals(('R', "trunk/mysubdir", 1), paths['trunk'])
 
-        self.assertEquals(tuple(bzr_parents), 
-                trunk.repository.get_revision(revid2).parent_ids)
+    def test_push_pointless(self):
+        repos_url = self.make_repository("a")
 
-        self.assertEquals([revid1, revid2], trunk.revision_history())
-        self.assertEquals(
-                '1 initialrevid\n2 mergerevid\n',
-                self.client_get_prop(repos_url+"/trunk", SVN_PROP_BZR_REVISION_ID+"trunk0",
-                                     c.get_latest_revnum()))
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        dc.close()
 
+        os.mkdir("dc")
+        svndir = BzrDir.open(repos_url+"/trunk")
+        bzrdir = svndir.sprout("dc")
 
-class TestPushTwice(TestCaseWithSubversionRepository):
+        wt = bzrdir.open_workingtree()
+        wt.lock_write()
+        wt.commit("This is pointless.")
+        svnbranch = svndir.open_branch()
+        svnbranch.pull(wt.branch)
+        self.assertEquals(svnbranch.last_revision(), wt.branch.last_revision())
+ 
+
+class TestPushTwice(SubversionTestCase):
     def test_push_twice(self):
         # bug 208566
         repos_url = self.make_repository('d')
@@ -1017,10 +1215,27 @@ class TestPushTwice(TestCaseWithSubversionRepository):
         svndir2.import_branch(wt.branch)
         self.assertEquals(expected_history, svndir2.open_branch().revision_history())
 
-        revid = wt.commit(message="Commit from Bzr")
-        expected_history = wt.branch.revision_history()
-
         svndir1.open_branch().pull(wt.branch)
         self.assertEquals(expected_history, svndir1.open_branch().revision_history())
         svndir2.open_branch().pull(wt.branch)
         self.assertEquals(expected_history, svndir2.open_branch().revision_history())
+
+
+class DetermineBranchPathTests(TestCase):
+
+    def test_no_nick_no_proj(self):
+        rev = Revision("fooid")
+        self.assertEquals("branches/merged",
+            determine_branch_path(rev, TrunkLayout(0)))
+
+    def test_nick_no_proj(self):
+        rev = Revision("fooid")
+        rev.properties['branch-nick'] = 'bla'
+        self.assertEquals("branches/bla",
+            determine_branch_path(rev, TrunkLayout(0)))
+
+    def test_nick_proj(self):
+        rev = Revision("fooid")
+        rev.properties['branch-nick'] = 'bla'
+        self.assertEquals("someproj/branches/bla",
+            determine_branch_path(rev, TrunkLayout(1), "someproj"))
