@@ -1,8 +1,8 @@
-# Copyright (C) 2005-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2005-2009 Jelmer Vernooij <jelmer@samba.org>
  
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -19,11 +19,17 @@ Tests for revision specificiers.
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import BzrError, InvalidRevisionSpec
-from bzrlib.revisionspec import RevisionSpec, RevisionInfo
+from bzrlib.errors import (
+    BzrError,
+    InvalidRevisionSpec,
+    )
+from bzrlib.revisionspec import (
+    RevisionInfo,
+    RevisionSpec,
+    )
 from bzrlib.tests import TestCase
 
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import SubversionTestCase
 
 
 class TestRevSpec(TestCase):
@@ -37,7 +43,7 @@ class TestRevSpec(TestCase):
         self.assertIs(None, RevisionSpec.from_string("svn:foo").get_branch())
 
 
-class TestRevSpecsBySubversion(TestCaseWithSubversionRepository):
+class TestRevSpecsBySubversion(SubversionTestCase):
     def test_by_single_revno(self):
         revspec = RevisionSpec.from_string("svn:2")
         repos_url = self.make_repository("a")
