@@ -1,8 +1,8 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,16 +16,27 @@
 
 """Subversion transport tests."""
 
-from bzrlib.errors import NoSuchFile, FileExists, InvalidURL
-from bzrlib import urlutils
-
-from bzrlib.plugins.svn import core, ra
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
-from bzrlib.plugins.svn.transport import SvnRaTransport, bzr_to_svn_url, _url_unescape_uri
-
+import subvertpy
+from subvertpy import ra
 from unittest import TestCase
 
-class SvnRaTest(TestCaseWithSubversionRepository):
+from bzrlib import urlutils
+from bzrlib.errors import (
+    FileExists,
+    InvalidURL,
+    NoSuchFile,
+    )
+
+from bzrlib.plugins.svn.tests import SubversionTestCase
+from bzrlib.plugins.svn.transport import (
+    SvnRaTransport,
+    bzr_to_svn_url,
+    url_join_unescaped_path,
+    _url_escape_uri,
+    _url_unescape_uri,
+    )
+
+class SvnRaTest(SubversionTestCase):
     def test_open_nonexisting(self):
         self.assertRaises(InvalidURL, SvnRaTransport, 
                           "svn+nonexisting://foo/bar")
@@ -126,10 +137,10 @@ class SvnRaTest(TestCaseWithSubversionRepository):
 
         c = ra.RemoteAccess(repos_url)
         self.assertEquals(c.check_path("bla", c.get_latest_revnum()), 
-                          core.NODE_DIR)
+                          subvertpy.NODE_DIR)
         t.mkdir("bla/subdir")
         self.assertEquals(c.check_path("bla/subdir", c.get_latest_revnum()), 
-                          core.NODE_DIR)
+                          subvertpy.NODE_DIR)
 
     def test_has_dot(self):
         t = SvnRaTransport(self.make_repository('a'))
@@ -145,7 +156,7 @@ class SvnRaTest(TestCaseWithSubversionRepository):
         self.assertRaises(NoSuchFile, t.mkdir, "bla/subdir")
         c = ra.RemoteAccess(repos_url)
         self.assertEquals(c.check_path("bla/subdir", c.get_latest_revnum()), 
-                          core.NODE_NONE)
+                          subvertpy.NODE_NONE)
 
     def test_mkdir_twice(self):
         repos_url = self.make_repository('a')
@@ -198,3 +209,11 @@ class UrlConversionTest(TestCase):
     def test_url_unescape_uri(self):
         self.assertEquals("http://svn.gnome.org/svn/gtk+/trunk",
                 _url_unescape_uri("http://svn.gnome.org/svn/gtk%2B/trunk"))
+
+    def test_url_escape_uri(self):
+        self.assertEquals("http://svn.gnome.org/svn/gtk%2B/trunk",
+                _url_escape_uri("http://svn.gnome.org/svn/gtk+/trunk"))
+
+    def test_url_join_unescaped_path(self):
+        self.assertEquals("http://svn.gnome.org/svn/gtk%2B/trunk",
+                url_join_unescaped_path("http://svn.gnome.org/svn/", "gtk+/trunk"))
