@@ -1,9 +1,9 @@
-# Copyright (C) 2006 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 # -*- coding: utf-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -17,22 +17,37 @@
 
 """Working tree tests."""
 
+import os
+
+from bzrlib import osutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoSuchFile, OutOfDateTree
+from bzrlib.errors import (
+    NoSuchFile,
+    NotBranchError,
+    OutOfDateTree,
+    )
 from bzrlib.inventory import Inventory
-from bzrlib.osutils import has_symlinks, supports_executable
-from bzrlib.tests import KnownFailure, TestCase
+from bzrlib.osutils import (
+    has_symlinks,
+    supports_executable,
+    )
+from bzrlib.repository import Repository
+from bzrlib.tests import (
+    TestCase,
+    TestSkipped,
+    )
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
+from bzrlib.plugins.svn.layout.standard import TrunkLayout
+from bzrlib.plugins.svn.mapping3.base import config_set_scheme
+from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
 from bzrlib.plugins.svn.transport import svn_config
-from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import SubversionTestCase
 from bzrlib.plugins.svn.workingtree import generate_ignore_list
 
-import os
-
-class TestWorkingTree(TestCaseWithSubversionRepository):
+class TestWorkingTree(SubversionTestCase):
     def test_add_duplicate(self):
         self.make_client('a', 'dc')
         self.build_tree({"dc/bl": "data"})
@@ -58,13 +73,31 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
 
     def test_special_char(self):
         self.make_client('a', 'dc')
-        self.build_tree({u"dc/I²C": "data"})
+        try:
+            self.build_tree({u"dc/I²C": "data"})
+        except UnicodeError:
+            raise TestSkipped("This platform does not support unicode paths")
         self.client_add("dc/I²C")
         tree = WorkingTree.open("dc")
         inv = tree.read_working_inventory()
         self.assertIsInstance(inv, Inventory)
         self.assertTrue(inv.has_filename(u"I²C"))
 
+    def test_not_branch_path(self):
+        repos_url = self.make_client('a', 'dc')
+        self.build_tree({"dc/trunk/file": "data"})
+        self.client_add("dc/trunk")
+        self.client_commit("dc", "initial")
+        self.client_update("dc")
+        self.build_tree({"dc/trunk/dir": None})
+        self.client_add("dc/trunk/dir")
+        config_set_scheme(Repository.open(repos_url), TrunkBranchingScheme(0), 
+                          None, True)
+        Repository.open(repos_url).store_layout(TrunkLayout(0))
+        self.assertRaises(NotBranchError, WorkingTree.open, "dc")
+        self.assertRaises(NotBranchError, WorkingTree.open, "dc/trunk/dir")
+        tree = WorkingTree.open("dc/trunk")
+
     def test_smart_add_file(self):
         self.make_client('a', 'dc')
         self.build_tree({"dc/bl": "data"})
@@ -76,6 +109,13 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         self.assertTrue(inv.has_filename("bl"))
         self.assertFalse(inv.has_filename("aa"))
 
+    def test_is_control_filename(self):
+        self.make_client('a', 'dc')
+        bzrdir = BzrDir.open("dc")
+        self.assertTrue(bzrdir.is_control_filename(".svn"))
+        self.assertTrue(bzrdir.is_control_filename(".svn/lock"))
+        self.assertFalse(bzrdir.is_control_filename("lock"))
+
     def test_smart_add_recurse(self):
         self.make_client('a', 'dc')
         self.build_tree({"dc/bl/foo": "data"})
@@ -264,8 +304,8 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         self.client_update("dc")
         tree = WorkingTree.open("dc")
         os.remove("dc/bl")
-        raise KnownFailure("revert not supported yet")
         tree.revert(["bl"])
+        self.assertFalse(tree.changes_from(tree.basis_tree()).has_changed())
         self.assertEqual("data", open('dc/bl').read())
 
     def test_rename_one(self):
@@ -291,7 +331,7 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         self.assertEqual(wt.branch.generate_revision_id(0), 
                          wt.basis_tree().inventory.revision_id)
         inv = Inventory()
-        root_id = wt.branch.repository.get_mapping().generate_file_id(wt.branch.repository.uuid, 0, "", u"")
+        root_id = wt.branch.repository.get_mapping().generate_file_id((wt.branch.repository.uuid, "", 0), u"")
         inv.revision_id = wt.branch.generate_revision_id(0)
         inv.add_path('', 'directory', root_id).revision = inv.revision_id
                               
@@ -351,13 +391,28 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         repos_url = self.make_client('a', 'dc')
 
         dc = self.get_commit_editor(repos_url)
-        dc.add_dir("bar")
+        dc.add_dir("trunk")
         dc.close()
 
-        tree = WorkingTree.open("dc")
-        br = Branch.open(repos_url)
-        tree.pull(br)
+        self.client_update("dc")
+
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.add_dir("branches")
+        foo = branches.add_dir("branches/foo", "trunk")
+        dc.close()
+
+        tree = WorkingTree.open("dc/trunk")
+        old_revid = tree.last_revision()
+        br = Branch.open("%s/branches/foo" % repos_url)
+        result = tree.pull(br)
         self.assertEquals(tree.last_revision(), br.last_revision())
+        self.assertEquals(tree.last_revision(), result.new_revid)
+        self.assertEquals(2, result.new_revno)
+        self.assertEquals(old_revid, result.old_revid)
+        self.assertEquals(1, result.old_revno)
+        self.assertEquals(None, result.master_branch)
+        self.assertEquals(tree.branch, result.target_branch)
+        self.assertEquals(br, result.source_branch)
  
     def test_working_inventory(self):
         self.make_client('a', 'dc')
@@ -424,6 +479,31 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         tree = WorkingTree.open("dc")
         self.assertFalse(tree.inventory.has_filename("bl"))
 
+    def test_fileid_missing(self):
+        self.make_client("repos", "svn-wc")
+        self.build_tree({"svn-wc/parent": None})
+        self.client_add("svn-wc/parent")
+        self.client_commit("svn-wc", "Added parent.")
+
+        self.build_tree({"svn-wc/peer": None})
+        self.client_add("svn-wc/peer")
+        self.client_commit("svn-wc", "Added peer")
+
+        self.build_tree({"svn-wc/peer/file": "A"})
+        self.client_add("svn-wc/peer/file")
+        self.client_commit("svn-wc", "Added file")
+        self.client_update("svn-wc")
+
+        self.build_tree({"svn-wc/peer/file": "B"})
+        self.client_copy("svn-wc/peer", "svn-wc/parent/child")
+        self.client_delete("svn-wc/peer")
+        self.client_commit("svn-wc", "Made peer a child, with mods.")
+        self.client_update("svn-wc")
+
+        WorkingTree.open("svn-wc").update()
+        self.client_update("svn-wc/parent")
+        WorkingTree.open("svn-wc").update()
+
     def test_extras(self):
         self.make_client('a', 'dc')
         self.build_tree({"dc/bl": None})
@@ -465,27 +545,19 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         tree.set_pending_merges([])
         self.assertEqual([lhs_parent_id], tree.get_parent_ids())
 
-    def test_set_pending_merges_prop(self):
-        self.make_client('a', 'dc')
-        self.build_tree({"dc/bl": None})
-        self.client_add("dc/bl")
-        
-        tree = WorkingTree.open("dc")
-        tree.set_pending_merges([
-            tree.branch.mapping.generate_revision_id("a-uuid-foo", 1, "branch/fpath"), "c"])
-        self.assertEqual(
-                "svn-v3-none:a-uuid-foo:branch%2Ffpath:1\tc\n",
-                self.client_get_prop("dc", "bzr:ancestry:v3-none"))
-
     def test_set_pending_merges_svk(self):
         self.make_client('a', 'dc')
-        self.build_tree({"dc/bl": None})
-        self.client_add("dc/bl")
+        self.build_tree({"dc/branches/foo": None})
+        self.client_add("dc/branches")
+        self.client_commit("dc", "add")
+
+        self.build_tree({"dc/trunk/bl": None})
+        self.client_add("dc/trunk")
         
         tree = WorkingTree.open("dc")
         tree.set_pending_merges([
-            tree.branch.mapping.generate_revision_id("a-uuid-foo", 1, "branch/path"), "c"])
-        self.assertEqual("a-uuid-foo:/branch/path:1\n", 
+            tree.branch.repository.generate_revision_id(1, "branches/foo", tree.branch.mapping), "c"])
+        self.assertEqual("%s:/branches/foo:1\n" % tree.branch.repository.uuid,  
                          self.client_get_prop("dc", "svk:merge"))
 
     def test_commit_callback(self):
@@ -533,7 +605,7 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         self.assertEqual(
                 tree.branch.generate_revision_id(1),
                 tree.basis_tree().get_revision_id())
-        delta = tree.basis_tree().changes_from(orig_tree)
+        delta = tree.basis_tree().changes_from(tree.branch.repository.revision_tree(tree.branch.generate_revision_id(0)))
         self.assertTrue(delta.has_changed())
         tree = WorkingTree.open("dc")
         delta = tree.basis_tree().changes_from(tree)
@@ -593,6 +665,8 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         tree.add(["file"], ["fooid"])
         tree.commit("msg")
         tree.rename_one("file", "file2")
+        delta = tree.branch.repository.get_revision_delta(tree.last_revision())
+        self.assertEquals([("file", "fooid", "file")], delta.added)
         self.assertEqual(None, tree.inventory.path2id("file"))
         self.assertEqual("fooid", tree.inventory.path2id("file2"))
         tree = WorkingTree.open("dc")
@@ -660,6 +734,24 @@ class TestWorkingTree(TestCaseWithSubversionRepository):
         self.build_tree({'de/some strange file': 'data-y'})
         self.assertRaises(OutOfDateTree, lambda: tree.commit("bar"))
 
+    def test_unicode_symlink(self):
+        if not has_symlinks():
+            return
+        repos_url = self.make_client('a', 'dc')
+        try:
+            self.build_tree({u"dc/\U00020001".encode(osutils._fs_enc): ""})
+        except UnicodeError:
+            raise TestSkipped("This platform does not support unicode code paths")
+        os.symlink(u"\U00020001", "dc/a")
+        self.build_tree({"dc/b": ""})
+        os.symlink("b", u"dc/\U00020002")
+        self.client_add("dc/a")
+        self.client_add("dc/b")
+        self.client_add(u"dc/\U00020001".encode("utf-8"))
+        self.client_add(u"dc/\U00020002".encode("utf-8"))
+        self.client_commit("dc", "Added files and links")
+        WorkingTree.open("dc").read_working_inventory()
+
 
 class IgnoreListTests(TestCase):
     def test_empty(self):
