@@ -1,9 +1,9 @@
-# Copyright (C) 2006 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2009 Jelmer Vernooij <jelmer@samba.org>
 # -*- coding: utf-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,29 +16,48 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Simple transport for accessing Subversion smart servers."""
 
-import bzrlib
-from bzrlib import debug, urlutils
-from bzrlib.errors import (NoSuchFile, TransportNotPossible, 
-                           FileExists, NotLocalUrl, InvalidURL, RedirectRequested)
-from bzrlib.trace import mutter, warning
-from bzrlib.transport import Transport
-
-import bzrlib.plugins.svn
-from bzrlib.plugins.svn import ra
-from bzrlib.plugins.svn.auth import create_auth_baton
-from bzrlib.plugins.svn.client import get_config
-from bzrlib.plugins.svn.core import SubversionException
-from bzrlib.plugins.svn.errors import convert_svn_error, NoSvnRepositoryPresent, ERR_BAD_URL, ERR_RA_SVN_REPOS_NOT_FOUND, ERR_FS_ALREADY_EXISTS, ERR_FS_NOT_DIRECTORY, ERR_RA_DAV_RELOCATED, ERR_RA_DAV_PATH_NOT_FOUND
+import subvertpy
+from subvertpy import ERR_FS_NOT_DIRECTORY
+from subvertpy.client import get_config
 import urlparse
 import urllib
 
+import bzrlib
+from bzrlib import (
+    debug,
+    ui,
+    urlutils,
+    )
+from bzrlib.errors import (
+    NoSuchFile,
+    TransportNotPossible, 
+    FileExists,
+    NotLocalUrl,
+    InvalidURL,
+    RedirectRequested,
+    )
+from bzrlib.trace import (
+    mutter,
+    warning,
+    )
+from bzrlib.transport import Transport
+
+import bzrlib.plugins.svn
+from bzrlib.plugins.svn.auth import create_auth_baton
+from bzrlib.plugins.svn.errors import (
+    convert_svn_error,
+    NoSvnRepositoryPresent,
+    convert_error,
+    )
+
 svn_config = get_config()
 
-def get_client_string():
-    """Return a string that can be send as part of the User Agent string."""
-    return "bzr%s+bzr-svn%s" % (bzrlib.__version__, bzrlib.plugins.svn.__version__)
 
- 
+# This variable is here to allow tests to temporarily disable features
+# to see how bzr-svn copes with that
+disabled_capabilities = set()
+
+
 # Don't run any tests on SvnTransport as it is not intended to be 
 # a full implementation of Transport
 def get_test_permutations():
@@ -66,6 +85,8 @@ def _url_unescape_uri(url):
     if scheme in ("http", "https"):
         # Without this, URLs with + in them break
         path = urllib.unquote(path)
+    while "//" in path:
+        path = path.replace("//", "/")
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
 
@@ -77,6 +98,10 @@ def _url_escape_uri(url):
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
 
+def url_join_unescaped_path(url, path):
+    return _url_escape_uri(urlutils.join(url, path))
+
+
 svnplus_warning_showed = False
 
 def warn_svnplus(url):
@@ -105,20 +130,52 @@ def bzr_to_svn_url(url):
     return url
 
 
+class SubversionProgressReporter(object):
+
+    def __init__(self):
+        self._last_progress = 0
+        # This variable isn't used yet as of bzr 1.12, and finding 
+        # the right Transport object will be tricky in bzr-svn's case
+        # so just setting it to None for now.
+        self._transport = None
+
+    def update(self, progress, total):
+        # The counter seems to reset sometimes
+        if self._last_progress > progress:
+            assert progress < 100000
+            self._last_progress = 0
+        changed = progress - self._last_progress
+        assert changed >= 0, "changed was %d (%d -> %d)" % (changed, self._last_progress, progress)
+        self._last_progress = progress
+        ui.ui_factory.report_transport_activity(self._transport, changed, None)
+
+
 def Connection(url):
+    # Subvertpy <= 0.6.3 has a bug in the reference counting of the 
+    # progress update function
+    if subvertpy.__version__ >= (0, 6, 3):
+        progress_cb = SubversionProgressReporter().update
+    else:
+        progress_cb = None
+
     try:
         mutter('opening SVN RA connection to %r' % url)
-        ret = ra.RemoteAccess(url.encode('utf8'), 
+        ret = subvertpy.ra.RemoteAccess(url.encode('utf8'), 
                 auth=create_auth_baton(url),
-                client_string_func=get_client_string)
-    except SubversionException, (msg, num):
-        if num in (ERR_RA_SVN_REPOS_NOT_FOUND,):
+                client_string_func=bzrlib.plugins.svn.get_client_string,
+                progress_cb=progress_cb)
+        if 'transport' in debug.debug_flags:
+            ret = MutteringRemoteAccess(ret)
+    except subvertpy.SubversionException, (msg, num):
+        if num in (subvertpy.ERR_RA_SVN_REPOS_NOT_FOUND,):
             raise NoSvnRepositoryPresent(url=url)
-        if num == ERR_BAD_URL:
+        if num == subvertpy.ERR_BAD_URL:
             raise InvalidURL(url)
-        if num == ERR_RA_DAV_PATH_NOT_FOUND:
+        if num in (subvertpy.ERR_RA_DAV_PATH_NOT_FOUND, subvertpy.ERR_FS_NOT_FOUND):
             raise NoSuchFile(url)
-        if num == ERR_RA_DAV_RELOCATED:
+        if num == subvertpy.ERR_RA_ILLEGAL_URL:
+            raise InvalidURL(url, msg)
+        if num == subvertpy.ERR_RA_DAV_RELOCATED:
             # Try to guess the new url
             if "'" in msg:
                 new_url = msg.split("'")[1]
@@ -126,9 +183,10 @@ def Connection(url):
                 new_url = msg[msg.index("»")+2:msg.index("«")]
             else:
                 raise AssertionError("Unable to parse error message: %s" % msg)
-            raise RedirectRequested(source=url, target=new_url, 
+            raise RedirectRequested(source=_url_escape_uri(url),
+                                    target=_url_escape_uri(urlutils.join(url, new_url)), 
                                     is_permanent=True)
-        raise
+        raise convert_error(subvertpy.SubversionException(msg, num))
 
     from bzrlib.plugins.svn import lazy_check_versions
     lazy_check_versions()
@@ -225,7 +283,6 @@ class SvnRaTransport(Transport):
     def get_uuid(self):
         if self._uuid is None:
             conn = self.get_connection()
-            self.mutter('svn get-uuid')
             try:
                 return conn.get_uuid()
             finally:
@@ -241,7 +298,6 @@ class SvnRaTransport(Transport):
 
     def get_svn_repos_root(self):
         if self._repos_root is None:
-            self.mutter('svn get-repos-root')
             conn = self.get_connection()
             try:
                 self._repos_root = conn.get_repos_root()
@@ -251,7 +307,6 @@ class SvnRaTransport(Transport):
 
     def get_latest_revnum(self):
         conn = self.get_connection()
-        self.mutter('svn get-latest-revnum')
         try:
             return conn.get_latest_revnum()
         finally:
@@ -264,8 +319,6 @@ class SvnRaTransport(Transport):
         assert isinstance(limit, int)
         from threading import Thread, Semaphore
 
-        self.mutter('svn iter-log -r%d:%d %r ' % (from_revnum, to_revnum, paths))
-
         class logfetcher(Thread):
             def __init__(self, transport, *args, **kwargs):
                 Thread.__init__(self)
@@ -281,10 +334,7 @@ class SvnRaTransport(Transport):
             def next(self):
                 self.semaphore.acquire()
                 ret = self.pending.pop(0)
-                if ret is None:
-                    self.transport.add_connection(self.conn)
-                elif isinstance(ret, Exception):
-                    self.transport.add_connection(self.conn)
+                if isinstance(ret, Exception):
                     raise ret
                 return ret
 
@@ -296,20 +346,21 @@ class SvnRaTransport(Transport):
                     self.semaphore.release()
                 try:
                     try:
-                        self.conn.get_log(callback=rcvr, *self.args, **self.kwargs)
+                        self.conn.get_log(rcvr, *self.args, **self.kwargs)
                         self.pending.append(None)
                     except Exception, e:
                         self.pending.append(e)
                 finally:
                     self.pending.append(Exception("Some exception was not handled"))
                     self.semaphore.release()
+                    self.transport.add_connection(self.conn)
 
         if paths is None:
             newpaths = None
         else:
             newpaths = [p.rstrip("/") for p in paths]
 
-        fetcher = logfetcher(self, paths=newpaths, start=from_revnum, end=to_revnum, limit=limit, discover_changed_paths=discover_changed_paths, strict_node_history=strict_node_history, include_merged_revisions=include_merged_revisions, revprops=revprops)
+        fetcher = logfetcher(self, newpaths, from_revnum, to_revnum, limit, discover_changed_paths=discover_changed_paths, strict_node_history=strict_node_history, include_merged_revisions=include_merged_revisions, revprops=revprops)
         fetcher.start()
         return iter(fetcher.next, None)
 
@@ -325,8 +376,6 @@ class SvnRaTransport(Transport):
         
         assert paths is None or all_true
 
-        self.mutter('svn log -r%d:%d %r' % (from_revnum, to_revnum, paths))
-
         if paths is None:
             newpaths = None
         else:
@@ -344,31 +393,31 @@ class SvnRaTransport(Transport):
 
     def change_rev_prop(self, revnum, name, value):
         conn = self.get_connection()
-        self.mutter('svn change-revprop -r%d %s=%s' % (revnum, name, value))
         try:
             return conn.change_rev_prop(revnum, name, value)
         finally:
             self.add_connection(conn)
 
-    def get_dir(self, path, revnum, kind=False):
+    def get_dir(self, path, revnum, fields=0):
         conn = self.get_connection()
-        self.mutter('svn get-dir -r%d %s' % (revnum, path))
         try:
-            return conn.get_dir(path, revnum, kind)
+            return conn.get_dir(path, revnum, fields)
         finally:
             self.add_connection(conn)
 
     def get_file(self, path, stream, revnum):
         conn = self.get_connection()
-        self.mutter('svn get-file -r%d %s' % (revnum, path))
         try:
             return conn.get_file(path, stream, revnum)
         finally:
             self.add_connection(conn)
 
-    def mutter(self, text, *args):
-        if 'transport' in debug.debug_flags:
-            mutter(text, *args)
+    def get_file_revs(self, path, start_revnum, end_revnum, handler):
+        conn = self.get_connection()
+        try:
+            return conn.get_file_revs(path, start_revnum, end_revnum, handler)
+        finally:
+            self.add_connection(conn)
 
     def list_dir(self, relpath):
         assert len(relpath) == 0 or relpath[0] != "/"
@@ -376,15 +425,12 @@ class SvnRaTransport(Transport):
             relpath = ""
         try:
             (dirents, _, _) = self.get_dir(relpath, self.get_latest_revnum())
-        except SubversionException, (msg, num):
-            if num == ERR_FS_NOT_DIRECTORY:
-                raise NoSuchFile(relpath)
-            raise
+        except subvertpy.SubversionException, (_, ERR_FS_NOT_DIRECTORY):
+            raise NoSuchFile(relpath)
         return dirents.keys()
 
     def check_path(self, path, revnum):
         conn = self.get_connection()
-        self.mutter('svn check-path -r%d %s' % (revnum, path))
         try:
             return conn.check_path(path, revnum)
         finally:
@@ -393,7 +439,6 @@ class SvnRaTransport(Transport):
     @convert_svn_error
     def mkdir(self, relpath, message="Creating directory"):
         conn = self.get_connection()
-        self.mutter('svn mkdir %s' % (relpath,))
         try:
             ce = conn.get_commit_editor({"svn:log": message})
             try:
@@ -407,33 +452,37 @@ class SvnRaTransport(Transport):
                 for c in reversed(toclose):
                     c.close()
                 ce.close()
-            except SubversionException, (msg, num):
+            except subvertpy.SubversionException, (msg, num):
                 ce.abort()
                 if num == ERR_FS_NOT_DIRECTORY:
                     raise NoSuchFile(msg)
-                if num == ERR_FS_ALREADY_EXISTS:
+                if num == subvertpy.ERR_FS_ALREADY_EXISTS:
                     raise FileExists(msg)
                 raise
         finally:
             self.add_connection(conn)
 
     def has_capability(self, cap):
+        if cap in disabled_capabilities:
+            return False
         if cap in self.capabilities:
             return self.capabilities[cap]
         conn = self.get_connection()
-        self.mutter('svn has-capability %s' % (cap,))
         try:
             try:
                 self.capabilities[cap] = conn.has_capability(cap)
+            except subvertpy.SubversionException, (msg, num):
+                if num != subvertpy.ERR_UNKNOWN_CAPABILITY:
+                    raise
+                self.capabilities[cap] = None
             except NotImplementedError:
-                self.capabilities[cap] = False # Assume the worst
+                self.capabilities[cap] = None # None for unknown
             return self.capabilities[cap]
         finally:
             self.add_connection(conn)
 
     def revprop_list(self, revnum):
         conn = self.get_connection()
-        self.mutter('svn revprop-list -r%d' % (revnum,))
         try:
             return conn.rev_proplist(revnum)
         finally:
@@ -441,7 +490,6 @@ class SvnRaTransport(Transport):
 
     def get_locations(self, path, peg_revnum, revnums):
         conn = self.get_connection()
-        self.mutter('svn get_locations -r%d %s (%r)' % (peg_revnum, path, revnums))
         try:
             return conn.get_locations(path, peg_revnum, revnums)
         finally:
@@ -498,3 +546,99 @@ class SvnRaTransport(Transport):
     def abspath(self, relpath):
         """See Transport.abspath()."""
         return urlutils.join(self.base, relpath)
+
+
+class MutteringRemoteAccess(object):
+
+    busy = property(lambda self: self.actual.busy)
+    url = property(lambda self: self.actual.url)
+
+    def __init__(self, actual):
+        self.actual = actual
+
+    def check_path(self, path, revnum):
+        mutter('svn check-path -r%d %s' % (revnum, path))
+        return self.actual.check_path(path, revnum)
+
+    def has_capability(self, cap):
+        return self.actual.has_capability(cap)
+
+    def get_uuid(self):
+        mutter('svn get-uuid')
+        return self.actual.get_uuid()
+
+    def get_repos_root(self):
+        mutter('svn get-repos-root')
+        return self.actual.get_repos_root()
+
+    def get_latest_revnum(self):
+        mutter('svn get-latest-revnum')
+        return self.actual.get_latest_revnum()
+
+    def get_log(self, callback, paths, from_revnum, to_revnum, limit, *args, **kwargs):
+        mutter('svn log -r%d:%d %r (limit: %r)' % (from_revnum, to_revnum, paths, limit))
+        return self.actual.get_log(callback, paths, 
+                    from_revnum, to_revnum, limit, *args, **kwargs)
+
+    def change_rev_prop(self, revnum, name, value):
+        mutter('svn change-revprop -r%d %s=%s' % (revnum, name, value))
+        return self.actual.change_rev_prop(revnum, name, value)
+
+    def get_dir(self, path, revnum=-1, fields=0):
+        mutter('svn get-dir -r%d %s' % (revnum, path))
+        return self.actual.get_dir(path, revnum, fields)
+
+    def get_file(self, path, stream, revnum):
+        mutter('svn get-file -r%d %s' % (revnum, path))
+        return self.actual.get_file(path, stream, revnum)
+
+    def get_file_revs(self, path, start_revnum, end_revnum, handler):
+        mutter('svn get-file-revs -r%d:%d %s' % (start_revnum, end_revnum, path))
+        return self.actual.get_file_revs(path, start_revnum, end_revnum, 
+            handler)
+
+    def revprop_list(self, revnum):
+        mutter('svn revprop-list -r%d' % (revnum,))
+        return self.actual.revprop_list(revnum)
+
+    def get_locations(self, path, peg_revnum, revnums):
+        mutter('svn get_locations -r%d %s (%r)' % (peg_revnum, path, revnums))
+        return self.actual.get_locations(path, peg_revnum, revnums)
+
+    def do_update(self, revnum, path, start_empty, editor):
+        mutter("svn update -r%d %s" % (revnum, path))
+        return self.actual.do_update(revnum, path, start_empty, editor)
+
+    def do_diff(self, revision_to_update, diff_target, versus_url, 
+                diff_editor, recurse=True, ignore_ancestry=False, text_deltas=False, 
+                depth=None):
+        mutter("svn diff -r%d %s -> %s" % (revision_to_update, diff_target, versus_url))
+        return self.actual.do_diff(revision_to_update, diff_target, versus_url,
+                diff_editor, recurse, ignore_ancestry, text_deltas)
+
+    def do_switch(self, revnum, path, start_empty, to_url, editor):
+        mutter("svn switch -r%d %s -> %s" % (revnum, path, to_url))
+        return self.actual.do_switch(revnum, path, start_empty, to_url, editor)
+
+    def reparent(self, url):
+        mutter("svn reparent %s" % url)
+        return self.actual.reparent(url)
+
+    def get_commit_editor(self, *args, **kwargs):
+        mutter("svn commit")
+        return self.actual.get_commit_editor(*args, **kwargs)
+
+    def rev_proplist(self, revnum):
+        mutter("svn rev-proplist -r%d" % revnum)
+        return self.actual.rev_proplist(revnum)
+
+    def replay_range(self, start_revision, end_revision, low_water_mark, cbs, 
+                     send_deltas=True):
+        mutter("svn replay-range %d -> %d (low water mark: %d)" % (start_revision, end_revision, low_water_mark))
+        return self.actual.replay_range(start_revision, end_revision, low_water_mark, cbs, 
+                   send_deltas)
+
+    def replay(self, revision, low_water_mark, editor, send_deltas=True):
+        mutter("svn replay %d (low water mark: %d)" % (revision, low_water_mark))
+        return self.actual.replay(revision, low_water_mark, editor, send_deltas)
+
