@@ -1,8 +1,8 @@
-# Copyright (C) 2005-2008 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2005-2009 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -15,20 +15,37 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Access to stored Subversion basis trees."""
 
-from bzrlib import osutils, urlutils
-from bzrlib.branch import Branch
-from bzrlib.inventory import Inventory, InventoryDirectory, TreeReference
-from bzrlib.revision import CURRENT_REVISION
-from bzrlib.trace import mutter
-from bzrlib.revisiontree import RevisionTree
-
-import os
-import md5
 from cStringIO import StringIO
+import os
+from subvertpy import (
+    delta,
+    properties,
+    wc,
+    NODE_DIR,
+    )
 import urllib
 
-from bzrlib.plugins.svn import core, properties, wc
-from bzrlib.plugins.svn.delta import apply_txdelta_handler
+from bzrlib import (
+    osutils,
+    urlutils,
+    )
+from bzrlib.branch import Branch
+from bzrlib.inventory import (
+    Inventory,
+    InventoryDirectory,
+    TreeReference,
+    )
+from bzrlib.osutils import md5
+from bzrlib.revision import CURRENT_REVISION
+from bzrlib.revisiontree import RevisionTree
+from bzrlib.trace import mutter
+
+from bzrlib.plugins.svn.fileids import idmap_lookup
+
+class SubversionTree(object):
+
+    def get_file_properties(self, file_id, path=None):
+        raise NotImplementedError(self.get_file_properties)
 
 
 def inventory_add_external(inv, parent_id, path, revid, ref_revnum, url):
@@ -65,32 +82,39 @@ def inventory_add_external(inv, parent_id, path, revid, ref_revnum, url):
     inv.add(ie)
 
 
-class SvnRevisionTree(RevisionTree):
+class SvnRevisionTree(RevisionTree, SubversionTree):
     """A tree that existed in a historical Subversion revision."""
     def __init__(self, repository, revision_id):
         self._repository = repository
         self._revision_id = revision_id
-        (self.branch_path, self.revnum, mapping) = repository.lookup_revision_id(revision_id)
+        self._revmeta, self.mapping = repository._get_revmeta(revision_id)
         self._inventory = Inventory()
         self._inventory.revision_id = revision_id
-        self.id_map = repository.get_fileid_map(self.revnum, self.branch_path, 
-                                                mapping)
+        self.id_map = repository.get_fileid_map(self._revmeta, self.mapping)
         editor = TreeBuildEditor(self)
         self.file_data = {}
+        self.file_properties = {}
         root_repos = repository.transport.get_svn_repos_root()
         conn = repository.transport.get_connection()
         try:
             reporter = conn.do_switch(
-                self.revnum, "", True, 
-                urlutils.join(root_repos, self.branch_path), editor)
-            reporter.set_path("", 0, True, None)
-            reporter.finish()
+                self._revmeta.revnum, "", True, 
+                urlutils.join(root_repos, self._revmeta.branch_path).rstrip("/"), editor)
+            try:
+                reporter.set_path("", 0, True, None)
+                reporter.finish()
+            except:
+                reporter.abort()
+                raise
         finally:
             repository.transport.add_connection(conn)
 
-    def get_file_text(self, file_id):
+    def get_file_text(self, file_id, path=None):
         return self.file_data[file_id]
 
+    def get_file_properties(self, file_id, path=None):
+        return self.file_properties[file_id]
+
 
 class TreeBuildEditor(object):
     """Builds a tree given Subversion tree transform calls."""
@@ -103,7 +127,8 @@ class TreeBuildEditor(object):
         self.revnum = revnum
 
     def open_root(self, revnum):
-        file_id, revision_id = self.tree.id_map[""]
+        file_id, revision_id, _ = idmap_lookup(self.tree.id_map, 
+                                               self.tree.mapping, "")
         ie = self.tree._inventory.add_path("", 'directory', file_id)
         ie.revision = revision_id
         return DirectoryTreeEditor(self.tree, file_id)
@@ -119,15 +144,18 @@ class DirectoryTreeEditor(object):
     def __init__(self, tree, file_id):
         self.tree = tree
         self.file_id = file_id
+        self.tree.file_properties[file_id] = {}
 
     def add_directory(self, path, copyfrom_path=None, copyfrom_revnum=-1):
         path = path.decode("utf-8")
-        file_id, revision_id = self.tree.id_map[path]
+        file_id, revision_id, _ = idmap_lookup(self.tree.id_map, 
+                                               self.tree.mapping, path)
         ie = self.tree._inventory.add_path(path, 'directory', file_id)
         ie.revision = revision_id
         return DirectoryTreeEditor(self.tree, file_id)
 
     def change_prop(self, name, value):
+        self.tree.file_properties[self.file_id][name] = value
         if name in (properties.PROP_ENTRY_COMMITTED_DATE,
                     properties.PROP_ENTRY_LAST_AUTHOR,
                     properties.PROP_ENTRY_LOCK_TOKEN,
@@ -150,6 +178,7 @@ class DirectoryTreeEditor(object):
 
 
 class FileTreeEditor(object):
+
     def __init__(self, tree, path):
         self.tree = tree
         self.path = path
@@ -157,8 +186,12 @@ class FileTreeEditor(object):
         self.is_special = None
         self.is_symlink = False
         self.last_file_rev = None
+        self.file_id, self.revision_id, _ = idmap_lookup(self.tree.id_map, 
+                                               self.tree.mapping, self.path)
+        self.tree.file_properties[self.file_id] = {}
 
     def change_prop(self, name, value):
+        self.tree.file_properties[self.file_id][name] = value
         if name == properties.PROP_EXECUTABLE:
             self.is_executable = (value != None)
         elif name == properties.PROP_SPECIAL:
@@ -179,7 +212,6 @@ class FileTreeEditor(object):
             mutter('unsupported file property %r', name)
 
     def close(self, checksum=None):
-        file_id, revision_id = self.tree.id_map[self.path]
 
         if self.file_stream:
             self.file_stream.seek(0)
@@ -191,14 +223,14 @@ class FileTreeEditor(object):
             self.is_symlink = (self.is_special and file_data.startswith("link "))
 
         if self.is_symlink:
-            ie = self.tree._inventory.add_path(self.path, 'symlink', file_id)
+            ie = self.tree._inventory.add_path(self.path, 'symlink', self.file_id)
         else:
-            ie = self.tree._inventory.add_path(self.path, 'file', file_id)
-        ie.revision = revision_id
+            ie = self.tree._inventory.add_path(self.path, 'file', self.file_id)
+        ie.revision = self.revision_id
 
-        actual_checksum = md5.new(file_data).hexdigest()
-        assert(checksum is None or checksum == actual_checksum,
-                "checksum mismatch: %r != %r" % (checksum, actual_checksum))
+        actual_checksum = md5(file_data).hexdigest()
+        assert checksum is None or checksum == actual_checksum, \
+                "checksum mismatch: %r != %r" % (checksum, actual_checksum)
 
         if self.is_symlink:
             ie.symlink_target = file_data[len("link "):]
@@ -209,25 +241,25 @@ class FileTreeEditor(object):
         else:
             ie.text_sha1 = osutils.sha_string(file_data)
             ie.text_size = len(file_data)
-            self.tree.file_data[file_id] = file_data
+            self.tree.file_data[self.file_id] = file_data
             ie.executable = self.is_executable
 
         self.file_stream = None
 
     def apply_textdelta(self, base_checksum):
         self.file_stream = StringIO()
-        return apply_txdelta_handler("", self.file_stream)
+        return delta.apply_txdelta_handler("", self.file_stream)
 
 
-class SvnBasisTree(RevisionTree):
+class SvnBasisTree(RevisionTree, SubversionTree):
     """Optimized version of SvnRevisionTree."""
+
     def __init__(self, workingtree):
+        mutter("opening basistree for %r at %d; %s" % (workingtree, workingtree.base_revnum, workingtree.base_revid))
         self.workingtree = workingtree
-        self._revision_id = workingtree.branch.generate_revision_id(
-                                      workingtree.base_revnum)
+        self._revision_id = workingtree.base_revid
         self.id_map = workingtree.branch.repository.get_fileid_map(
-                workingtree.base_revnum, 
-                workingtree.branch.get_branch_path(workingtree.base_revnum), 
+                workingtree._get_base_revmeta(),
                 workingtree.branch.mapping)
         self._inventory = Inventory(root_id=None)
         self._repository = workingtree.branch.repository
@@ -260,7 +292,7 @@ class SvnBasisTree(RevisionTree):
             if entry.schedule in (wc.SCHEDULE_NORMAL, 
                                   wc.SCHEDULE_DELETE, 
                                   wc.SCHEDULE_REPLACE):
-                return self.id_map[workingtree.branch.unprefix(relpath.decode("utf-8"))]
+                return idmap_lookup(self.id_map, workingtree.branch.mapping, workingtree.branch.unprefix(relpath.decode("utf-8")))[:2]
             return (None, None)
 
         def add_dir_to_inv(relpath, adm, parent_id):
@@ -276,7 +308,7 @@ class SvnBasisTree(RevisionTree):
             if relpath == u"":
                 self._inventory.revision_id = revid
 
-            for name, entry in entries.items():
+            for name, entry in entries.iteritems():
                 name = name.decode("utf-8")
                 if name == u"":
                     continue
@@ -288,9 +320,8 @@ class SvnBasisTree(RevisionTree):
 
                 assert entry
                 
-                if entry.kind == core.NODE_DIR:
-                    subwc = wc.WorkingCopy(adm, 
-                            self.workingtree.abspath(subrelpath).encode("utf-8"))
+                if entry.kind == NODE_DIR:
+                    subwc = self.workingtree._get_wc(subrelpath)
                     try:
                         add_dir_to_inv(subrelpath, subwc, id)
                     finally:
@@ -312,9 +343,25 @@ class SvnBasisTree(RevisionTree):
     def get_file_byname(self, name):
         return open(self.abspath(name))
 
-    def get_file_text(self, file_id):
-        return self.get_file_byname(self.id2path(file_id)).read()
+    def get_file_text(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        return self.get_file_byname(path).read()
 
-    def annotate_iter(self, file_id,
-                      default_revision=CURRENT_REVISION):
-        raise NotImplementedError(self.annotate_iter)
+    def get_file_properties(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        wc = self.workingtree._get_wc(path)
+        try:
+            (_, orig_props) = wc.get_prop_diffs(self.workingtree.abspath(path).encode("utf-8"))
+        finally:
+            wc.close()
+        return orig_props
+
+    def annotate_iter(self, file_id, default_revision=CURRENT_REVISION):
+        return self.workingtree.branch.repository._annotate(urlutils.join(self.workingtree.branch.get_branch_path(), self.inventory.id2path(file_id)).strip("/"), self.workingtree.base_revnum,  file_id, self.get_revision_id(), self.workingtree.branch.mapping)
+
+    def iter_files_bytes(self, file_ids):
+        for file_id, identifier in file_ids:
+            cur_file = (self.get_file_text(file_id),)
+            yield identifier, cur_file
